@@ -18,7 +18,6 @@
 */
 
 #include <algorithm>
-#include <pthread.h>
 #include <sstream>
 #include <stdlib.h>
 #include <stdio.h>
@@ -30,6 +29,7 @@
 #include "players.h"
 
 extern "C" {
+#include <audacious/input.h>
 #include <audacious/misc.h>
 #include <audacious/i18n.h>
 #include <libaudcore/audstrings.h>
@@ -58,8 +58,6 @@ extern "C" {
 /***** Global variables *****/
 
 static bool_t audio_error = FALSE;
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static bool_t stop_flag;
 
 // Configuration (and defaults)
 static struct
@@ -77,11 +75,8 @@ static struct
   CPlayer *p;
   CAdPlugDatabase *db;
   unsigned int subsong, songlength;
-  int seek;
   char * filename;
-} plr = {0, 0, 0, 0, -1, NULL};
-
-static InputPlayback *playback;
+} plr = {0, 0, 0, 0, NULL};
 
 /***** Debugging *****/
 
@@ -114,11 +109,6 @@ factory (VFSFile * fd, Copl * newopl)
   return CAdPlug::factory (fd, newopl, conf.players);
 }
 
-extern "C" {
-void adplug_stop(InputPlayback * data);
-bool_t adplug_play(InputPlayback * data, const char * filename, VFSFile * file, int start_time, int stop_time, bool_t pause);
-}
-
 /***** Main player (!! threaded !!) *****/
 
 extern "C" Tuple * adplug_get_tuple (const char * filename, VFSFile * fd)
@@ -136,16 +126,16 @@ extern "C" Tuple * adplug_get_tuple (const char * filename, VFSFile * fd)
     ti = tuple_new_from_filename (filename);
 
     if (! p->getauthor().empty())
-      tuple_set_str(ti, FIELD_ARTIST, NULL, p->getauthor().c_str());
+      tuple_set_str(ti, FIELD_ARTIST, p->getauthor().c_str());
 
     if (! p->gettitle().empty())
-      tuple_set_str(ti, FIELD_TITLE, NULL, p->gettitle().c_str());
+      tuple_set_str(ti, FIELD_TITLE, p->gettitle().c_str());
     else if (! p->getdesc().empty())
-      tuple_set_str(ti, FIELD_TITLE, NULL, p->getdesc().c_str());
+      tuple_set_str(ti, FIELD_TITLE, p->getdesc().c_str());
 
-    tuple_set_str(ti, FIELD_CODEC, NULL, p->gettype().c_str());
-    tuple_set_str(ti, FIELD_QUALITY, NULL, _("sequenced"));
-    tuple_set_int(ti, FIELD_LENGTH, NULL, p->songlength (plr.subsong));
+    tuple_set_str(ti, FIELD_CODEC, p->gettype().c_str());
+    tuple_set_str(ti, FIELD_QUALITY, _("sequenced"));
+    tuple_set_int(ti, FIELD_LENGTH, p->songlength (plr.subsong));
     delete p;
   }
 
@@ -155,8 +145,7 @@ extern "C" Tuple * adplug_get_tuple (const char * filename, VFSFile * fd)
 // Define sampsize macro (only usable inside play_loop()!)
 #define sampsize ((bit16 ? 2 : 1) * (stereo ? 2 : 1))
 
-static bool_t play_loop (InputPlayback * playback, const char * filename,
- VFSFile * fd)
+static bool_t play_loop (const char * filename, VFSFile * fd)
 /* Main playback thread. Takes the filename to play as argument. */
 {
   dbg_printf ("play_loop(\"%s\"): ", filename);
@@ -195,53 +184,38 @@ static bool_t play_loop (InputPlayback * playback, const char * filename,
 
   // Set XMMS main window information
   dbg_printf ("xmms, ");
-  playback->set_params (playback, freq * sampsize * 8, freq, stereo ? 2 : 1);
+  aud_input_set_bitrate (freq * sampsize * 8);
 
   // Rewind player to right subsong
   dbg_printf ("rewind, ");
   plr.p->rewind (plr.subsong);
 
-  pthread_mutex_lock (& mutex);
-  plr.seek = -1;
-  stop_flag = FALSE;
-  playback->set_pb_ready (playback);
-  pthread_mutex_unlock (& mutex);
-
   // main playback loop
   dbg_printf ("loop.\n");
   while ((playing || conf.endless))
   {
-    pthread_mutex_lock (& mutex);
+    if (aud_input_check_stop ())
+      break;
 
-    if (stop_flag)
-    {
-        pthread_mutex_unlock (& mutex);
-        break;
-    }
+    int seek = aud_input_check_seek ();
 
     // seek requested ?
-    if (plr.seek != -1)
+    if (seek != -1)
     {
-      int time = playback->output->written_time ();
+      int time = aud_input_written_time ();
 
       // backward seek ?
-      if (plr.seek < time)
+      if (seek < time)
       {
         plr.p->rewind (plr.subsong);
         time = 0;
       }
 
       // seek to requested position
-      while (time < plr.seek && plr.p->update ())
+      while (time < seek && plr.p->update ())
         time += (int) (1000 / plr.p->getrefresh ());
-
-      // Reset output plugin and some values
-      playback->output->flush (time);
-      plr.seek = -1;
     }
 
-    pthread_mutex_unlock (& mutex);
-
     // fill sound buffer
     towrite = SNDBUFSIZE;
     sndbufpos = sndbuf;
@@ -259,13 +233,9 @@ static bool_t play_loop (InputPlayback * playback, const char * filename,
       toadd -= (long) (plr.p->getrefresh () * i);
     }
 
-    playback->output->write_audio (sndbuf, SNDBUFSIZE * sampsize);
+    aud_input_write_audio (sndbuf, SNDBUFSIZE * sampsize);
   }
 
-  pthread_mutex_lock (& mutex);
-  stop_flag = FALSE;
-  pthread_mutex_unlock (& mutex);
-
   // free everything and exit
   dbg_printf ("free");
   delete plr.p;
@@ -303,62 +273,23 @@ adplug_is_our_fd (const char * filename, VFSFile * fd)
 /***** Player control *****/
 
 extern "C" bool_t
-adplug_play (InputPlayback * data, const char * filename, VFSFile * file, int start_time, int stop_time, bool_t pause)
+adplug_play (const char * filename, VFSFile * file)
 {
-  playback = data;
   dbg_printf ("adplug_play(\"%s\"): ", filename);
   audio_error = FALSE;
 
   // open output plugin
   dbg_printf ("open, ");
-  if (!playback->output->
-      open_audio (conf.bit16 ? FORMAT_16 : FORMAT_8, conf.freq,
-                  conf.stereo ? 2 : 1))
+  if (!aud_input_open_audio (conf.bit16 ? FORMAT_16 : FORMAT_8, conf.freq, conf.stereo ? 2 : 1))
   {
     audio_error = TRUE;
     return TRUE;
   }
 
-  play_loop (playback, filename, file);
+  play_loop (filename, file);
   return FALSE;
 }
 
-extern "C" void adplug_stop (InputPlayback * p)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (! stop_flag)
-    {
-        stop_flag = TRUE;
-        p->output->abort_write ();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
-extern "C" void adplug_pause (InputPlayback * p, bool_t pause)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (! stop_flag)
-        p->output->pause (pause);
-
-    pthread_mutex_unlock (& mutex);
-}
-
-extern "C" void adplug_mseek (InputPlayback * p, int time)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (! stop_flag)
-    {
-        plr.seek = time;
-        p->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
 /***** Configuration file handling *****/
 
 #define CFG_VERSION "AdPlug"
@@ -382,20 +313,20 @@ extern "C" bool_t adplug_init (void)
   // Read file type exclusion list
   dbg_printf ("exclusion, ");
   {
-    char * cfgstr = aud_get_string (CFG_VERSION, "Exclude");
+    char * cfgstr = aud_get_str (CFG_VERSION, "Exclude");
 
     if (cfgstr[0])
     {
         char exclude[strlen (cfgstr) + 2];
         memcpy (exclude, cfgstr, sizeof exclude - 1);
         exclude[sizeof exclude - 1] = 0;
-        string_replace_char (exclude, ':', 0);
+        str_replace_char (exclude, ':', 0);
 
         for (char * p = exclude; * p; p += strlen (p) + 1)
             conf.players.remove (conf.players.lookup_filetype (p));
     }
 
-    free (cfgstr);
+    str_unref (cfgstr);
   }
 
   // Load database from disk and hand it to AdPlug
@@ -407,9 +338,8 @@ extern "C" bool_t adplug_init (void)
 
     if (homedir)
     {
-      char * homeuri = filename_to_uri (homedir);
-      std::string userdb = std::string (homeuri) + "/" ADPLUG_CONFDIR "/" + ADPLUGDB_FILE;
-      free (homeuri);
+      std::string userdb;
+      userdb = std::string ("file://") + homedir + "/" ADPLUG_CONFDIR "/" + ADPLUGDB_FILE;
 
       if (vfs_file_test (userdb.c_str (), VFS_EXISTS))
       {
@@ -452,5 +382,5 @@ adplug_quit (void)
       exclude += (*i)->filetype;
     }
 
-  aud_set_string (CFG_VERSION, "Exclude", exclude.c_str ());
+  aud_set_str (CFG_VERSION, "Exclude", exclude.c_str ());
 }
