@@ -1,3 +1,8 @@
+#ifndef __INTERFACE_H
+#define __INTERFACE_H
+
 GtkWidget* create_alarm_dialog (void);
 GtkWidget* create_config_dialog (void);
 GtkWidget* create_reminder_dialog (const gchar *reminder_msg);
+
+#endif
