@@ -22,6 +22,7 @@
 
 #include <stdio.h>
 #include <audacious/misc.h>
+#include <libaudcore/audstrings.h>
 
 #define ERROR(...) fprintf (stderr, "alsa: " __VA_ARGS__)
 
@@ -78,6 +79,6 @@ extern int alsa_config_drop_workaround, alsa_config_drain_workaround,
 
 void alsa_config_load (void);
 void alsa_config_save (void);
-void alsa_configure (void);
+void * alsa_create_config_widget (void);
 
 #endif
