@@ -22,17 +22,15 @@
 
 #include <audacious/i18n.h>
 #include <audacious/misc.h>
-#include <libaudgui/libaudgui.h>
-#include <libaudgui/libaudgui-gtk.h>
 
 #include "alsa.h"
+
 char * alsa_config_pcm = NULL, * alsa_config_mixer = NULL,
  * alsa_config_mixer_element = NULL;
 int alsa_config_drain_workaround = 1;
 
 static GtkListStore * pcm_list, * mixer_list, * mixer_element_list;
-static GtkWidget * window, * pcm_combo, * mixer_combo, * mixer_element_combo,
- * drain_workaround_check;
+static GtkWidget * pcm_combo, * mixer_combo, * mixer_element_combo, * drain_workaround_check;
 
 static GtkTreeIter * list_lookup_member (GtkListStore * list, const char * text)
 {
@@ -49,11 +47,11 @@ static GtkTreeIter * list_lookup_member (GtkListStore * list, const char * text)
 
         if (! strcmp (iter_text, text))
         {
-            free (iter_text);
+            g_free (iter_text);
             return & iter;
         }
 
-        free (iter_text);
+        g_free (iter_text);
     }
     while (gtk_tree_model_iter_next ((GtkTreeModel *) list, & iter));
 
@@ -83,11 +81,11 @@ static void get_defined_devices (const char * type, int capture, void (* found)
             char * description = snd_device_name_get_hint (hints[count], "DESC");
 
             found (name, description);
-            free (name);
-            free (description);
+            g_free (name);
+            g_free (description);
         }
 
-        free (type);
+        g_free (type);
     }
 
 FAILED:
@@ -121,7 +119,7 @@ static void get_cards (void (* found) (int card, const char * description))
         if ((description = get_card_description (card)) != NULL)
         {
             found (card, description);
-            free (description);
+            g_free (description);
         }
     }
 
@@ -142,7 +140,7 @@ static char * get_device_description (snd_ctl_t * control, int device, int
     switch (snd_ctl_pcm_info (control, info))
     {
     case 0:
-        return strdup (snd_pcm_info_get_name (info));
+        return g_strdup (snd_pcm_info_get_name (info));
 
     case -ENOENT: /* capture but we want playback (or other way around) */
         return NULL;
@@ -180,7 +178,7 @@ static void get_devices (int card, int capture, void (* found) (const char *
         if (description != NULL)
             found (name, description);
 
-        free (description);
+        g_free (description);
     }
 
 FAILED:
@@ -297,12 +295,12 @@ static void guess_mixer_element (void)
     mixer_element_list_fill ();
 
     static const char * guesses[] = {"Master", "PCM", "Wave"};
-    for (int count = 0; count < G_N_ELEMENTS (guesses); count ++)
+    for (int count = 0; count < ARRAY_LEN (guesses); count ++)
     {
         if (list_has_member (mixer_element_list, guesses[count]))
         {
-            free (alsa_config_mixer_element);
-            alsa_config_mixer_element = strdup (guesses[count]);
+            str_unref (alsa_config_mixer_element);
+            alsa_config_mixer_element = str_get (guesses[count]);
             return;
         }
     }
@@ -320,9 +318,9 @@ void alsa_config_load (void)
 {
     aud_config_set_defaults ("alsa", alsa_defaults);
 
-    alsa_config_pcm = aud_get_string ("alsa", "pcm");
-    alsa_config_mixer = aud_get_string ("alsa", "mixer");
-    alsa_config_mixer_element = aud_get_string ("alsa", "mixer-element");
+    alsa_config_pcm = aud_get_str ("alsa", "pcm");
+    alsa_config_mixer = aud_get_str ("alsa", "mixer");
+    alsa_config_mixer_element = aud_get_str ("alsa", "mixer-element");
     alsa_config_drain_workaround = aud_get_bool ("alsa", "drain-workaround");
 
     if (! alsa_config_mixer_element[0])
@@ -349,16 +347,16 @@ void alsa_config_save (void)
         mixer_element_list = NULL;
     }
 
-    aud_set_string ("alsa", "pcm", alsa_config_pcm);
-    aud_set_string ("alsa", "mixer", alsa_config_mixer);
-    aud_set_string ("alsa", "mixer-element", alsa_config_mixer_element);
+    aud_set_str ("alsa", "pcm", alsa_config_pcm);
+    aud_set_str ("alsa", "mixer", alsa_config_mixer);
+    aud_set_str ("alsa", "mixer-element", alsa_config_mixer_element);
     aud_set_bool ("alsa", "drain-workaround", alsa_config_drain_workaround);
 
-    free (alsa_config_pcm);
+    str_unref (alsa_config_pcm);
     alsa_config_pcm = NULL;
-    free (alsa_config_mixer);
+    str_unref (alsa_config_mixer);
     alsa_config_mixer = NULL;
-    free (alsa_config_mixer_element);
+    str_unref (alsa_config_mixer_element);
     alsa_config_mixer_element = NULL;
 }
 
@@ -423,13 +421,9 @@ static const char * combo_selected_text (GtkWidget * combo, GtkListStore * list)
     return text;
 }
 
-static void create_window (void)
+static GtkWidget * create_vbox (void)
 {
-    window = gtk_dialog_new_with_buttons (_("ALSA Output Plugin Preferences"),
-     NULL, 0, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
-    gtk_window_set_resizable ((GtkWindow *) window, 0);
-
-    GtkWidget * vbox = gtk_dialog_get_content_area ((GtkDialog *) window);
+    GtkWidget * vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 
     gtk_box_pack_start ((GtkBox *) vbox, combo_new (_("PCM device:"), pcm_list,
      & pcm_combo, 1), 0, 0, 0);
@@ -444,7 +438,7 @@ static void create_window (void)
      alsa_config_drain_workaround);
     gtk_box_pack_start ((GtkBox *) vbox, drain_workaround_check, 0, 0, 0);
 
-    gtk_widget_show_all (window);
+    return vbox;
 }
 
 static void pcm_changed (GtkComboBox * combo, void * unused)
@@ -454,8 +448,8 @@ static void pcm_changed (GtkComboBox * combo, void * unused)
     if (new == NULL || ! strcmp (new, alsa_config_pcm))
         return;
 
-    free (alsa_config_pcm);
-    alsa_config_pcm = strdup (combo_selected_text (pcm_combo, pcm_list));
+    str_unref (alsa_config_pcm);
+    alsa_config_pcm = str_get (combo_selected_text (pcm_combo, pcm_list));
 
     aud_output_reset (OUTPUT_RESET_SOFT);
 }
@@ -467,8 +461,8 @@ static void mixer_changed (GtkComboBox * combo, void * unused)
     if (new == NULL || ! strcmp (new, alsa_config_mixer))
         return;
 
-    free (alsa_config_mixer);
-    alsa_config_mixer = strdup (new);
+    str_unref (alsa_config_mixer);
+    alsa_config_mixer = str_get (new);
 
     mixer_element_list_refill ();
     guess_mixer_element ();
@@ -488,8 +482,8 @@ static void mixer_element_changed (GtkComboBox * combo, void * unused)
      alsa_config_mixer_element)))
         return;
 
-    free (alsa_config_mixer_element);
-    alsa_config_mixer_element = strdup (new);
+    str_unref (alsa_config_mixer_element);
+    alsa_config_mixer_element = str_get (new);
 
     alsa_close_mixer ();
     alsa_open_mixer ();
@@ -510,27 +504,15 @@ static void connect_callbacks (void)
      mixer_element_changed, NULL);
     g_signal_connect ((GObject *) drain_workaround_check, "toggled", (GCallback)
      boolean_toggled, & alsa_config_drain_workaround);
-    g_signal_connect ((GObject *) window, "response", (GCallback)
-     gtk_widget_destroy, window);
-    g_signal_connect ((GObject *) window, "destroy", (GCallback)
-     gtk_widget_destroyed, & window);
-
-    audgui_destroy_on_escape (window);
 }
 
-void alsa_configure (void)
+void * alsa_create_config_widget (void)
 {
-    if (window != NULL)
-    {
-        gtk_window_present ((GtkWindow *) window);
-        return;
-    }
-
     pcm_list_fill ();
     mixer_list_fill ();
     mixer_element_list_fill ();
 
-    create_window ();
+    GtkWidget * vbox = create_vbox ();
 
     combo_select_by_text (pcm_combo, pcm_list, alsa_config_pcm);
     combo_select_by_text (mixer_combo, mixer_list, alsa_config_mixer);
@@ -538,4 +520,6 @@ void alsa_configure (void)
      alsa_config_mixer_element);
 
     connect_callbacks ();
+
+    return vbox;
 }
