@@ -23,7 +23,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
+
 #include <audacious/i18n.h>
+#include <audacious/misc.h>
 
 #include "i_configure.h"
 /* this is needed to retrieve information */
@@ -35,7 +37,7 @@
 void i_fileinfo_ev_destroy (GtkWidget * win, void * mf)
 {
     i_midi_free ((midifile_t *) mf);
-    free (mf);
+    g_free (mf);
 }
 
 
@@ -133,7 +135,7 @@ void i_fileinfo_gui (const char * filename_uri)
     if (fileinfowin)
         return;
 
-    mf = malloc (sizeof (midifile_t));
+    mf = g_new (midifile_t, 1);
 
     /****************** midifile parser ******************/
     if (!i_midi_parse_from_filename (filename_uri, mf))
@@ -146,7 +148,6 @@ void i_fileinfo_gui (const char * filename_uri)
 
     fileinfowin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_type_hint (GTK_WINDOW (fileinfowin), GDK_WINDOW_TYPE_HINT_DIALOG);
-    gtk_window_set_position (GTK_WINDOW (fileinfowin), GTK_WIN_POS_CENTER);
     g_signal_connect (G_OBJECT (fileinfowin), "destroy", G_CALLBACK (i_fileinfo_ev_destroy), mf);
     g_signal_connect (G_OBJECT (fileinfowin), "destroy", G_CALLBACK (gtk_widget_destroyed), &fileinfowin);
     gtk_container_set_border_width (GTK_CONTAINER (fileinfowin), 10);
@@ -188,10 +189,12 @@ void i_fileinfo_gui (const char * filename_uri)
      *** MIDI INFO BOX ***/
     midiinfoboxes_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
 
+    int comments_extract = aud_get_int ("amidiplug", "ap_opts_comments_extract");
+    int lyrics_extract = aud_get_int ("amidiplug", "ap_opts_lyrics_extract");
+
     /* pick the entire space if both comments and lyrics boxes are not displayed,
        pick only required space if at least one of them is displayed */
-    if ((amidiplug_cfg_ap->ap_opts_comments_extract == 0) &&
-            (amidiplug_cfg_ap->ap_opts_lyrics_extract == 0))
+    if (! comments_extract && ! lyrics_extract)
         gtk_box_pack_start (GTK_BOX (fileinfowin_columns_hbox), midiinfoboxes_vbox, TRUE, TRUE, 0);
     else
         gtk_box_pack_start (GTK_BOX (fileinfowin_columns_hbox), midiinfoboxes_vbox, FALSE, FALSE, 0);
@@ -287,12 +290,10 @@ void i_fileinfo_gui (const char * filename_uri)
     lyrics_tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW (lyrics_tv));
 
     /* call the buffer fill routine if at least one between comments and lyrics is enabled */
-    if ((amidiplug_cfg_ap->ap_opts_comments_extract > 0) ||
-            (amidiplug_cfg_ap->ap_opts_lyrics_extract > 0))
+    if (comments_extract || lyrics_extract)
         i_fileinfo_text_fill (mf, text_tb, lyrics_tb);
 
-    if ((amidiplug_cfg_ap->ap_opts_comments_extract > 0) &&
-            (gtk_text_buffer_get_char_count (text_tb) == 0))
+    if (comments_extract && ! gtk_text_buffer_get_char_count (text_tb))
     {
         GtkTextIter start, end;
         GtkTextTag * tag = gtk_text_buffer_create_tag (text_tb, "italicstyle",
@@ -304,8 +305,7 @@ void i_fileinfo_gui (const char * filename_uri)
         gtk_text_buffer_apply_tag (text_tb, tag, &start, &end);
     }
 
-    if ((amidiplug_cfg_ap->ap_opts_lyrics_extract > 0) &&
-            (gtk_text_buffer_get_char_count (lyrics_tb) == 0))
+    if (lyrics_extract && ! gtk_text_buffer_get_char_count (lyrics_tb))
     {
         GtkTextIter start, end;
         GtkTextTag * tag = gtk_text_buffer_create_tag (lyrics_tb, "italicstyle",
@@ -318,18 +318,17 @@ void i_fileinfo_gui (const char * filename_uri)
     }
 
     /* hide boxes for disabled options (comments and/or lyrics) */
-    if ((amidiplug_cfg_ap->ap_opts_comments_extract == 0) &&
-            (amidiplug_cfg_ap->ap_opts_lyrics_extract == 0))
+    if (! comments_extract && ! lyrics_extract)
     {
         gtk_widget_set_no_show_all (miditextboxes_vbox, TRUE);
         gtk_widget_hide (miditextboxes_vbox);
     }
-    else if (amidiplug_cfg_ap->ap_opts_comments_extract == 0)
+    else if (! comments_extract)
     {
         gtk_widget_set_no_show_all (text_frame, TRUE);
         gtk_widget_hide (text_frame);
     }
-    else if (amidiplug_cfg_ap->ap_opts_lyrics_extract == 0)
+    else if (! lyrics_extract)
     {
         gtk_widget_set_no_show_all (lyrics_frame, TRUE);
         gtk_widget_hide (lyrics_frame);
@@ -339,7 +338,7 @@ void i_fileinfo_gui (const char * filename_uri)
      *** FOOTER ***/
     footer_hbbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
     gtk_button_box_set_layout (GTK_BUTTON_BOX (footer_hbbox), GTK_BUTTONBOX_END);
-    footer_bclose = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+    footer_bclose = gtk_button_new_with_mnemonic (_("_Close"));
     g_signal_connect (G_OBJECT (footer_bclose), "clicked", G_CALLBACK (i_fileinfo_ev_close), fileinfowin);
     gtk_container_add (GTK_CONTAINER (footer_hbbox), footer_bclose);
     gtk_box_pack_start (GTK_BOX (fileinfowin_vbox), footer_hbbox, FALSE, FALSE, 0);
