@@ -23,12 +23,21 @@
 
 #include "i_midi.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "i_common.h"
+#include <glib.h>
+
+#include <audacious/misc.h>
+
 #include "i_configure.h"
 
+/* #define DEBUGMSG(...) fprintf (stderr, __VA_ARGS__) */
+#define DEBUGMSG(...)
+
+#define WARNANDBREAK(...) { fprintf (stderr, __VA_ARGS__); break; }
+
 #define ERRMSG_MIDITRACK() { fprintf(stderr, "%s: invalid MIDI data (offset %#x)", mf->file_name, mf->file_offset ); return 0; }
 
 
@@ -127,8 +136,8 @@ midievent_t * i_midi_file_new_event (midifile_track_t * track, int sysex_length)
 {
     midievent_t * event;
 
-    event = malloc (sizeof (midievent_t));
-    event->sysex = sysex_length ? malloc (sysex_length) : NULL;
+    event = g_new (midievent_t, 1);
+    event->sysex = sysex_length ? g_malloc (sysex_length) : NULL;
     event->next = NULL;
 
     /* append at the end of the track's linked list */
@@ -218,8 +227,8 @@ int i_midi_file_read_track (midifile_t * mf, midifile_track_t * track,
             /* if this note is not in standard drum channel (10), apply transpose */
             if (event->data.d[0] != 9)
             {
-                int data_tr = (i_midi_file_read_byte (mf) & 0x7f) +
-                              amidiplug_cfg_ap->ap_opts_transpose_value;
+                int transpose = aud_get_int ("amidiplug", "ap_opts_transpose_value");
+                int data_tr = (i_midi_file_read_byte (mf) & 0x7f) + transpose;
 
                 if (data_tr > 127) data_tr = 127;
                 else if (data_tr < 0) data_tr = 0;
@@ -228,8 +237,8 @@ int i_midi_file_read_track (midifile_t * mf, midifile_track_t * track,
             }
             else /* this note is in standard drum channel (10), apply drum shift */
             {
-                int data_ds = (i_midi_file_read_byte (mf) & 0x7f) +
-                              amidiplug_cfg_ap->ap_opts_drumshift_value; /* always > 0 */
+                int drumshift = aud_get_int ("amidiplug", "ap_opts_drumshift_value");
+                int data_ds = (i_midi_file_read_byte (mf) & 0x7f) + drumshift; /* always > 0 */
 
                 if (data_ds > 127) data_ds -= 127;
 
@@ -354,7 +363,7 @@ int i_midi_file_read_track (midifile_t * mf, midifile_track_t * track,
 
                 case 0x01: /* text comments */
                 {
-                    if (amidiplug_cfg_ap->ap_opts_comments_extract > 0)
+                    if (aud_get_int ("amidiplug", "ap_opts_comments_extract") > 0)
                     {
                         int ic = 0;
 
@@ -364,7 +373,7 @@ int i_midi_file_read_track (midifile_t * mf, midifile_track_t * track,
                         event = i_midi_file_new_event (track, 0);
                         event->type = SND_SEQ_EVENT_META_TEXT;
                         event->tick = tick;
-                        event->data.metat = malloc (len + 1);
+                        event->data.metat = g_malloc (len + 1);
 
                         for (ic = 0 ; ic < len ; ic++)
                             event->data.metat[ic] = i_midi_file_read_byte (mf);
@@ -378,7 +387,7 @@ int i_midi_file_read_track (midifile_t * mf, midifile_track_t * track,
 
                 case 0x05: /* lyrics */
                 {
-                    if (amidiplug_cfg_ap->ap_opts_lyrics_extract > 0)
+                    if (aud_get_int ("amidiplug", "ap_opts_lyrics_extract"))
                     {
                         int ic = 0;
 
@@ -388,7 +397,7 @@ int i_midi_file_read_track (midifile_t * mf, midifile_track_t * track,
                         event = i_midi_file_new_event (track, 0);
                         event->type = SND_SEQ_EVENT_META_LYRIC;
                         event->tick = tick;
-                        event->data.metat = malloc (len + 1);
+                        event->data.metat = g_malloc (len + 1);
 
                         for (ic = 0 ; ic < len ; ic++)
                             event->data.metat[ic] = i_midi_file_read_byte (mf);
@@ -456,8 +465,7 @@ int i_midi_file_parse_smf (midifile_t * mf, int port_count)
         return 0;
     }
 
-    mf->tracks = malloc (mf->num_tracks * sizeof (midifile_track_t));
-    memset (mf->tracks, 0, mf->num_tracks * sizeof (midifile_track_t));
+    mf->tracks = g_new0 (midifile_track_t, mf->num_tracks);
 
     mf->time_division = i_midi_file_read_int (mf,2);
 
@@ -571,14 +579,13 @@ void i_midi_init (midifile_t * mf)
     mf->playing_tick = 0;
     mf->avg_microsec_per_tick = 0;
     mf->length = 0;
-    mf->skip_offset = 0;
     return;
 }
 
 
 void i_midi_free (midifile_t * mf)
 {
-    free (mf->file_name);
+    g_free (mf->file_name);
     mf->file_name = NULL;
 
     if (mf->tracks)
@@ -598,15 +605,15 @@ void i_midi_free (midifile_t * mf)
 
                 if ((event_tmp->type == SND_SEQ_EVENT_META_TEXT) ||
                         (event_tmp->type == SND_SEQ_EVENT_META_LYRIC))
-                    free (event_tmp->data.metat);
+                    g_free (event_tmp->data.metat);
 
-                free (event_tmp->sysex);
-                free (event_tmp);
+                g_free (event_tmp->sysex);
+                g_free (event_tmp);
             }
         }
 
         /* free track array */
-        free (mf->tracks);
+        g_free (mf->tracks);
         mf->tracks = NULL;
     }
 }
@@ -839,7 +846,7 @@ int i_midi_parse_from_filename (const char * filename, midifile_t * mf)
         return 0;
     }
 
-    mf->file_name = strdup (filename);
+    mf->file_name = g_strdup (filename);
 
     switch (i_midi_file_read_id (mf))
     {
