@@ -37,7 +37,7 @@ void i_about_gui (void)
         return;
 
     aboutwin = gtk_dialog_new_with_buttons (_("About AMIDI-Plug"), NULL, 0,
-                                            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+                                            _("_Close"), GTK_RESPONSE_CLOSE, NULL);
     gtk_window_set_resizable (GTK_WINDOW (aboutwin), FALSE);
 
     g_signal_connect (aboutwin, "response", (GCallback) gtk_widget_destroy, NULL);
@@ -67,7 +67,7 @@ void i_about_gui (void)
 
     GtkWidget * label = gtk_label_new (text);
     gtk_box_pack_start ((GtkBox *) vbox, label, FALSE, FALSE, 0);
-    free (text);
+    g_free (text);
 
     gtk_widget_show_all (aboutwin);
 }
