@@ -23,13 +23,13 @@
 #include <audacious/i18n.h>
 #include <audacious/misc.h>
 #include <audacious/plugin.h>
+#include <libaudcore/audstrings.h>
 
 #define N_ITEMS 2
 #define N_MENUS 3
 
 static const gchar * titles[N_ITEMS] = {N_("Play CD"), N_("Add CD")};
-static const gint menus[N_MENUS] = {AUD_MENU_MAIN, AUD_MENU_PLAYLIST_ADD,
- AUD_MENU_PLAYLIST_RCLICK};
+static const gint menus[N_MENUS] = {AUD_MENU_MAIN, AUD_MENU_PLAYLIST_ADD, AUD_MENU_PLAYLIST};
 
 static void cd_play (void) {aud_drct_pl_open ("cdda://"); }
 static void cd_add (void) {aud_drct_pl_add ("cdda://", -1); }
@@ -39,8 +39,7 @@ static gboolean cd_init (void)
 {
     for (gint m = 0; m < N_MENUS; m ++)
         for (gint i = 0; i < N_ITEMS; i ++)
-            aud_plugin_menu_add (menus[m], funcs[i], _(titles[i]),
-             GTK_STOCK_CDROM);
+            aud_plugin_menu_add (menus[m], funcs[i], _(titles[i]), "media-optical");
 
     return TRUE;
 }
