@@ -45,6 +45,7 @@
 
 #include <audacious/debug.h>
 #include <audacious/i18n.h>
+#include <audacious/input.h>
 #include <audacious/misc.h>
 #include <audacious/playlist.h>
 #include <audacious/plugin.h>
@@ -75,7 +76,6 @@ typedef struct
 trackinfo_t;
 
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static int seek_time;
 static bool_t playing;
 
 /* lock mutex to read / set these variables */
@@ -88,15 +88,12 @@ static int monitor_source = 0;
 
 static bool_t cdaudio_init (void);
 static int cdaudio_is_our_file (const char * filename, VFSFile * file);
-static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
- file, int start, int stop, bool_t pause);
-static void cdaudio_stop (InputPlayback * pinputplayback);
-static void cdaudio_pause (InputPlayback * p, bool_t paused);
-static void cdaudio_mseek (InputPlayback * p, int time);
+static bool_t cdaudio_play (const char * name, VFSFile * file);
 static void cdaudio_cleanup (void);
 static Tuple * make_tuple (const char * filename, VFSFile * file);
 static void scan_cd (void);
 static void refresh_trackinfo (bool_t warning);
+static void reset_trackinfo (void);
 static int calculate_track_length (int startlsn, int endlsn);
 static int find_trackno_from_filename (const char * filename);
 
@@ -142,7 +139,7 @@ static const PreferencesWidget cdaudio_widgets[] = {
 
 static const PluginPreferences cdaudio_prefs = {
  .widgets = cdaudio_widgets,
- .n_widgets = G_N_ELEMENTS (cdaudio_widgets)};
+ .n_widgets = ARRAY_LEN (cdaudio_widgets)};
 
 AUD_INPUT_PLUGIN
 (
@@ -154,9 +151,6 @@ AUD_INPUT_PLUGIN
     .cleanup = cdaudio_cleanup,
     .is_our_file_from_vfs = cdaudio_is_our_file,
     .play = cdaudio_play,
-    .stop = cdaudio_stop,
-    .pause = cdaudio_pause,
-    .mseek = cdaudio_mseek,
     .probe_for_tuple = make_tuple,
     .schemes = schemes,
     .have_subtune = TRUE,
@@ -284,8 +278,7 @@ static void cdaudio_set_fullinfo (trackinfo_t * t,
 }
 
 /* play thread only */
-static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
- file, int start, int stop, bool_t pause)
+static bool_t cdaudio_play (const char * name, VFSFile * file)
 {
     pthread_mutex_lock (& mutex);
 
@@ -309,7 +302,7 @@ static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
         cdaudio_error (_("Track %d not found."), trackno);
     else if (! cdda_track_audiop (pcdrom_drive, trackno))
         cdaudio_error (_("Track %d is a data track."), trackno);
-    else if (! p->output->open_audio (FMT_S16_LE, 44100, 2))
+    else if (! aud_input_open_audio (FMT_S16_LE, 44100, 2))
         cdaudio_error (_("Failed to open audio output."));
     else
         okay = TRUE;
@@ -323,17 +316,9 @@ static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
     int startlsn = trackinfo[trackno].startlsn;
     int endlsn = trackinfo[trackno].endlsn;
 
-    seek_time = (start > 0) ? start : -1;
     playing = TRUE;
 
-    if (stop >= 0)
-        endlsn = MIN (endlsn, startlsn + stop * 75 / 1000);
-
-    if (pause)
-        p->output->pause (TRUE);
-
-    p->set_params (p, 1411200, 44100, 2);
-    p->set_pb_ready (p);
+    aud_input_set_bitrate (1411200);
 
     int buffer_size = aud_get_int (NULL, "output_buffer_size");
     int speed = aud_get_int ("CDDA", "disc_speed");
@@ -343,14 +328,11 @@ static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
     int currlsn = startlsn;
     int retry_count = 0, skip_count = 0;
 
-    while (playing)
+    while (! aud_input_check_stop ())
     {
+        int seek_time = aud_input_check_seek ();
         if (seek_time >= 0)
-        {
-            p->output->flush (seek_time);
             currlsn = startlsn + (seek_time * 75 / 1000);
-            seek_time = -1;
-        }
 
         sectors = MIN (sectors, endlsn + 1 - currlsn);
         if (sectors < 1)
@@ -364,7 +346,7 @@ static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
          currlsn, sectors);
 
         if (ret == DRIVER_OP_SUCCESS)
-            p->output->write_audio (buffer, 2352 * sectors);
+            aud_input_write_audio (buffer, 2352 * sectors);
 
         pthread_mutex_lock (& mutex);
 
@@ -404,55 +386,12 @@ static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
     return TRUE;
 }
 
-/* main thread only */
-static void cdaudio_stop (InputPlayback * p)
-{
-    pthread_mutex_lock (& mutex);
-    playing = FALSE;
-    p->output->abort_write();
-    pthread_mutex_unlock (& mutex);
-}
-
-/* main thread only */
-static void cdaudio_pause (InputPlayback * p, bool_t pause)
-{
-    pthread_mutex_lock (& mutex);
-    p->output->pause (pause);
-    pthread_mutex_unlock (& mutex);
-}
-
-/* main thread only */
-static void cdaudio_mseek (InputPlayback * p, int time)
-{
-    pthread_mutex_lock (& mutex);
-    seek_time = time;
-    p->output->abort_write();
-    pthread_mutex_unlock (& mutex);
-}
-
 /* main thread only */
 static void cdaudio_cleanup (void)
 {
     pthread_mutex_lock (& mutex);
 
-    if (monitor_source)
-    {
-        g_source_remove (monitor_source);
-        monitor_source = 0;
-    }
-
-    if (pcdrom_drive != NULL)
-    {
-        cdda_close (pcdrom_drive);
-        pcdrom_drive = NULL;
-    }
-
-    if (trackinfo != NULL)
-    {
-        g_free (trackinfo);
-        trackinfo = NULL;
-    }
-
+    reset_trackinfo ();
     libcddb_shutdown ();
 
     pthread_mutex_unlock (& mutex);
@@ -461,17 +400,21 @@ static void cdaudio_cleanup (void)
 /* thread safe */
 static Tuple * make_tuple (const char * filename, VFSFile * file)
 {
-    Tuple *tuple = NULL;
-    int trackno;
+    bool_t whole_disk = ! strcmp (filename, "cdda://");
+    Tuple * tuple = NULL;
 
     pthread_mutex_lock (& mutex);
 
+    /* reset cached info when adding CD to the playlist */
+    if (whole_disk && ! playing)
+        reset_trackinfo ();
+
     if (trackinfo == NULL)
         refresh_trackinfo (TRUE);
     if (trackinfo == NULL)
         goto DONE;
 
-    if (!strcmp (filename, "cdda://"))
+    if (whole_disk)
     {
         tuple = tuple_new_from_filename (filename);
 
@@ -479,7 +422,7 @@ static Tuple * make_tuple (const char * filename, VFSFile * file)
         int i = 0;
 
         /* only add the audio tracks to the playlist */
-        for (trackno = firsttrackno; trackno <= lasttrackno; trackno++)
+        for (int trackno = firsttrackno; trackno <= lasttrackno; trackno++)
             if (cdda_track_audiop (pcdrom_drive, trackno))
                 subtunes[i ++] = trackno;
 
@@ -488,7 +431,7 @@ static Tuple * make_tuple (const char * filename, VFSFile * file)
         goto DONE;
     }
 
-    trackno = find_trackno_from_filename (filename);
+    int trackno = find_trackno_from_filename (filename);
 
     if (trackno < firsttrackno || trackno > lasttrackno)
     {
@@ -504,24 +447,24 @@ static Tuple * make_tuple (const char * filename, VFSFile * file)
 
     tuple = tuple_new_from_filename (filename);
     tuple_set_format (tuple, _("Audio CD"), 2, 44100, 1411);
-    tuple_set_int (tuple, FIELD_TRACK_NUMBER, NULL, trackno);
-    tuple_set_int (tuple, FIELD_LENGTH, NULL, calculate_track_length
+    tuple_set_int (tuple, FIELD_TRACK_NUMBER, trackno);
+    tuple_set_int (tuple, FIELD_LENGTH, calculate_track_length
      (trackinfo[trackno].startlsn, trackinfo[trackno].endlsn));
 
     if (trackinfo[trackno].name[0])
-        tuple_set_str (tuple, FIELD_TITLE, NULL, trackinfo[trackno].name);
+        tuple_set_str (tuple, FIELD_TITLE, trackinfo[trackno].name);
     else
     {
         SPRINTF (title, _("Track %d"), trackno);
-        tuple_set_str (tuple, FIELD_TITLE, NULL, title);
+        tuple_set_str (tuple, FIELD_TITLE, title);
     }
 
     if (trackinfo[trackno].performer[0])
-        tuple_set_str (tuple, FIELD_ARTIST, NULL, trackinfo[trackno].performer);
+        tuple_set_str (tuple, FIELD_ARTIST, trackinfo[trackno].performer);
     if (trackinfo[0].name[0])
-        tuple_set_str (tuple, FIELD_ALBUM, NULL, trackinfo[0].name);
+        tuple_set_str (tuple, FIELD_ALBUM, trackinfo[0].name);
     if (trackinfo[trackno].genre[0])
-        tuple_set_str (tuple, FIELD_GENRE, NULL, trackinfo[trackno].genre);
+        tuple_set_str (tuple, FIELD_GENRE, trackinfo[trackno].genre);
 
   DONE:
     pthread_mutex_unlock (& mutex);
@@ -534,7 +477,7 @@ static void open_cd (void)
     AUDDBG ("Opening CD drive.\n");
     g_return_if_fail (pcdrom_drive == NULL);
 
-    char * device = aud_get_string ("CDDA", "device");
+    char * device = aud_get_str ("CDDA", "device");
 
     if (device[0])
     {
@@ -557,7 +500,7 @@ static void open_cd (void)
             cdio_free_device_list (ppcd_drives);
     }
 
-    free (device);
+    str_unref (device);
 }
 
 /* mutex must be locked */
@@ -708,16 +651,16 @@ static void scan_cd (void)
                 cddb_cache_enable (pcddb_conn);
                 // cddb_cache_set_dir(pcddb_conn, "~/.cddbslave");
 
-                char * server = aud_get_string ("CDDA", "cddbserver");
-                char * path = aud_get_string ("CDDA", "cddbpath");
+                char * server = aud_get_str ("CDDA", "cddbserver");
+                char * path = aud_get_str ("CDDA", "cddbpath");
                 int port = aud_get_int ("CDDA", "cddbport");
 
                 if (aud_get_bool (NULL, "use_proxy"))
                 {
-                    char * prhost = aud_get_string (NULL, "proxy_host");
+                    char * prhost = aud_get_str (NULL, "proxy_host");
                     int prport = aud_get_int (NULL, "proxy_port");
-                    char * pruser = aud_get_string (NULL, "proxy_user");
-                    char * prpass = aud_get_string (NULL, "proxy_pass");
+                    char * pruser = aud_get_str (NULL, "proxy_user");
+                    char * prpass = aud_get_str (NULL, "proxy_pass");
 
                     cddb_http_proxy_enable (pcddb_conn);
                     cddb_set_http_proxy_server_name (pcddb_conn, prhost);
@@ -725,9 +668,9 @@ static void scan_cd (void)
                     cddb_set_http_proxy_username (pcddb_conn, pruser);
                     cddb_set_http_proxy_password (pcddb_conn, prpass);
 
-                    free (prhost);
-                    free (pruser);
-                    free (prpass);
+                    str_unref (prhost);
+                    str_unref (pruser);
+                    str_unref (prpass);
 
                     cddb_set_server_name (pcddb_conn, server);
                     cddb_set_server_port (pcddb_conn, port);
@@ -745,8 +688,8 @@ static void scan_cd (void)
                     cddb_set_server_port (pcddb_conn, port);
                 }
 
-                free (server);
-                free (path);
+                str_unref (server);
+                str_unref (path);
 
                 pcddb_disc = cddb_disc_new ();
 
@@ -855,8 +798,6 @@ static void scan_cd (void)
 /* mutex must be locked */
 static void refresh_trackinfo (bool_t warning)
 {
-    trigger_monitor ();
-
     if (pcdrom_drive == NULL)
     {
         open_cd ();
@@ -879,15 +820,7 @@ static void refresh_trackinfo (bool_t warning)
                 cdaudio_error (_("Unsupported disk type."));
         }
 
-        /* reset libcdio, else it will not read a new disk correctly */
-        if (pcdrom_drive)
-        {
-            cdda_close (pcdrom_drive);
-            pcdrom_drive = NULL;
-        }
-
-        g_free (trackinfo);
-        trackinfo = NULL;
+        reset_trackinfo ();
         return;
     }
 
@@ -896,9 +829,31 @@ static void refresh_trackinfo (bool_t warning)
         g_free (trackinfo);
         trackinfo = NULL;
         scan_cd ();
+
+        if (trackinfo != NULL)
+            trigger_monitor ();
     }
 }
 
+/* mutex must be locked */
+static void reset_trackinfo (void)
+{
+    if (monitor_source)
+    {
+        g_source_remove (monitor_source);
+        monitor_source = 0;
+    }
+
+    if (pcdrom_drive != NULL)
+    {
+        cdda_close (pcdrom_drive);
+        pcdrom_drive = NULL;
+    }
+
+    g_free (trackinfo);
+    trackinfo = NULL;
+}
+
 /* thread safe (mutex may be locked) */
 static int calculate_track_length (int startlsn, int endlsn)
 {
