@@ -42,7 +42,7 @@ static const PreferencesWidget compressor_widgets[] = {
 
 static const PluginPreferences compressor_prefs = {
  .widgets = compressor_widgets,
- .n_widgets = sizeof compressor_widgets / sizeof compressor_widgets[0]};
+ .n_widgets = ARRAY_LEN (compressor_widgets)};
 
 void compressor_config_load (void)
 {
