@@ -7,11 +7,12 @@
  */
 
 #include <math.h>
-#include <pthread.h>
+#include <stdio.h>
 #include <string.h>
 
 extern "C" {
 #include <libaudcore/audstrings.h>
+#include <audacious/input.h>
 #include <audacious/plugin.h>
 }
 
@@ -19,25 +20,21 @@ extern "C" {
 #include "Music_Emu.h"
 #include "Gzip_Reader.h"
 
-static pthread_mutex_t seek_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_cond_t seek_cond = PTHREAD_COND_INITIALIZER;
-static gint seek_value = -1;
-static gboolean stop_flag = FALSE;
-
-static const gint fade_threshold = 10 * 1000;
-static const gint fade_length    = 8 * 1000;
+static const int fade_threshold = 10 * 1000;
+static const int fade_length    = 8 * 1000;
 
 static blargg_err_t log_err(blargg_err_t err)
 {
-    if (err) g_critical("console: %s\n", err);
+    if (err)
+        fprintf (stderr, "console: %s\n", err);
     return err;
 }
 
 static void log_warning(Music_Emu * emu)
 {
-    const gchar *str = emu->warning();
+    const char *str = emu->warning();
     if (str != NULL)
-        g_warning("console: %s\n", str);
+        fprintf (stderr, "console: %s\n", str);
 }
 
 /* Handles URL parsing, file opening and identification, and file
@@ -46,36 +43,36 @@ static void log_warning(Music_Emu * emu)
  */
 class ConsoleFileHandler {
 public:
-    gchar *m_path;            // path without track number specification
-    gint m_track;             // track number (0 = first track)
+    char *m_path;            // path without track number specification
+    int m_track;             // track number (0 = first track)
     Music_Emu* m_emu;         // set to 0 to take ownership
     gme_type_t m_type;
 
     // Parses path and identifies file type
-    ConsoleFileHandler(const gchar* path, VFSFile *fd = NULL);
+    ConsoleFileHandler(const char* path, VFSFile *fd = NULL);
 
     // Creates emulator and returns 0. If this wasn't a music file or
     // emulator couldn't be created, returns 1.
-    gint load(gint sample_rate);
+    int load(int sample_rate);
 
     // Deletes owned emu and closes file
     ~ConsoleFileHandler();
 
 private:
-    gchar m_header[4];
+    char m_header[4];
     Vfs_File_Reader vfs_in;
     Gzip_Reader gzip_in;
 };
 
-ConsoleFileHandler::ConsoleFileHandler(const gchar *path, VFSFile *fd)
+ConsoleFileHandler::ConsoleFileHandler(const char *path, VFSFile *fd)
 {
     m_emu   = NULL;
     m_type  = 0;
     m_track = -1;
 
-    const gchar * sub;
+    const char * sub;
     uri_parse (path, NULL, NULL, & sub, & m_track);
-    m_path = g_strndup (path, sub - path);
+    m_path = str_nget (path, sub - path);
 
     m_track -= 1;
 
@@ -105,10 +102,10 @@ ConsoleFileHandler::ConsoleFileHandler(const gchar *path, VFSFile *fd)
 ConsoleFileHandler::~ConsoleFileHandler()
 {
     gme_delete(m_emu);
-    g_free(m_path);
+    str_unref (m_path);
 }
 
-gint ConsoleFileHandler::load(gint sample_rate)
+int ConsoleFileHandler::load(int sample_rate)
 {
     if (!m_type)
         return 1;
@@ -133,8 +130,8 @@ gint ConsoleFileHandler::load(gint sample_rate)
 
 #if 0
     // load .m3u from same directory( replace/add extension with ".m3u")
-    gchar *m3u_path = g_strdup(m_path);
-    gchar *ext = strrchr(m3u_path, '.');
+    char *m3u_path = g_strdup(m_path);
+    char *ext = strrchr(m3u_path, '.');
     if (ext == NULL)
     {
         ext = g_strdup_printf("%s.m3u", m3u_path);
@@ -153,31 +150,24 @@ gint ConsoleFileHandler::load(gint sample_rate)
     return 0;
 }
 
-static inline void set_str (Tuple * tuple, int field, const char * str)
-{
-    char * valid = str_to_utf8 (str);
-    tuple_set_str (tuple, field, NULL, valid);
-    free (valid);
-}
-
-static Tuple * get_track_ti(const gchar *path, const track_info_t *info, const gint track)
+static Tuple * get_track_ti(const char *path, const track_info_t *info, const int track)
 {
     Tuple *ti = tuple_new_from_filename(path);
 
     if (ti != NULL)
     {
-        set_str (ti, FIELD_ARTIST, info->author);
-        set_str (ti, FIELD_ALBUM, info->game);
-        set_str (ti, FIELD_TITLE, info->song);
-        set_str (ti, FIELD_COPYRIGHT, info->copyright);
-        set_str (ti, FIELD_CODEC, info->system);
-        set_str (ti, FIELD_COMMENT, info->comment);
+        tuple_set_str (ti, FIELD_ARTIST, info->author);
+        tuple_set_str (ti, FIELD_ALBUM, info->game);
+        tuple_set_str (ti, FIELD_TITLE, info->song);
+        tuple_set_str (ti, FIELD_COPYRIGHT, info->copyright);
+        tuple_set_str (ti, FIELD_CODEC, info->system);
+        tuple_set_str (ti, FIELD_COMMENT, info->comment);
 
         if (track >= 0)
         {
-            tuple_set_int(ti, FIELD_TRACK_NUMBER, NULL, track + 1);
-            tuple_set_int(ti, FIELD_SUBSONG_ID, NULL, track + 1);
-            tuple_set_int(ti, FIELD_SUBSONG_NUM, NULL, info->track_count);
+            tuple_set_int(ti, FIELD_TRACK_NUMBER, track + 1);
+            tuple_set_int(ti, FIELD_SUBSONG_ID, track + 1);
+            tuple_set_int(ti, FIELD_SUBSONG_NUM, info->track_count);
         }
         else
             tuple_set_subtunes (ti, info->track_count, NULL);
@@ -189,13 +179,13 @@ static Tuple * get_track_ti(const gchar *path, const track_info_t *info, const g
             length = audcfg.loop_length * 1000;
         else if (length >= fade_threshold)
             length += fade_length;
-        tuple_set_int(ti, FIELD_LENGTH, NULL, length);
+        tuple_set_int(ti, FIELD_LENGTH, length);
     }
 
     return ti;
 }
 
-extern "C" Tuple * console_probe_for_tuple(const gchar *filename, VFSFile *fd)
+extern "C" Tuple * console_probe_for_tuple(const char *filename, VFSFile *fd)
 {
     ConsoleFileHandler fh(filename, fd);
 
@@ -212,12 +202,10 @@ extern "C" Tuple * console_probe_for_tuple(const gchar *filename, VFSFile *fd)
     return NULL;
 }
 
-extern "C" gboolean console_play(InputPlayback *playback, const gchar *filename,
-    VFSFile *file, gint start_time, gint stop_time, gboolean pause)
+extern "C" bool_t console_play(const char *filename, VFSFile *file)
 {
-    gint length, end_delay, sample_rate;
+    int length, sample_rate;
     track_info_t info;
-    gboolean error = FALSE;
 
     // identify file
     ConsoleFileHandler fh(filename);
@@ -269,9 +257,9 @@ extern "C" gboolean console_play(InputPlayback *playback, const gchar *filename,
         Tuple *ti = get_track_ti(fh.m_path, &info, fh.m_track);
         if (ti != NULL)
         {
-            length = tuple_get_int(ti, FIELD_LENGTH, NULL);
+            length = tuple_get_int(ti, FIELD_LENGTH);
             tuple_unref(ti);
-            playback->set_params(playback, fh.m_emu->voice_count() * 1000, sample_rate, 2);
+            aud_input_set_bitrate(fh.m_emu->voice_count() * 1000);
         }
     }
 
@@ -281,12 +269,9 @@ extern "C" gboolean console_play(InputPlayback *playback, const gchar *filename,
 
     log_warning(fh.m_emu);
 
-    if (!playback->output->open_audio(FMT_S16_NE, sample_rate, 2))
+    if (!aud_input_open_audio(FMT_S16_NE, sample_rate, 2))
         return FALSE;
 
-    if (pause)
-        playback->output->pause(TRUE);
-
     // set fade time
     if (length <= 0)
         length = audcfg.loop_length * 1000;
@@ -294,92 +279,24 @@ extern "C" gboolean console_play(InputPlayback *playback, const gchar *filename,
         length -= fade_length / 2;
     fh.m_emu->set_fade(length, fade_length);
 
-    stop_flag = FALSE;
-    end_delay = 0;
-    playback->set_pb_ready(playback);
-
-    while (!stop_flag)
+    while (!aud_input_check_stop())
     {
         /* Perform seek, if requested */
-        pthread_mutex_lock(&seek_mutex);
+        int seek_value = aud_input_check_seek();
         if (seek_value >= 0)
-        {
-            playback->output->flush(seek_value);
             fh.m_emu->seek(seek_value);
-            seek_value = -1;
-            pthread_cond_signal(&seek_cond);
-        }
-        pthread_mutex_unlock(&seek_mutex);
 
         /* Fill and play buffer of audio */
-        gint const buf_size = 1024;
+        int const buf_size = 1024;
         Music_Emu::sample_t buf[buf_size];
-        if (end_delay)
-        {
-            // TODO: remove delay once host doesn't cut the end of track off
-            if (!--end_delay)
-                stop_flag = TRUE;
-            memset(buf, 0, sizeof(buf));
-        }
-        else
-        {
-            fh.m_emu->play(buf_size, buf);
-            if (fh.m_emu->track_ended())
-            {
-                end_delay = (fh.m_emu->sample_rate() * 3 * 2) / buf_size;
-            }
-        }
 
-        playback->output->write_audio(buf, sizeof(buf));
+        fh.m_emu->play(buf_size, buf);
+
+        aud_input_write_audio(buf, sizeof(buf));
+
+        if (fh.m_emu->track_ended())
+            break;
     }
 
-    // stop playing
-    stop_flag = TRUE;
-
-    return !error;
-}
-
-extern "C" void console_seek(InputPlayback *playback, gint time)
-{
-    pthread_mutex_lock(&seek_mutex);
-
-    if (!stop_flag)
-    {
-        seek_value = time;
-        playback->output->abort_write();
-        pthread_cond_signal(&seek_cond);
-        pthread_cond_wait(&seek_cond, &seek_mutex);
-    }
-
-    pthread_mutex_unlock(&seek_mutex);
-}
-
-extern "C" void console_stop(InputPlayback *playback)
-{
-    pthread_mutex_lock(&seek_mutex);
-
-    if (!stop_flag)
-    {
-        stop_flag = TRUE;
-        playback->output->abort_write();
-        pthread_cond_signal(&seek_cond);
-    }
-
-    pthread_mutex_unlock (&seek_mutex);
-}
-
-extern "C" void console_pause(InputPlayback * playback, gboolean pause)
-{
-    pthread_mutex_lock(&seek_mutex);
-
-    if (!stop_flag)
-        playback->output->pause(pause);
-
-    pthread_mutex_unlock(&seek_mutex);
-}
-
-extern "C" gboolean console_init (void)
-{
-    console_cfg_load();
     return TRUE;
 }
