@@ -32,14 +32,11 @@
 
 #include "ffaudio-stdinc.h"
 #include <audacious/i18n.h>
+#include <audacious/input.h>
 #include <audacious/debug.h>
 #include <audacious/audtag.h>
 #include <libaudcore/audstrings.h>
 
-static pthread_mutex_t ctrl_mutex = PTHREAD_MUTEX_INITIALIZER;
-static gint64 seek_value = -1;
-static gboolean stop_flag = FALSE;
-
 static pthread_mutex_t data_mutex = PTHREAD_MUTEX_INITIALIZER;
 static GHashTable * extension_dict = NULL;
 
@@ -283,7 +280,11 @@ ffaudio_codec_is_seekable(AVCodec *codec)
      */
     switch (codec->id) {
 #ifndef FFAUDIO_NO_BLACKLIST
+#if CHECK_LIBAVCODEC_VERSION(54, 25, 0)
+        case AV_CODEC_ID_MUSEPACK8:
+#else
         case CODEC_ID_MUSEPACK8:
+#endif
             AUDDBG("codec is blacklisted from seeking\n");
             return FALSE;
 #endif
@@ -321,7 +322,7 @@ static const ffaudio_meta_t metaentries[] = {
 
 static void read_metadata_dict (Tuple * tuple, AVDictionary * dict)
 {
-    for (int i = 0; i < G_N_ELEMENTS (metaentries); i ++)
+    for (int i = 0; i < ARRAY_LEN (metaentries); i ++)
     {
         const ffaudio_meta_t * m = & metaentries[i];
         AVDictionaryEntry * entry = NULL;
@@ -332,9 +333,9 @@ static void read_metadata_dict (Tuple * tuple, AVDictionary * dict)
         if (entry && entry->value)
         {
             if (m->ttype == TUPLE_STRING)
-                tuple_set_str (tuple, m->field, NULL, entry->value);
+                tuple_set_str (tuple, m->field, entry->value);
             else if (m->ttype == TUPLE_INT)
-                tuple_set_int (tuple, m->field, NULL, atoi (entry->value));
+                tuple_set_int (tuple, m->field, atoi (entry->value));
         }
     }
 }
@@ -352,11 +353,11 @@ static Tuple * read_tuple (const gchar * filename, VFSFile * file)
         {
             tuple = tuple_new_from_filename (filename);
 
-            tuple_set_int (tuple, FIELD_LENGTH, NULL, ic->duration / 1000);
-            tuple_set_int (tuple, FIELD_BITRATE, NULL, ic->bit_rate / 1000);
+            tuple_set_int (tuple, FIELD_LENGTH, ic->duration / 1000);
+            tuple_set_int (tuple, FIELD_BITRATE, ic->bit_rate / 1000);
 
             if (cinfo.codec->long_name)
-                tuple_set_str (tuple, FIELD_CODEC, NULL, cinfo.codec->long_name);
+                tuple_set_str (tuple, FIELD_CODEC, cinfo.codec->long_name);
 
             if (ic->metadata)
                 read_metadata_dict (tuple, ic->metadata);
@@ -380,13 +381,13 @@ ffaudio_probe_for_tuple(const gchar *filename, VFSFile *fd)
     if (t == NULL)
         return NULL;
 
-    vfs_rewind(fd);
-    tag_tuple_read(t, fd);
+    if (! vfs_fseek (fd, 0, SEEK_SET))
+        tag_tuple_read (t, fd);
 
     return t;
 }
 
-static gboolean ffaudio_write_tag (const Tuple * tuple, VFSFile * file)
+static gboolean ffaudio_write_tag (const char * filename, VFSFile * file, const Tuple * tuple)
 {
     if (! file)
         return FALSE;
@@ -397,8 +398,7 @@ static gboolean ffaudio_write_tag (const Tuple * tuple, VFSFile * file)
     return tag_tuple_write(tuple, file, TAG_TYPE_NONE);
 }
 
-static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
- VFSFile * file, gint start_time, gint stop_time, gboolean pause)
+static gboolean ffaudio_play (const gchar * filename, VFSFile * file)
 {
     AUDDBG ("Playing %s.\n", filename);
     if (! file)
@@ -454,51 +454,37 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
     /* Open audio output */
     AUDDBG("opening audio output\n");
 
-    if (playback->output->open_audio(out_fmt, cinfo.context->sample_rate, cinfo.context->channels) <= 0)
+    if (aud_input_open_audio(out_fmt, cinfo.context->sample_rate, cinfo.context->channels) <= 0)
     {
         error = TRUE;
         goto error_exit;
     }
 
-    playback->set_gain_from_playlist(playback);
-
     AUDDBG("setting parameters\n");
 
-    if (pause)
-        playback->output->pause(TRUE);
+    aud_input_set_bitrate(ic->bit_rate);
 
-    playback->set_params(playback, ic->bit_rate, cinfo.context->sample_rate, cinfo.context->channels);
-
-    pthread_mutex_lock (& ctrl_mutex);
-
-    stop_flag = FALSE;
-    seek_value = (start_time > 0) ? start_time : -1;
-    playback->set_pb_ready(playback);
     errcount = 0;
     seekable = ffaudio_codec_is_seekable(cinfo.codec);
 
-    pthread_mutex_unlock (& ctrl_mutex);
-
-    while (!stop_flag && (stop_time < 0 ||
-     playback->output->written_time () < stop_time))
+    while (! aud_input_check_stop ())
     {
-        AVPacket tmp;
-        gint ret;
-
-        pthread_mutex_lock (& ctrl_mutex);
+        int seek_value = aud_input_check_seek ();
 
         if (seek_value >= 0 && seekable)
         {
-            playback->output->flush (seek_value);
             if (av_seek_frame (ic, -1, (gint64) seek_value * AV_TIME_BASE /
              1000, AVSEEK_FLAG_ANY) < 0)
             {
                 _ERROR("error while seeking\n");
             } else
                 errcount = 0;
+
+            seek_value = -1;
         }
-        seek_value = -1;
-        pthread_mutex_unlock (& ctrl_mutex);
+
+        AVPacket tmp;
+        gint ret;
 
         /* Read next frame (or more) of data */
         if ((ret = av_read_frame(ic, &pkt)) < 0)
@@ -529,23 +515,20 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
 
         /* Decode and play packet/frame */
         memcpy(&tmp, &pkt, sizeof(tmp));
-        while (tmp.size > 0 && !stop_flag)
+        while (tmp.size > 0 && ! aud_input_check_stop ())
         {
             /* Check for seek request and bail out if we have one */
-            pthread_mutex_lock (& ctrl_mutex);
-            if (seek_value != -1)
-            {
-                if (!seekable)
-                    seek_value = -1;
-                else
-                {
-                    pthread_mutex_unlock (& ctrl_mutex);
-                    break;
-                }
-            }
-            pthread_mutex_unlock (& ctrl_mutex);
+            if (seek_value < 0)
+                seek_value = aud_input_check_seek ();
 
+            if (seek_value >= 0)
+                break;
+
+#if CHECK_LIBAVCODEC_VERSION (55, 28, 1)
+            AVFrame * frame = av_frame_alloc ();
+#else
             AVFrame * frame = avcodec_alloc_frame ();
+#endif
             int decoded = 0;
             int len = avcodec_decode_audio4 (cinfo.context, frame, & decoded, & tmp);
 
@@ -567,18 +550,22 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
             {
                 if (bufsize < size)
                 {
-                    buf = realloc (buf, size);
+                    buf = g_realloc (buf, size);
                     bufsize = size;
                 }
 
                 audio_interlace ((const void * *) frame->data, out_fmt,
                  cinfo.context->channels, buf, frame->nb_samples);
-                playback->output->write_audio (buf, size);
+                aud_input_write_audio (buf, size);
             }
             else
-                playback->output->write_audio (frame->data[0], size);
+                aud_input_write_audio (frame->data[0], size);
 
-            av_free (frame);
+#if CHECK_LIBAVCODEC_VERSION (55, 28, 1)
+            av_frame_free (& frame);
+#else
+            avcodec_free_frame (& frame);
+#endif
         }
 
         if (pkt.data)
@@ -586,8 +573,6 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
     }
 
 error_exit:
-    stop_flag = TRUE;
-
     if (pkt.data)
         av_free_packet(&pkt);
     if (codec_opened)
@@ -595,47 +580,11 @@ error_exit:
     if (ic != NULL)
         close_input_file(ic);
 
-    free (buf);
+    g_free (buf);
 
     return ! error;
 }
 
-static void ffaudio_stop(InputPlayback * playback)
-{
-    pthread_mutex_lock (& ctrl_mutex);
-
-    if (!stop_flag)
-    {
-        stop_flag = TRUE;
-        playback->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& ctrl_mutex);
-}
-
-static void ffaudio_pause(InputPlayback * playback, gboolean pause)
-{
-    pthread_mutex_lock (& ctrl_mutex);
-
-    if (!stop_flag)
-        playback->output->pause(pause);
-
-    pthread_mutex_unlock (& ctrl_mutex);
-}
-
-static void ffaudio_seek (InputPlayback * playback, gint time)
-{
-    pthread_mutex_lock (& ctrl_mutex);
-
-    if (!stop_flag)
-    {
-        seek_value = time;
-        playback->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& ctrl_mutex);
-}
-
 static const char ffaudio_about[] =
  N_("Multi-format audio decoding plugin for Audacious using\n"
     "FFmpeg multimedia framework (http://www.ffmpeg.org/)\n"
@@ -685,6 +634,8 @@ static const gchar *ffaudio_fmts[] = {
     NULL
 };
 
+static const char * const ffaudio_mimes[] = {"application/ogg", NULL};
+
 AUD_INPUT_PLUGIN
 (
     .name = N_("FFmpeg Plugin"),
@@ -692,13 +643,11 @@ AUD_INPUT_PLUGIN
     .about_text = ffaudio_about,
     .init = ffaudio_init,
     .cleanup = ffaudio_cleanup,
+    .extensions = ffaudio_fmts,
+    .mimes = ffaudio_mimes,
     .is_our_file_from_vfs = ffaudio_probe,
     .probe_for_tuple = ffaudio_probe_for_tuple,
     .play = ffaudio_play,
-    .stop = ffaudio_stop,
-    .pause = ffaudio_pause,
-    .mseek = ffaudio_seek,
-    .extensions = ffaudio_fmts,
     .update_song_tuple = ffaudio_write_tag,
 
     /* lowest priority fallback */
