@@ -28,6 +28,7 @@
 #include <stdint.h>
 
 #include <audacious/misc.h>
+#include <libaudcore/audstrings.h>
 
 static gint (*write_output)(void *ptr, gint length);
 
@@ -59,7 +60,7 @@ static void vorbis_init(write_output_callback write_output_func)
 static void add_string_from_tuple (vorbis_comment * vc, const char * name,
  const Tuple * tuple, gint field)
 {
-    gchar * val = tuple_get_str (tuple, field, NULL);
+    gchar * val = tuple_get_str (tuple, field);
     if (! val)
         return;
 
@@ -90,15 +91,15 @@ static gint vorbis_open(void)
         add_string_from_tuple (& vc, "date", tuple, FIELD_DATE);
         add_string_from_tuple (& vc, "comment", tuple, FIELD_COMMENT);
 
-        if ((scrint = tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL)))
+        if ((scrint = tuple_get_int(tuple, FIELD_TRACK_NUMBER)))
         {
-            g_snprintf(tmpstr, sizeof(tmpstr), "%d", scrint);
+            str_itoa (scrint, tmpstr, sizeof tmpstr);
             vorbis_comment_add_tag(&vc, "tracknumber", tmpstr);
         }
 
-        if ((scrint = tuple_get_int(tuple, FIELD_YEAR, NULL)))
+        if ((scrint = tuple_get_int(tuple, FIELD_YEAR)))
         {
-            g_snprintf(tmpstr, sizeof(tmpstr), "%d", scrint);
+            str_itoa (scrint, tmpstr, sizeof tmpstr);
             vorbis_comment_add_tag(&vc, "year", tmpstr);
         }
     }
@@ -206,7 +207,7 @@ static void vorbis_configure(void)
     if (! configure_win)
     {
         configure_win = gtk_dialog_new_with_buttons
-         (_("Vorbis Encoder Configuration"), NULL, 0, GTK_STOCK_CLOSE,
+         (_("Vorbis Encoder Configuration"), NULL, 0, _("_Close"),
          GTK_RESPONSE_CLOSE, NULL);
 
         g_signal_connect (configure_win, "response", (GCallback) gtk_widget_destroy, NULL);
