@@ -18,20 +18,18 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <pthread.h>
 #include <string.h>
+#include <glib.h>
 
 #include <audacious/debug.h>
 #include <audacious/i18n.h>
+#include <audacious/input.h>
 #include <audacious/plugin.h>
 
 #include "flacng.h"
 
 static FLAC__StreamDecoder *decoder;
 static callback_info *info;
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static int seek_value;
-static bool_t stop_flag = FALSE;
 
 static bool_t flac_init (void)
 {
@@ -119,12 +117,11 @@ static void squeeze_audio(int32_t* src, void* dst, unsigned count, unsigned res)
             break;
 
         default:
-            FLACNG_ERROR("Can not convert to %d bps\n", res);
+            FLACNG_ERROR("Can not convert to %u bps\n", res);
     }
 }
 
-static bool_t flac_play (InputPlayback * playback, const char * filename,
- VFSFile * file, int start_time, int stop_time, bool_t pause)
+static bool_t flac_play (const char * filename, VFSFile * file)
 {
     if (!file)
         return FALSE;
@@ -141,61 +138,27 @@ static bool_t flac_play (InputPlayback * playback, const char * filename,
         goto ERR_NO_CLOSE;
     }
 
-    if ((play_buffer = malloc (BUFFER_SIZE_BYTE)) == NULL)
-    {
-        FLACNG_ERROR("Could not allocate conversion buffer\n");
-        error = TRUE;
-        goto ERR_NO_CLOSE;
-    }
+    play_buffer = g_malloc (BUFFER_SIZE_BYTE);
 
-    if (! playback->output->open_audio (SAMPLE_FMT (info->bits_per_sample),
+    if (! aud_input_open_audio (SAMPLE_FMT (info->bits_per_sample),
         info->sample_rate, info->channels))
     {
         error = TRUE;
         goto ERR_NO_CLOSE;
     }
 
-    if (pause)
-        playback->output->pause (TRUE);
+    aud_input_set_bitrate(info->bitrate);
 
-    seek_value = (start_time > 0) ? start_time : -1;
-    stop_flag = FALSE;
-
-    playback->set_params(playback, info->bitrate, info->sample_rate, info->channels);
-    playback->set_pb_ready(playback);
-    playback->set_gain_from_playlist(playback);
-
-    int64_t samples_remaining = INT64_MAX;
-    if (start_time >= 0 && stop_time >= 0)
-        samples_remaining = (int64_t) (stop_time - start_time) *
-         info->sample_rate / 1000 * info->channels;
-
-    while (samples_remaining && FLAC__stream_decoder_get_state(decoder) !=
-     FLAC__STREAM_DECODER_END_OF_STREAM)
+    while (FLAC__stream_decoder_get_state(decoder) != FLAC__STREAM_DECODER_END_OF_STREAM)
     {
-        pthread_mutex_lock (& mutex);
-
-        if (stop_flag)
-        {
-            pthread_mutex_unlock (& mutex);
+        if (aud_input_check_stop ())
             break;
-        }
 
+        int seek_value = aud_input_check_seek ();
         if (seek_value >= 0)
-        {
-            playback->output->flush (seek_value);
             FLAC__stream_decoder_seek_absolute (decoder, (int64_t)
              seek_value * info->sample_rate / 1000);
 
-            if (stop_time >= 0)
-                samples_remaining = (int64_t) (stop_time - seek_value) *
-                 info->sample_rate / 1000 * info->channels;
-
-            seek_value = -1;
-        }
-
-        pthread_mutex_unlock (& mutex);
-
         /* Try to decode a single frame of audio */
         if (FLAC__stream_decoder_process_single(decoder) == FALSE)
         {
@@ -204,23 +167,14 @@ static bool_t flac_play (InputPlayback * playback, const char * filename,
             break;
         }
 
-        if (info->buffer_used >= samples_remaining)
-            info->buffer_used = samples_remaining;
-
         squeeze_audio(info->output_buffer, play_buffer, info->buffer_used, info->bits_per_sample);
-        playback->output->write_audio(play_buffer, info->buffer_used * SAMPLE_SIZE(info->bits_per_sample));
-
-        samples_remaining -= info->buffer_used;
+        aud_input_write_audio(play_buffer, info->buffer_used * SAMPLE_SIZE(info->bits_per_sample));
 
         reset_info(info);
     }
 
-    pthread_mutex_lock (& mutex);
-    stop_flag = TRUE;
-    pthread_mutex_unlock (& mutex);
-
 ERR_NO_CLOSE:
-    free (play_buffer);
+    g_free (play_buffer);
     reset_info(info);
 
     if (FLAC__stream_decoder_flush(decoder) == FALSE)
@@ -229,42 +183,6 @@ ERR_NO_CLOSE:
     return ! error;
 }
 
-static void flac_stop(InputPlayback *playback)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (!stop_flag)
-    {
-        stop_flag = TRUE;
-        playback->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
-static void flac_pause(InputPlayback *playback, bool_t pause)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (!stop_flag)
-        playback->output->pause(pause);
-
-    pthread_mutex_unlock (& mutex);
-}
-
-static void flac_seek (InputPlayback * playback, int time)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (!stop_flag)
-    {
-        seek_value = time;
-        playback->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
 static const char flac_about[] =
  N_("Original code by\n"
     "Ralf Ertzinger <ralf@skytale.net>\n\n"
@@ -280,9 +198,6 @@ AUD_INPUT_PLUGIN
     .init = flac_init,
     .cleanup = flac_cleanup,
     .play = flac_play,
-    .stop = flac_stop,
-    .pause = flac_pause,
-    .mseek = flac_seek,
     .probe_for_tuple = flac_probe_for_tuple,
     .is_our_file_from_vfs = flac_is_our_fd,
     .extensions = flac_fmts,
