@@ -20,7 +20,10 @@
 #ifndef GTKUI_H
 #define GTKUI_H
 
+#include <stdint.h>
+
 #include <gtk/gtk.h>
+#include <audacious/types.h>
 
 /* menus.c */
 GtkWidget * make_menu_bar (GtkAccelGroup * accel);
@@ -29,16 +32,20 @@ GtkWidget * make_menu_rclick (GtkAccelGroup * accel);
 GtkWidget * make_menu_tab (GtkAccelGroup * accel);
 extern int menu_tab_playlist_id;
 
+/* settings.c */
+extern const PluginPreferences gtkui_prefs;
+
 /* ui_gtk.c */
 void set_ab_repeat_a (void);
 void set_ab_repeat_b (void);
 void clear_ab_repeat (void);
-void show_menu (gboolean show);
-void show_infoarea (gboolean show);
-void show_infoarea_vis (gboolean show);
-void show_statusbar (gboolean show);
-void popup_menu_rclick (guint button, guint32 time);
-void popup_menu_tab (guint button, guint32 time, int playlist);
+void show_hide_menu (void);
+void show_hide_infoarea (void);
+void show_hide_infoarea_vis (void);
+void show_hide_statusbar (void);
+void popup_menu_rclick (unsigned button, uint32_t time);
+void popup_menu_tab (unsigned button, uint32_t time, int playlist);
 void activate_search_tool (void);
+void update_step_size (void);
 
 #endif
