@@ -29,9 +29,9 @@
 #include <audacious/plugin.h>
 #include <audacious/plugins.h>
 #include <audacious/misc.h>
+#include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 #include <libaudgui/libaudgui.h>
-#include <libaudgui/libaudgui-gtk.h>
 
 #include "gtkui.h"
 #include "layout.h"
@@ -41,17 +41,20 @@
 #include "ui_statusbar.h"
 #include "playlist_util.h"
 
-static const gchar * const gtkui_defaults[] = {
+static const char * const gtkui_defaults[] = {
  "infoarea_show_vis", "TRUE",
  "infoarea_visible", "TRUE",
  "menu_visible", "TRUE",
+ "playlist_tabs_visible", "TRUE",
  "statusbar_visible", "TRUE",
+ "entry_count_visible", "FALSE",
  "close_button_visible", "TRUE",
 
  "autoscroll", "TRUE",
  "playlist_columns", "title artist album queued length",
  "playlist_headers", "TRUE",
  "show_remaining_time", "FALSE",
+ "step_size", "5",
 
  "player_x", "-1000",
  "player_y", "-1000",
@@ -64,8 +67,8 @@ static const gchar * const gtkui_defaults[] = {
 static PluginHandle * search_tool;
 
 static GtkWidget *volume;
-static gboolean volume_slider_is_moving = FALSE;
-static guint update_volume_timeout_source = 0;
+static bool_t volume_slider_is_moving = FALSE;
+static unsigned update_volume_timeout_source = 0;
 static gulong volume_change_handler_id;
 
 static GtkAccelGroup * accel;
@@ -77,39 +80,30 @@ static GtkToolItem * menu_button, * search_button, * button_play, * button_stop,
 static GtkWidget * slider, * label_time;
 static GtkWidget * menu_main, * menu_rclick, * menu_tab;
 
-static GtkWidget * error_win = NULL;
+static bool_t slider_is_moving = FALSE;
+static int slider_seek_time = -1;
+static unsigned delayed_title_change_source = 0;
+static unsigned update_song_timeout_source = 0;
 
-static gboolean slider_is_moving = FALSE;
-static gint slider_seek_time = -1;
-static guint delayed_title_change_source = 0;
-static guint update_song_timeout_source = 0;
-
-static gboolean init (void);
+static bool_t init (void);
 static void cleanup (void);
-static void ui_show (gboolean show);
-static gboolean ui_is_shown (void);
-static gboolean ui_is_focused (void);
-static void ui_show_error (const gchar * text);
+static void ui_show (bool_t show);
 
 AUD_IFACE_PLUGIN
 (
     .name = N_("GTK Interface"),
     .domain = PACKAGE,
+    .prefs = & gtkui_prefs,
     .init = init,
     .cleanup = cleanup,
     .show = ui_show,
-    .is_shown = ui_is_shown,
-    .is_focused = ui_is_focused,
-    .show_error = ui_show_error,
-    .show_filebrowser = audgui_run_filebrowser,
-    .show_jump_to_track = audgui_jump_to_track,
-    .run_gtk_plugin = (void (*) (void *, const gchar *)) layout_add,
+    .run_gtk_plugin = (void (*) (void *, const char *)) layout_add,
     .stop_gtk_plugin = (void (*) (void *)) layout_remove,
 )
 
 static void save_window_size (void)
 {
-    gint x, y, w, h;
+    int x, y, w, h;
     gtk_window_get_position ((GtkWindow *) window, & x, & y);
     gtk_window_get_size ((GtkWindow *) window, & w, & h);
 
@@ -121,10 +115,10 @@ static void save_window_size (void)
 
 static void restore_window_size (void)
 {
-    gint x = aud_get_int ("gtkui", "player_x");
-    gint y = aud_get_int ("gtkui", "player_y");
-    gint w = aud_get_int ("gtkui", "player_width");
-    gint h = aud_get_int ("gtkui", "player_height");
+    int x = aud_get_int ("gtkui", "player_x");
+    int y = aud_get_int ("gtkui", "player_y");
+    int w = aud_get_int ("gtkui", "player_width");
+    int h = aud_get_int ("gtkui", "player_height");
 
     gtk_window_set_default_size ((GtkWindow *) window, w, h);
 
@@ -132,9 +126,9 @@ static void restore_window_size (void)
         gtk_window_move ((GtkWindow *) window, x, y);
 }
 
-static gboolean window_delete()
+static bool_t window_delete()
 {
-    gboolean handle = FALSE;
+    bool_t handle = FALSE;
 
     hook_call("window close", &handle);
 
@@ -182,7 +176,7 @@ void clear_ab_repeat (void)
     aud_drct_set_ab_repeat (-1, -1);
 }
 
-static gboolean title_change_cb (void)
+static bool_t title_change_cb (void)
 {
     if (delayed_title_change_source)
     {
@@ -194,10 +188,9 @@ static gboolean title_change_cb (void)
     {
         if (aud_drct_get_ready ())
         {
-            gchar * title = aud_drct_get_title ();
-            gchar * title_s = g_strdup_printf (_("%s - Audacious"), title);
+            char * title = aud_drct_get_title ();
+            SPRINTF (title_s, _("%s - Audacious"), title);
             gtk_window_set_title ((GtkWindow *) window, title_s);
-            g_free (title_s);
             str_unref (title);
         }
         else
@@ -209,48 +202,24 @@ static gboolean title_change_cb (void)
     return FALSE;
 }
 
-static void ui_show (gboolean show)
+static void ui_show (bool_t show)
 {
     if (show)
     {
-        if (! ui_is_shown ())
+        if (! gtk_widget_get_visible (window))
             restore_window_size ();
 
         gtk_window_present ((GtkWindow *) window);
-
-        /* turn visualization back on if necessary */
-        ui_infoarea_show_vis (aud_get_bool ("gtkui", "infoarea_show_vis"));
     }
     else
     {
-        if (ui_is_shown ())
+        if (gtk_widget_get_visible (window))
             save_window_size ();
 
         gtk_widget_hide (window);
-
-        /* turn visualization off to reduce CPU usage */
-        ui_infoarea_show_vis (FALSE);
     }
-}
 
-static gboolean ui_is_shown (void)
-{
-    return gtk_widget_get_visible (window);
-}
-
-static gboolean ui_is_focused (void)
-{
-/* gtk_window_is_active() is too unreliable, unfortunately. --jlindgren */
-#if 0
-    return gtk_window_is_active ((GtkWindow *) window);
-#else
-    return ui_is_shown ();
-#endif
-}
-
-static void ui_show_error (const gchar * text)
-{
-    audgui_simple_message (& error_win, GTK_MESSAGE_ERROR, _("Error"), _(text));
+    show_hide_infoarea_vis ();
 }
 
 static void append_str (char * buf, int bufsize, const char * str)
@@ -260,20 +229,12 @@ static void append_str (char * buf, int bufsize, const char * str)
 
 static void append_time_str (char * buf, int bufsize, int time)
 {
-    time /= 1000;
-
-    if (time < 3600)
-        snprintf (buf + strlen (buf), bufsize - strlen (buf),
-         aud_get_bool (NULL, "leading_zero") ? "%02d:%02d" : "%d:%02d",
-         time / 60, time % 60);
-    else
-        snprintf (buf + strlen (buf), bufsize - strlen (buf), "%d:%02d:%02d",
-         time / 3600, (time / 60) % 60, time % 60);
+    audgui_format_time (buf + strlen (buf), bufsize - strlen (buf), time);
 }
 
-static void set_time_label (gint time, gint len)
+static void set_time_label (int time, int len)
 {
-    gchar s[128] = "<b>";
+    char s[128] = "<b>";
 
     if (len && aud_get_bool ("gtkui", "show_remaining_time"))
         append_time_str (s, sizeof s, len - time);
@@ -302,23 +263,26 @@ static void set_time_label (gint time, gint len)
     }
 
     append_str (s, sizeof s, "</b>");
-    gtk_label_set_markup ((GtkLabel *) label_time, s);
+
+    /* only update label if necessary */
+    if (strcmp (gtk_label_get_label ((GtkLabel *) label_time), s))
+        gtk_label_set_markup ((GtkLabel *) label_time, s);
 }
 
-static void set_slider (gint time)
+static void set_slider (int time)
 {
     gtk_range_set_value ((GtkRange *) slider, time);
 }
 
-static gboolean time_counter_cb (void)
+static bool_t time_counter_cb (void)
 {
     if (slider_is_moving)
         return TRUE;
 
     slider_seek_time = -1;  // delayed reset to avoid seeking twice
 
-    gint time = aud_drct_get_time ();
-    gint length = aud_drct_get_length ();
+    int time = aud_drct_get_time ();
+    int length = aud_drct_get_length ();
 
     if (length > 0)
         set_slider (time);
@@ -328,9 +292,9 @@ static gboolean time_counter_cb (void)
     return TRUE;
 }
 
-static void do_seek (gint time)
+static void do_seek (int time)
 {
-    gint length = aud_drct_get_length ();
+    int length = aud_drct_get_length ();
     time = CLAMP (time, 0, length);
 
     set_slider (time);
@@ -347,11 +311,11 @@ static void do_seek (gint time)
     }
 }
 
-static gboolean ui_slider_change_value_cb (GtkRange * range,
- GtkScrollType scroll, gdouble value)
+static bool_t ui_slider_change_value_cb (GtkRange * range,
+ GtkScrollType scroll, double value)
 {
-    gint length = aud_drct_get_length ();
-    gint time = CLAMP ((gint) value, 0, length);
+    int length = aud_drct_get_length ();
+    int time = CLAMP ((int) value, 0, length);
 
     set_time_label (time, length);
 
@@ -363,13 +327,13 @@ static gboolean ui_slider_change_value_cb (GtkRange * range,
     return FALSE;
 }
 
-static gboolean ui_slider_button_press_cb(GtkWidget * widget, GdkEventButton * event, gpointer user_data)
+static bool_t ui_slider_button_press_cb(GtkWidget * widget, GdkEventButton * event, void * user_data)
 {
     slider_is_moving = TRUE;
     return FALSE;
 }
 
-static gboolean ui_slider_button_release_cb(GtkWidget * widget, GdkEventButton * event, gpointer user_data)
+static bool_t ui_slider_button_release_cb(GtkWidget * widget, GdkEventButton * event, void * user_data)
 {
     if (slider_seek_time != -1)
         do_seek (slider_seek_time);
@@ -378,33 +342,33 @@ static gboolean ui_slider_button_release_cb(GtkWidget * widget, GdkEventButton *
     return FALSE;
 }
 
-static gboolean ui_volume_value_changed_cb(GtkButton * button, gdouble volume, gpointer user_data)
+static bool_t ui_volume_value_changed_cb(GtkButton * button, double volume, void * user_data)
 {
-    aud_drct_set_volume((gint) volume, (gint) volume);
+    aud_drct_set_volume((int) volume, (int) volume);
 
     return TRUE;
 }
 
-static void ui_volume_pressed_cb(GtkButton *button, gpointer user_data)
+static void ui_volume_pressed_cb(GtkButton *button, void * user_data)
 {
     volume_slider_is_moving = TRUE;
 }
 
-static void ui_volume_released_cb(GtkButton *button, gpointer user_data)
+static void ui_volume_released_cb(GtkButton *button, void * user_data)
 {
     volume_slider_is_moving = FALSE;
 }
 
-static gboolean ui_volume_slider_update(gpointer data)
+static bool_t ui_volume_slider_update(void * data)
 {
-    gint volume;
+    int volume;
 
     if (volume_slider_is_moving || data == NULL)
         return TRUE;
 
     aud_drct_get_volume_main(&volume);
 
-    if (volume == (gint) gtk_scale_button_get_value(GTK_SCALE_BUTTON(data)))
+    if (volume == (int) gtk_scale_button_get_value(GTK_SCALE_BUTTON(data)))
         return TRUE;
 
     g_signal_handler_block(data, volume_change_handler_id);
@@ -414,7 +378,7 @@ static gboolean ui_volume_slider_update(gpointer data)
     return TRUE;
 }
 
-static void set_slider_length (gint length)
+static void set_slider_length (int length)
 {
     if (length > 0)
     {
@@ -425,10 +389,16 @@ static void set_slider_length (gint length)
         gtk_widget_hide (slider);
 }
 
+void update_step_size (void)
+{
+    double step_size = aud_get_double ("gtkui", "step_size");
+    gtk_range_set_increments ((GtkRange *) slider, step_size * 1000, step_size * 1000);
+}
+
 static void pause_cb (void)
 {
-    gtk_tool_button_set_stock_id ((GtkToolButton *) button_play,
-     aud_drct_get_paused () ? GTK_STOCK_MEDIA_PLAY : GTK_STOCK_MEDIA_PAUSE);
+    gtk_tool_button_set_icon_name ((GtkToolButton *) button_play,
+     aud_drct_get_paused () ? "media-playback-start" : "media-playback-pause");
 }
 
 static void ui_playback_begin (void)
@@ -475,39 +445,32 @@ static void ui_playback_stop (void)
     delayed_title_change_source = g_idle_add ((GSourceFunc) title_change_cb,
      NULL);
 
-    gtk_tool_button_set_stock_id ((GtkToolButton *) button_play, GTK_STOCK_MEDIA_PLAY);
+    gtk_tool_button_set_icon_name ((GtkToolButton *) button_play, "media-playback-start");
     gtk_widget_set_sensitive ((GtkWidget *) button_stop, FALSE);
     gtk_widget_hide (slider);
     gtk_widget_hide (label_time);
 }
 
 static GtkToolItem * toolbar_button_add (GtkWidget * toolbar,
- void (* callback) (void), const gchar * stock_id)
+ void (* callback) (void), const char * icon)
 {
-    GtkToolItem * item = gtk_tool_button_new_from_stock (stock_id);
+    GtkToolItem * item = gtk_tool_button_new (NULL, NULL);
+    gtk_tool_button_set_icon_name ((GtkToolButton *) item, icon);
     gtk_toolbar_insert ((GtkToolbar *) toolbar, item, -1);
-
     g_signal_connect (item, "clicked", callback, NULL);
     return item;
 }
 
-static GtkToolItem * toggle_button_new (const gchar * icon, const gchar * alt,
+static GtkToolItem * toggle_button_new (const char * icon,
  void (* toggled) (GtkToggleToolButton * button))
 {
     GtkToolItem * item = gtk_toggle_tool_button_new ();
-
-    if (! alt)
-        gtk_tool_button_set_stock_id ((GtkToolButton *) item, icon);
-    else if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), icon))
-        gtk_tool_button_set_icon_name ((GtkToolButton *) item, icon);
-    else
-        gtk_tool_button_set_label ((GtkToolButton *) item, alt);
-
+    gtk_tool_button_set_icon_name ((GtkToolButton *) item, icon);
     g_signal_connect (item, "toggled", (GCallback) toggled, NULL);
     return item;
 }
 
-static GtkWidget *markup_label_new(const gchar * str)
+static GtkWidget *markup_label_new(const char * str)
 {
     GtkWidget *label = gtk_label_new(str);
     g_object_set(G_OBJECT(label), "use-markup", TRUE, NULL);
@@ -519,7 +482,7 @@ static void window_mapped_cb (GtkWidget * widget, void * unused)
     gtk_widget_grab_focus (playlist_get_treeview (aud_playlist_get_active ()));
 }
 
-static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, void * unused)
+static bool_t window_keypress_cb (GtkWidget * widget, GdkEventKey * event, void * unused)
 {
     switch (event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK))
     {
@@ -559,11 +522,11 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
             return TRUE;
         case GDK_KEY_Left:
             if (aud_drct_get_playing ())
-                do_seek (aud_drct_get_time () - 5000);
+                do_seek (aud_drct_get_time () - aud_get_double ("gtkui", "step_size") * 1000);
             return TRUE;
         case GDK_KEY_Right:
             if (aud_drct_get_playing ())
-                do_seek (aud_drct_get_time () + 5000);
+                do_seek (aud_drct_get_time () + aud_get_double ("gtkui", "step_size") * 1000);
             return TRUE;
         }
 
@@ -599,11 +562,11 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
         {
           case GDK_KEY_Left:
             if (aud_drct_get_playing ())
-                do_seek (aud_drct_get_time () - 5000);
+                do_seek (aud_drct_get_time () - aud_get_double ("gtkui", "step_size") * 1000);
             break;
           case GDK_KEY_Right:
             if (aud_drct_get_playing ())
-                do_seek (aud_drct_get_time () + 5000);
+                do_seek (aud_drct_get_time () + aud_get_double ("gtkui", "step_size") * 1000);
             break;
           default:
             return FALSE;
@@ -615,7 +578,7 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
     return TRUE;
 }
 
-static gboolean playlist_keypress_cb (GtkWidget * widget, GdkEventKey * event, void * unused)
+static bool_t playlist_keypress_cb (GtkWidget * widget, GdkEventKey * event, void * unused)
 {
     switch (event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK))
     {
@@ -680,7 +643,7 @@ static void toggle_search_tool (GtkToggleToolButton * button)
     aud_plugin_enable (search_tool, gtk_toggle_tool_button_get_active (button));
 }
 
-static gboolean search_tool_toggled (PluginHandle * plugin, void * unused)
+static bool_t search_tool_toggled (PluginHandle * plugin, void * unused)
 {
     gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) search_button,
      aud_plugin_get_enabled (plugin));
@@ -689,7 +652,7 @@ static gboolean search_tool_toggled (PluginHandle * plugin, void * unused)
 
 static void config_save (void)
 {
-    if (ui_is_shown ())
+    if (gtk_widget_get_visible (window))
         save_window_size ();
 
     layout_save ();
@@ -704,7 +667,7 @@ static void ui_hooks_associate(void)
     hook_associate ("playback pause", (HookFunction) pause_cb, NULL);
     hook_associate ("playback unpause", (HookFunction) pause_cb, NULL);
     hook_associate ("playback stop", (HookFunction) ui_playback_stop, NULL);
-    hook_associate("playlist update", ui_playlist_notebook_update, NULL);
+    hook_associate ("playlist update", ui_playlist_notebook_update, NULL);
     hook_associate ("playlist activate", ui_playlist_notebook_activate, NULL);
     hook_associate ("playlist set playing", ui_playlist_notebook_set_playing, NULL);
     hook_associate ("playlist position", ui_playlist_notebook_position, NULL);
@@ -730,15 +693,12 @@ static void ui_hooks_disassociate(void)
     hook_dissociate ("config save", (HookFunction) config_save);
 }
 
-static gboolean init (void)
+static bool_t init (void)
 {
     search_tool = aud_plugin_lookup_basename ("search-tool");
 
     aud_config_set_defaults ("gtkui", gtkui_defaults);
 
-    audgui_set_default_icon();
-    audgui_register_stock_icons();
-
     pw_col_init ();
 
     window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -764,7 +724,7 @@ static gboolean init (void)
     /* search button */
     if (search_tool)
     {
-        search_button = toggle_button_new (GTK_STOCK_FIND, NULL, toggle_search_tool);
+        search_button = toggle_button_new ("edit-find", toggle_search_tool);
         gtk_toolbar_insert ((GtkToolbar *) toolbar, search_button, -1);
         gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) search_button,
          aud_plugin_get_enabled (search_tool));
@@ -772,12 +732,12 @@ static gboolean init (void)
     }
 
     /* playback buttons */
-    toolbar_button_add (toolbar, button_open_pressed, GTK_STOCK_OPEN);
-    toolbar_button_add (toolbar, button_add_pressed, GTK_STOCK_ADD);
-    button_play = toolbar_button_add (toolbar, aud_drct_play_pause, GTK_STOCK_MEDIA_PLAY);
-    button_stop = toolbar_button_add (toolbar, aud_drct_stop, GTK_STOCK_MEDIA_STOP);
-    toolbar_button_add (toolbar, aud_drct_pl_prev, GTK_STOCK_MEDIA_PREVIOUS);
-    toolbar_button_add (toolbar, aud_drct_pl_next, GTK_STOCK_MEDIA_NEXT);
+    toolbar_button_add (toolbar, button_open_pressed, "document-open");
+    toolbar_button_add (toolbar, button_add_pressed, "list-add");
+    button_play = toolbar_button_add (toolbar, aud_drct_play_pause, "media-playback-start");
+    button_stop = toolbar_button_add (toolbar, aud_drct_stop, "media-playback-stop");
+    toolbar_button_add (toolbar, aud_drct_pl_prev, "media-skip-backward");
+    toolbar_button_add (toolbar, aud_drct_pl_next, "media-skip-forward");
 
     /* time slider and label */
     GtkToolItem * boxitem1 = gtk_tool_item_new ();
@@ -788,13 +748,14 @@ static gboolean init (void)
     gtk_container_add ((GtkContainer *) boxitem1, box1);
 
     slider = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, NULL);
-    gtk_range_set_increments ((GtkRange *) slider, 5000, 5000);
     gtk_scale_set_draw_value(GTK_SCALE(slider), FALSE);
     gtk_widget_set_size_request(slider, 120, -1);
     gtk_widget_set_valign (slider, GTK_ALIGN_CENTER);
     gtk_widget_set_can_focus(slider, FALSE);
     gtk_box_pack_start ((GtkBox *) box1, slider, TRUE, TRUE, 6);
 
+    update_step_size ();
+
     label_time = markup_label_new(NULL);
     gtk_box_pack_end ((GtkBox *) box1, label_time, FALSE, FALSE, 6);
 
@@ -802,9 +763,9 @@ static gboolean init (void)
     gtk_widget_set_no_show_all (label_time, TRUE);
 
     /* repeat and shuffle buttons */
-    button_repeat = toggle_button_new ("media-playlist-repeat", "RP", toggle_repeat);
+    button_repeat = toggle_button_new ("media-playlist-repeat", toggle_repeat);
     gtk_toolbar_insert ((GtkToolbar *) toolbar, button_repeat, -1);
-    button_shuffle = toggle_button_new ("media-playlist-shuffle", "SF", toggle_shuffle);
+    button_shuffle = toggle_button_new ("media-playlist-shuffle", toggle_shuffle);
     gtk_toolbar_insert ((GtkToolbar *) toolbar, button_shuffle, -1);
 
     /* volume button */
@@ -815,11 +776,12 @@ static gboolean init (void)
     gtk_container_add ((GtkContainer *) boxitem2, box2);
 
     volume = gtk_volume_button_new();
+    g_object_set ((GObject *) volume, "size", GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
     gtk_button_set_relief(GTK_BUTTON(volume), GTK_RELIEF_NONE);
     gtk_scale_button_set_adjustment(GTK_SCALE_BUTTON(volume), GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 100, 1, 5, 0)));
     gtk_widget_set_can_focus(volume, FALSE);
 
-    gint lvol = 0, rvol = 0;
+    int lvol = 0, rvol = 0;
     aud_drct_get_volume(&lvol, &rvol);
     gtk_scale_button_set_value(GTK_SCALE_BUTTON(volume), (lvol + rvol) / 2);
 
@@ -838,14 +800,9 @@ static gboolean init (void)
     gtk_box_pack_start ((GtkBox *) vbox, (GtkWidget *) UI_PLAYLIST_NOTEBOOK, TRUE, TRUE, 0);
 
     /* optional UI elements */
-    show_menu (aud_get_bool ("gtkui", "menu_visible"));
-    show_infoarea (aud_get_bool ("gtkui", "infoarea_visible"));
-
-    if (aud_get_bool ("gtkui", "statusbar_visible"))
-    {
-        statusbar = ui_statusbar_new ();
-        gtk_box_pack_end ((GtkBox *) vbox_outer, statusbar, FALSE, FALSE, 0);
-    }
+    show_hide_menu ();
+    show_hide_infoarea ();
+    show_hide_statusbar ();
 
     AUDDBG("hooks associate\n");
     ui_hooks_associate();
@@ -889,9 +846,6 @@ static gboolean init (void)
 
 static void cleanup (void)
 {
-    if (error_win)
-        gtk_widget_destroy (error_win);
-
     if (menu_main)
         gtk_widget_destroy (menu_main);
 
@@ -921,7 +875,6 @@ static void cleanup (void)
     if (search_tool)
         aud_plugin_remove_watch (search_tool, search_tool_toggled, NULL);
 
-    pw_col_cleanup ();
     gtk_widget_destroy (window);
     layout_cleanup ();
 }
@@ -952,11 +905,9 @@ static void menu_hide_cb (void)
     gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) menu_button, FALSE);
 }
 
-void show_menu (gboolean show)
+void show_hide_menu (void)
 {
-    aud_set_bool ("gtkui", "menu_visible", show);
-
-    if (show)
+    if (aud_get_bool ("gtkui", "menu_visible"))
     {
         /* remove menu button from toolbar and show menu bar */
         if (menu_main)
@@ -989,7 +940,8 @@ void show_menu (gboolean show)
 
         if (! menu_button)
         {
-            menu_button = gtk_toggle_tool_button_new_from_stock (AUD_STOCK_AUDACIOUS);
+            menu_button = gtk_toggle_tool_button_new ();
+            gtk_tool_button_set_icon_name ((GtkToolButton *) menu_button, "audacious");
             g_signal_connect (menu_button, "destroy", (GCallback)
              gtk_widget_destroyed, & menu_button);
             gtk_widget_show ((GtkWidget *) menu_button);
@@ -999,9 +951,9 @@ void show_menu (gboolean show)
     }
 }
 
-void show_infoarea (gboolean show)
+void show_hide_infoarea (void)
 {
-    aud_set_bool ("gtkui", "infoarea_visible", show);
+    bool_t show = aud_get_bool ("gtkui", "infoarea_visible");
 
     if (show && ! infoarea)
     {
@@ -1010,9 +962,7 @@ void show_infoarea (gboolean show)
         gtk_box_pack_end ((GtkBox *) vbox, infoarea, FALSE, FALSE, 0);
         gtk_widget_show_all (infoarea);
 
-        /* only turn on visualization if interface is shown */
-        if (ui_is_shown ())
-            ui_infoarea_show_vis (aud_get_bool ("gtkui", "infoarea_show_vis"));
+        show_hide_infoarea_vis ();
     }
 
     if (! show && infoarea)
@@ -1022,19 +972,21 @@ void show_infoarea (gboolean show)
     }
 }
 
-void show_infoarea_vis (gboolean show)
+void show_hide_infoarea_vis (void)
 {
-    aud_set_bool ("gtkui", "infoarea_show_vis", show);
-    ui_infoarea_show_vis (show);
+    /* only turn on visualization if interface is shown */
+    ui_infoarea_show_vis (gtk_widget_get_visible (window) && aud_get_bool
+     ("gtkui", "infoarea_show_vis"));
 }
 
-void show_statusbar (gboolean show)
+void show_hide_statusbar (void)
 {
-    aud_set_bool ("gtkui", "statusbar_visible", show);
+    bool_t show = aud_get_bool ("gtkui", "statusbar_visible");
 
     if (show && ! statusbar)
     {
         statusbar = ui_statusbar_new ();
+        g_signal_connect (statusbar, "destroy", (GCallback) gtk_widget_destroyed, & statusbar);
         gtk_box_pack_end ((GtkBox *) vbox_outer, statusbar, FALSE, FALSE, 0);
         gtk_widget_show_all (statusbar);
     }
@@ -1046,13 +998,13 @@ void show_statusbar (gboolean show)
     }
 }
 
-void popup_menu_rclick (guint button, guint32 time)
+void popup_menu_rclick (unsigned button, uint32_t time)
 {
     gtk_menu_popup ((GtkMenu *) menu_rclick, NULL, NULL, NULL, NULL, button,
      time);
 }
 
-void popup_menu_tab (guint button, guint32 time, int playlist)
+void popup_menu_tab (unsigned button, uint32_t time, int playlist)
 {
     menu_tab_playlist_id = aud_playlist_get_unique_id (playlist);
     gtk_menu_popup ((GtkMenu *) menu_tab, NULL, NULL, NULL, NULL, button, time);
