@@ -27,6 +27,7 @@
 #include <audacious/misc.h>
 #include <audacious/playlist.h>
 #include <audacious/plugin.h>
+#include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 #include <libaudgui/list.h>
 #include <libaudgui/libaudgui.h>
@@ -37,10 +38,10 @@
 #include "playlist_util.h"
 
 static GtkWidget * notebook = NULL;
-static gint highlighted = -1;
+static int highlighted = -1;
 
-static gint switch_handler = 0;
-static gint reorder_handler = 0;
+static int switch_handler = 0;
+static int reorder_handler = 0;
 
 static void add_button_cb (GtkButton * button, void * unused)
 {
@@ -52,8 +53,8 @@ static void make_add_button (GtkWidget * notebook)
 {
     GtkWidget * button = gtk_button_new ();
     gtk_button_set_relief ((GtkButton *) button, GTK_RELIEF_NONE);
-    gtk_container_add ((GtkContainer *) button, gtk_image_new_from_stock
-     (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+    gtk_container_add ((GtkContainer *) button, gtk_image_new_from_icon_name
+     ("list-add", GTK_ICON_SIZE_MENU));
     gtk_widget_set_can_focus (button, FALSE);
 
     g_signal_connect (button, "clicked", (GCallback) add_button_cb, NULL);
@@ -67,15 +68,15 @@ static void close_button_cb (GtkWidget * button, void * id)
     audgui_confirm_playlist_delete (aud_playlist_by_unique_id (GPOINTER_TO_INT (id)));
 }
 
-static GtkWidget * make_close_button (GtkWidget * ebox, gint list)
+static GtkWidget * make_close_button (GtkWidget * ebox, int list)
 {
     GtkWidget * button = gtk_button_new ();
-    GtkWidget * image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+    GtkWidget * image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
     gtk_button_set_image ((GtkButton *) button, image);
     gtk_button_set_relief ((GtkButton *) button, GTK_RELIEF_NONE);
     gtk_button_set_focus_on_click ((GtkButton *) button, FALSE);
     gtk_widget_set_name (button, "gtkui-tab-close-button");
-    g_object_set_data ((GObject *) ebox, "close_button", button);
+
     g_signal_connect (button, "clicked", (GCallback) close_button_cb,
      GINT_TO_POINTER (aud_playlist_get_unique_id (list)));
 
@@ -96,31 +97,11 @@ static GtkWidget * make_close_button (GtkWidget * ebox, gint list)
      GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
     g_object_unref (provider);
 
-    if (aud_get_bool ("gtkui", "close_button_visible"))
-        gtk_widget_show (button);
+    gtk_widget_show (button);
 
     return button;
 }
 
-void show_close_buttons (gboolean show)
-{
-    aud_set_bool ("gtkui", "close_button_visible", show);
-
-    gint pages = gtk_notebook_get_n_pages ((GtkNotebook *) notebook);
-
-    for (gint i = 0; i < pages; i ++)
-    {
-        GtkWidget * page = gtk_notebook_get_nth_page ((GtkNotebook *) notebook, i);
-        GtkWidget * ebox = gtk_notebook_get_tab_label ((GtkNotebook *) notebook, page);
-        GtkWidget * button = g_object_get_data ((GObject *) ebox, "close_button");
-
-        if (show)
-            gtk_widget_show (button);
-        else
-            gtk_widget_hide (button);
-    }
-}
-
 GtkNotebook *ui_playlist_get_notebook(void)
 {
     return GTK_NOTEBOOK(notebook);
@@ -134,23 +115,23 @@ static void save_column_widths ()
     char * widths, * expand;
     ui_playlist_widget_get_column_widths (treeview, & widths, & expand);
 
-    aud_set_string ("gtkui", "column_widths", widths);
-    aud_set_string ("gtkui", "column_expand", expand);
+    aud_set_str ("gtkui", "column_widths", widths);
+    aud_set_str ("gtkui", "column_expand", expand);
 
-    free (widths);
-    free (expand);
+    str_unref (widths);
+    str_unref (expand);
 }
 
 static void apply_column_widths (GtkWidget * treeview)
 {
-    char * widths = aud_get_string ("gtkui", "column_widths");
-    char * expand = aud_get_string ("gtkui", "column_expand");
+    char * widths = aud_get_str ("gtkui", "column_widths");
+    char * expand = aud_get_str ("gtkui", "column_expand");
 
     if (widths && widths[0] && expand && expand[0])
         ui_playlist_widget_set_column_widths (treeview, widths, expand);
 
-    free (widths);
-    free (expand);
+    str_unref (widths);
+    str_unref (expand);
 }
 
 static void tab_title_reset(GtkWidget *ebox)
@@ -161,7 +142,7 @@ static void tab_title_reset(GtkWidget *ebox)
     gtk_widget_show(label);
 }
 
-static void tab_title_save(GtkEntry *entry, gpointer ebox)
+static void tab_title_save(GtkEntry *entry, void * ebox)
 {
     int id = GPOINTER_TO_INT (g_object_get_data ((GObject *) ebox, "playlist-id"));
     GtkWidget *label = g_object_get_data(G_OBJECT(ebox), "label");
@@ -171,7 +152,7 @@ static void tab_title_save(GtkEntry *entry, gpointer ebox)
     gtk_widget_show(label);
 }
 
-static gboolean tab_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+static bool_t tab_key_press_cb(GtkWidget *widget, GdkEventKey *event, void * user_data)
 {
     if (event->keyval == GDK_KEY_Escape)
         tab_title_reset(widget);
@@ -179,7 +160,7 @@ static gboolean tab_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
     return FALSE;
 }
 
-static gboolean tab_button_press_cb(GtkWidget *ebox, GdkEventButton *event, gpointer user_data)
+static bool_t tab_button_press_cb(GtkWidget *ebox, GdkEventButton *event, void * user_data)
 {
     int id = GPOINTER_TO_INT (g_object_get_data ((GObject *) ebox, "playlist-id"));
     int playlist = aud_playlist_by_unique_id (id);
@@ -196,7 +177,26 @@ static gboolean tab_button_press_cb(GtkWidget *ebox, GdkEventButton *event, gpoi
     return FALSE;
 }
 
-static void tab_changed (GtkNotebook * notebook, GtkWidget * page, gint
+static bool_t scroll_cb (GtkWidget * widget, GdkEventScroll * event)
+{
+    switch (event->direction)
+    {
+    case GDK_SCROLL_UP:
+    case GDK_SCROLL_LEFT:
+        aud_playlist_set_active (aud_playlist_get_active () - 1);
+        return TRUE;
+
+    case GDK_SCROLL_DOWN:
+    case GDK_SCROLL_RIGHT:
+        aud_playlist_set_active (aud_playlist_get_active () + 1);
+        return TRUE;
+
+    default:
+        return FALSE;
+    }
+}
+
+static void tab_changed (GtkNotebook * notebook, GtkWidget * page, int
  page_num, void * unused)
 {
     save_column_widths ();
@@ -205,27 +205,34 @@ static void tab_changed (GtkNotebook * notebook, GtkWidget * page, gint
     aud_playlist_set_active (page_num);
 }
 
-static void tab_reordered(GtkNotebook *notebook, GtkWidget *child, guint page_num, gpointer user_data)
+static void tab_reordered(GtkNotebook *notebook, GtkWidget *child, unsigned page_num, void * user_data)
 {
     GtkWidget * widget = g_object_get_data ((GObject *) child, "treeview");
     g_return_if_fail (widget);
     aud_playlist_reorder (ui_playlist_widget_get_playlist (widget), page_num, 1);
 }
 
-static GtkLabel *get_tab_label(gint playlist)
+static GtkLabel *get_tab_label(int playlist)
 {
     GtkWidget *page = gtk_notebook_get_nth_page(UI_PLAYLIST_NOTEBOOK, playlist);
     GtkWidget *ebox = gtk_notebook_get_tab_label(UI_PLAYLIST_NOTEBOOK, page);
     return GTK_LABEL(g_object_get_data(G_OBJECT(ebox), "label"));
 }
 
-static void set_tab_label (gint list, GtkLabel * label)
+static void set_tab_label (int list, GtkLabel * label)
 {
-    gchar * title = aud_playlist_get_title (list);
+    char * title = aud_playlist_get_title (list);
+
+    if (aud_get_bool ("gtkui", "entry_count_visible"))
+    {
+        char * temp = str_printf ("%s (%d)", title, aud_playlist_entry_count (list));
+        str_unref (title);
+        title = temp;
+    }
 
     if (list == aud_playlist_get_playing ())
     {
-        gchar * markup = g_markup_printf_escaped ("<b>%s</b>", title);
+        char * markup = g_markup_printf_escaped ("<b>%s</b>", title);
         gtk_label_set_markup (label, markup);
         g_free (markup);
     }
@@ -235,8 +242,14 @@ static void set_tab_label (gint list, GtkLabel * label)
     str_unref (title);
 }
 
-void ui_playlist_notebook_edit_tab_title (int playlist)
+void start_rename_playlist (int playlist)
 {
+    if (! gtk_notebook_get_show_tabs ((GtkNotebook *) notebook))
+    {
+        audgui_show_playlist_rename (playlist);
+        return;
+    }
+
     GtkWidget * page = gtk_notebook_get_nth_page (UI_PLAYLIST_NOTEBOOK, playlist);
     GtkWidget * ebox = gtk_notebook_get_tab_label (UI_PLAYLIST_NOTEBOOK, page);
 
@@ -244,7 +257,7 @@ void ui_playlist_notebook_edit_tab_title (int playlist)
     GtkWidget *entry = g_object_get_data(G_OBJECT(ebox), "entry");
     gtk_widget_hide(label);
 
-    gchar * title = aud_playlist_get_title (playlist);
+    char * title = aud_playlist_get_title (playlist);
     gtk_entry_set_text ((GtkEntry *) entry, title);
     str_unref (title);
     gtk_widget_grab_focus(entry);
@@ -252,12 +265,12 @@ void ui_playlist_notebook_edit_tab_title (int playlist)
     gtk_widget_show(entry);
 }
 
-void ui_playlist_notebook_create_tab(gint playlist)
+void ui_playlist_notebook_create_tab(int playlist)
 {
     GtkWidget *scrollwin, *treeview;
     GtkWidget *label, *entry, *ebox, *hbox;
     GtkAdjustment *vscroll;
-    gint position = aud_playlist_get_position (playlist);
+    int position = aud_playlist_get_position (playlist);
 
     scrollwin = gtk_scrolled_window_new(NULL, NULL);
     vscroll = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrollwin));
@@ -286,7 +299,13 @@ void ui_playlist_notebook_create_tab(gint playlist)
     gtk_widget_show_all(ebox);
     gtk_widget_hide(entry);
 
-    gtk_box_pack_end ((GtkBox *) hbox, make_close_button (ebox, playlist), FALSE, FALSE, 0);
+    GtkWidget * button = NULL;
+
+    if (aud_get_bool ("gtkui", "close_button_visible"))
+    {
+        button = make_close_button (ebox, playlist);
+        gtk_box_pack_end ((GtkBox *) hbox, button, FALSE, FALSE, 0);
+    }
 
     g_object_set_data(G_OBJECT(ebox), "label", label);
     g_object_set_data(G_OBJECT(ebox), "entry", entry);
@@ -313,12 +332,23 @@ void ui_playlist_notebook_create_tab(gint playlist)
     g_signal_connect(entry, "activate", G_CALLBACK(tab_title_save), ebox);
     g_signal_connect_swapped (vscroll, "value-changed",
      G_CALLBACK(ui_playlist_widget_scroll), treeview);
+
+    /* we have to connect to "scroll-event" on the notebook, the tabs, AND the
+     * close buttons (sigh) */
+    gtk_widget_add_events (ebox, GDK_SCROLL_MASK);
+    g_signal_connect (ebox, "scroll-event", (GCallback) scroll_cb, NULL);
+
+    if (button)
+    {
+        gtk_widget_add_events (button, GDK_SCROLL_MASK);
+        g_signal_connect (button, "scroll-event", (GCallback) scroll_cb, NULL);
+    }
 }
 
 void ui_playlist_notebook_populate(void)
 {
-    gint playlists = aud_playlist_count();
-    gint count;
+    int playlists = aud_playlist_count();
+    int count;
 
     for (count = 0; count < playlists; count++)
         ui_playlist_notebook_create_tab(count);
@@ -345,7 +375,7 @@ void ui_playlist_notebook_empty (void)
         g_signal_handler_disconnect (notebook, reorder_handler);
     reorder_handler = 0;
 
-    gint n_pages = gtk_notebook_get_n_pages ((GtkNotebook *) notebook);
+    int n_pages = gtk_notebook_get_n_pages ((GtkNotebook *) notebook);
     while (n_pages)
         gtk_notebook_remove_page ((GtkNotebook *) notebook, -- n_pages);
 }
@@ -357,15 +387,15 @@ static void add_remove_pages (void)
 
     save_column_widths ();
 
-    gint lists = aud_playlist_count ();
-    gint pages = gtk_notebook_get_n_pages ((GtkNotebook *) notebook);
+    int lists = aud_playlist_count ();
+    int pages = gtk_notebook_get_n_pages ((GtkNotebook *) notebook);
 
     /* scan through existing treeviews */
-    for (gint i = 0; i < pages; )
+    for (int i = 0; i < pages; )
     {
         GtkWidget * page = gtk_notebook_get_nth_page ((GtkNotebook *) notebook, i);
         GtkWidget * tree = g_object_get_data ((GObject *) page, "treeview");
-        gint tree_id = GPOINTER_TO_INT (g_object_get_data ((GObject *) tree, "playlist-id"));
+        int tree_id = GPOINTER_TO_INT (g_object_get_data ((GObject *) tree, "playlist-id"));
 
         /* do we have an orphaned treeview? */
         if (aud_playlist_by_unique_id (tree_id) < 0)
@@ -376,7 +406,7 @@ static void add_remove_pages (void)
         }
 
         /* do we have the right treeview? */
-        gint list_id = aud_playlist_get_unique_id (i);
+        int list_id = aud_playlist_get_unique_id (i);
 
         if (tree_id == list_id)
         {
@@ -386,9 +416,9 @@ static void add_remove_pages (void)
         }
 
         /* look for the right treeview */
-        gint found = FALSE;
+        int found = FALSE;
 
-        for (gint j = i + 1; j < pages; j ++)
+        for (int j = i + 1; j < pages; j ++)
         {
             page = gtk_notebook_get_nth_page ((GtkNotebook *) notebook, j);
             tree = g_object_get_data ((GObject *) page, "treeview");
@@ -423,28 +453,30 @@ static void add_remove_pages (void)
     apply_column_widths (playlist_get_treeview (active));
     gtk_notebook_set_current_page ((GtkNotebook *) notebook, active);
 
+    show_hide_playlist_tabs ();
+
     g_signal_handlers_unblock_by_func (notebook, (void *) tab_changed, NULL);
     g_signal_handlers_unblock_by_func (notebook, (void *) tab_reordered, NULL);
 }
 
 void ui_playlist_notebook_update (void * data, void * user)
 {
-    gint global_level = GPOINTER_TO_INT (data);
+    int global_level = GPOINTER_TO_INT (data);
 
     if (global_level == PLAYLIST_UPDATE_STRUCTURE)
         add_remove_pages ();
 
-    gint lists = aud_playlist_count ();
+    int lists = aud_playlist_count ();
 
-    for (gint list = 0; list < lists; list ++)
+    for (int list = 0; list < lists; list ++)
     {
         if (global_level >= PLAYLIST_UPDATE_METADATA)
             set_tab_label (list, get_tab_label (list));
 
         GtkWidget * treeview = playlist_get_treeview (list);
 
-        gint at, count;
-        gint level = aud_playlist_updated_range (list, & at, & count);
+        int at, count;
+        int level = aud_playlist_updated_range (list, & at, & count);
 
         if (level)
             ui_playlist_widget_update (treeview, level, at, count);
@@ -457,8 +489,8 @@ void ui_playlist_notebook_update (void * data, void * user)
 
 void ui_playlist_notebook_position (void * data, void * user)
 {
-    gint list = GPOINTER_TO_INT (data);
-    gint row = aud_playlist_get_position (list);
+    int list = GPOINTER_TO_INT (data);
+    int row = aud_playlist_get_position (list);
 
     if (aud_get_bool ("gtkui", "autoscroll"))
     {
@@ -479,18 +511,18 @@ void ui_playlist_notebook_activate (void * data, void * user)
 
 void ui_playlist_notebook_set_playing (void * data, void * user)
 {
-    gint new = aud_playlist_get_unique_id (aud_playlist_get_playing ());
+    int new = aud_playlist_get_unique_id (aud_playlist_get_playing ());
 
     if (highlighted == new)
         return;
 
-    gint pages = gtk_notebook_get_n_pages ((GtkNotebook *) notebook);
+    int pages = gtk_notebook_get_n_pages ((GtkNotebook *) notebook);
 
-    for (gint i = 0; i < pages; i ++)
+    for (int i = 0; i < pages; i ++)
     {
         GtkWidget * page = gtk_notebook_get_nth_page ((GtkNotebook *) notebook, i);
         GtkWidget * tree = g_object_get_data ((GObject *) page, "treeview");
-        gint tree_id = GPOINTER_TO_INT (g_object_get_data ((GObject *) tree, "playlist-id"));
+        int tree_id = GPOINTER_TO_INT (g_object_get_data ((GObject *) tree, "playlist-id"));
 
         if (tree_id == highlighted || tree_id == new)
             set_tab_label (i, get_tab_label (i));
@@ -514,19 +546,19 @@ GtkWidget * ui_playlist_notebook_new (void)
     gtk_notebook_set_scrollable ((GtkNotebook *) notebook, TRUE);
     make_add_button (notebook);
 
+    show_hide_playlist_tabs ();
+
     hook_associate ("config save", (HookFunction) save_column_widths, NULL);
 
+    gtk_widget_add_events (notebook, GDK_SCROLL_MASK);
+    g_signal_connect (notebook, "scroll-event", (GCallback) scroll_cb, NULL);
     g_signal_connect (notebook, "destroy", (GCallback) destroy_cb, NULL);
+
     return notebook;
 }
 
-void playlist_show_headers (gboolean show)
+void show_hide_playlist_tabs (void)
 {
-    gboolean old = aud_get_bool ("gtkui", "playlist_headers");
-    if (old == show)
-        return;
-
-    aud_set_bool ("gtkui", "playlist_headers", show);
-    ui_playlist_notebook_empty ();
-    ui_playlist_notebook_populate ();
+    gtk_notebook_set_show_tabs ((GtkNotebook *) notebook, aud_get_bool ("gtkui",
+     "playlist_tabs_visible") || aud_playlist_count () > 1);
 }
