@@ -47,6 +47,7 @@
 #include <audacious/misc.h>
 #include <audacious/plugin.h>
 #include <libaudcore/hook.h>
+#include <libaudgui/libaudgui.h>
 
 #include "plugin.h"
 #include "gui.h"
@@ -243,16 +244,16 @@ gboolean handle_keyevent (EVENT event)
     }
 
     /* Open Jump-To-File dialog */
-    if (event == EVENT_JUMP_TO_FILE)
+    if (event == EVENT_JUMP_TO_FILE && ! aud_headless_mode ())
     {
-        aud_interface_show_jump_to_track ();
+        audgui_jump_to_track ();
         return TRUE;
     }
 
     /* Toggle Windows */
-    if (event == EVENT_TOGGLE_WIN)
+    if (event == EVENT_TOGGLE_WIN && ! aud_headless_mode ())
     {
-        aud_interface_show (! (aud_interface_is_shown () && aud_interface_is_focused ()));
+        aud_interface_show (! aud_interface_is_shown ());
         return TRUE;
     }
 
@@ -301,8 +302,7 @@ void add_hotkey(HotkeyConfiguration** pphotkey, KeySym keysym, gint mask, gint t
     keycode = XKeysymToKeycode(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), keysym);
     if (keycode == 0) return;
     if (photkey->key) {
-        photkey->next = (HotkeyConfiguration*)
-            malloc(sizeof (HotkeyConfiguration));
+        photkey->next = g_new(HotkeyConfiguration, 1);
         photkey = photkey->next;
         *pphotkey = photkey;
         photkey->next = NULL;
@@ -360,8 +360,7 @@ void load_config (void)
         gchar *text = NULL;
 
         if (hotkey->key) {
-            hotkey->next = (HotkeyConfiguration*)
-                malloc(sizeof (HotkeyConfiguration));
+            hotkey->next = g_new(HotkeyConfiguration, 1);
             hotkey = hotkey->next;
             hotkey->next = NULL;
             hotkey->key = 0;
@@ -435,7 +434,7 @@ static void cleanup (void)
         HotkeyConfiguration * old;
         old = hotkey;
         hotkey = hotkey->next;
-        free(old);
+        g_free(old);
     }
     plugin_cfg.first.next = NULL;
     plugin_cfg.first.key = 0;
