@@ -17,7 +17,6 @@
  * the use of this software.
  */
 
-#include <dirent.h>
 #include <errno.h>
 #include <math.h>
 #include <stdio.h>
@@ -107,6 +106,12 @@ static ControlData * parse_control (const LADSPA_Descriptor * desc, int port)
     return control;
 }
 
+static void free_control (ControlData * control)
+{
+    g_free (control->name);
+    g_slice_free (ControlData, control);
+}
+
 static PluginData * open_plugin (const char * path, const LADSPA_Descriptor * desc)
 {
     const char * slash = strrchr (path, G_DIR_SEPARATOR);
@@ -127,7 +132,7 @@ static PluginData * open_plugin (const char * path, const LADSPA_Descriptor * de
         {
             ControlData * control = parse_control (desc, i);
             if (control)
-                index_append (plugin->controls, control);
+                index_insert (plugin->controls, -1, control);
         }
         else if (LADSPA_IS_PORT_AUDIO (desc->PortDescriptors[i]) &&
          LADSPA_IS_PORT_INPUT (desc->PortDescriptors[i]))
@@ -142,16 +147,8 @@ static PluginData * open_plugin (const char * path, const LADSPA_Descriptor * de
 
 static void close_plugin (PluginData * plugin)
 {
-    int count = index_count (plugin->controls);
-    for (int i = 0; i < count; i ++)
-    {
-        ControlData * control = index_get (plugin->controls, i);
-        g_free (control->name);
-        g_slice_free (ControlData, control);
-    }
-
     g_free (plugin->path);
-    index_free (plugin->controls);
+    index_free_full (plugin->controls, (IndexFreeFunc) free_control);
     g_array_free (plugin->in_ports, 1);
     g_array_free (plugin->out_ports, 1);
     g_slice_free (PluginData, plugin);
@@ -181,7 +178,7 @@ static void * open_module (const char * path)
     {
         PluginData * plugin = open_plugin (path, desc);
         if (plugin)
-            index_append (plugins, plugin);
+            index_insert (plugins, -1, plugin);
     }
 
     return handle;
@@ -189,28 +186,29 @@ static void * open_module (const char * path)
 
 static void open_modules_for_path (const char * path)
 {
-    DIR * folder = opendir (path);
+    GDir * folder = g_dir_open (path, 0, NULL);
     if (! folder)
     {
         fprintf (stderr, "ladspa: Failed to read folder %s: %s\n", path, strerror (errno));
         return;
     }
 
-    struct dirent * entry;
-    while ((entry = readdir (folder)))
+    const char * name;
+    while ((name = g_dir_read_name (folder)))
     {
-        if (entry->d_name[0] == '.' || ! str_has_suffix_nocase (entry->d_name, G_MODULE_SUFFIX))
+        if (! str_has_suffix_nocase (name, G_MODULE_SUFFIX))
             continue;
 
-        char filename[strlen (path) + strlen (entry->d_name) + 2];
-        snprintf (filename, sizeof filename, "%s" G_DIR_SEPARATOR_S "%s", path, entry->d_name);
-
+        char * filename = filename_build (path, name);
         void * handle = open_module (filename);
+
         if (handle)
-            index_append (modules, handle);
+            index_insert (modules, -1, handle);
+
+        str_unref (filename);
     }
 
-    closedir (folder);
+    g_dir_close (folder);
 }
 
 static void open_modules_for_paths (const char * paths)
@@ -234,17 +232,8 @@ static void open_modules (void)
 
 static void close_modules (void)
 {
-    int count = index_count (plugins);
-    for (int i = 0; i < count; i ++)
-        close_plugin (index_get (plugins, i));
-
-    index_delete (plugins, 0, count);
-
-    count = index_count (modules);
-    for (int i = 0; i < count; i ++)
-        g_module_close (index_get (modules, i));
-
-    index_delete (modules, 0, count);
+    index_delete_full (plugins, 0, -1, (IndexFreeFunc) close_plugin);
+    index_delete_full (modules, 0, -1, (IndexFreeFunc) g_module_close);
 }
 
 LoadedPlugin * enable_plugin_locked (PluginData * plugin)
@@ -269,7 +258,7 @@ LoadedPlugin * enable_plugin_locked (PluginData * plugin)
 
     loaded->settings_win = NULL;
 
-    index_append (loadeds, loaded);
+    index_insert (loadeds, -1, loaded);
     return loaded;
 }
 
@@ -304,6 +293,7 @@ static PluginData * find_plugin (const char * path, const char * label)
 static void save_enabled_to_config (void)
 {
     int count = index_count (loadeds);
+    int old_count = aud_get_int ("ladspa", "plugin_count");
     aud_set_int ("ladspa", "plugin_count", count);
 
     for (int i = 0; i < count; i ++)
@@ -312,20 +302,39 @@ static void save_enabled_to_config (void)
         char key[32];
 
         snprintf (key, sizeof key, "plugin%d_path", i);
-        aud_set_string ("ladspa", key, loaded->plugin->path);
+        aud_set_str ("ladspa", key, loaded->plugin->path);
 
         snprintf (key, sizeof key, "plugin%d_label", i);
-        aud_set_string ("ladspa", key, loaded->plugin->desc->Label);
+        aud_set_str ("ladspa", key, loaded->plugin->desc->Label);
+
+        snprintf (key, sizeof key, "plugin%d_controls", i);
 
         int ccount = index_count (loaded->plugin->controls);
+        double temp[ccount];
+
         for (int ci = 0; ci < ccount; ci ++)
-        {
-            snprintf (key, sizeof key, "plugin%d_control%d", i, ci);
-            aud_set_double ("ladspa", key, loaded->values[ci]);
-        }
+            temp[ci] = loaded->values[ci];
+
+        char * controls = double_array_to_str (temp, ccount);
+        aud_set_str ("ladspa", key, controls);
+        str_unref (controls);
 
         disable_plugin_locked (0);
     }
+
+    for (int i = count; i < old_count; i ++)
+    {
+        char key[32];
+
+        snprintf (key, sizeof key, "plugin%d_path", i);
+        aud_set_str ("ladspa", key, "");
+
+        snprintf (key, sizeof key, "plugin%d_label", i);
+        aud_set_str ("ladspa", key, "");
+
+        snprintf (key, sizeof key, "plugin%d_controls", i);
+        aud_set_str ("ladspa", key, "");
+    }
 }
 
 static void load_enabled_from_config (void)
@@ -337,26 +346,44 @@ static void load_enabled_from_config (void)
         char key[32];
 
         snprintf (key, sizeof key, "plugin%d_path", i);
-        char * path = aud_get_string ("ladspa", key);
+        char * path = aud_get_str ("ladspa", key);
 
         snprintf (key, sizeof key, "plugin%d_label", i);
-        char * label = aud_get_string ("ladspa", key);
+        char * label = aud_get_str ("ladspa", key);
 
         PluginData * plugin = find_plugin (path, label);
         if (plugin)
         {
             LoadedPlugin * loaded = enable_plugin_locked (plugin);
 
+            snprintf (key, sizeof key, "plugin%d_controls", i);
+
             int ccount = index_count (loaded->plugin->controls);
-            for (int ci = 0; ci < ccount; ci ++)
+            double temp[ccount];
+
+            char * controls = aud_get_str ("ladspa", key);
+
+            if (str_to_double_array (controls, temp, ccount))
             {
-                snprintf (key, sizeof key, "plugin%d_control%d", i, ci);
-                loaded->values[ci] = aud_get_double ("ladspa", key);
+                for (int ci = 0; ci < ccount; ci ++)
+                    loaded->values[ci] = temp[ci];
             }
+            else
+            {
+                /* migrate from old config format */
+                for (int ci = 0; ci < ccount; ci ++)
+                {
+                    snprintf (key, sizeof key, "plugin%d_control%d", i, ci);
+                    loaded->values[ci] = aud_get_double ("ladspa", key);
+                    aud_set_str ("ladspa", key, "");
+                }
+            }
+
+            str_unref (controls);
         }
 
-        g_free (path);
-        g_free (label);
+        str_unref (path);
+        str_unref (label);
     }
 }
 
@@ -370,7 +397,7 @@ static int init (void)
 
     aud_config_set_defaults ("ladspa", ladspa_defaults);
 
-    module_path = aud_get_string ("ladspa", "module_path");
+    module_path = aud_get_str ("ladspa", "module_path");
 
     open_modules ();
     load_enabled_from_config ();
@@ -386,8 +413,7 @@ static void cleanup (void)
 
     pthread_mutex_lock (& mutex);
 
-    aud_config_clear_section ("ladspa");
-    aud_set_string ("ladspa", "module_path", module_path);
+    aud_set_str ("ladspa", "module_path", module_path);
     save_enabled_to_config ();
     close_modules ();
 
@@ -398,7 +424,7 @@ static void cleanup (void)
     index_free (loadeds);
     loadeds = NULL;
 
-    g_free (module_path);
+    str_unref (module_path);
     module_path = NULL;
 
     pthread_mutex_unlock (& mutex);
@@ -411,8 +437,8 @@ static void set_module_path (GtkEntry * entry)
     save_enabled_to_config ();
     close_modules ();
 
-    g_free (module_path);
-    module_path = g_strdup (gtk_entry_get_text (entry));
+    str_unref (module_path);
+    module_path = str_get (gtk_entry_get_text (entry));
 
     open_modules ();
     load_enabled_from_config ();
@@ -492,7 +518,7 @@ static void configure_plugin (LoadedPlugin * loaded)
 
     snprintf (buf, sizeof buf, _("%s Settings"), plugin->desc->Name);
     loaded->settings_win = gtk_dialog_new_with_buttons (buf, (GtkWindow *)
-     config_win, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE,
+     config_win, GTK_DIALOG_DESTROY_WITH_PARENT, _("_Close"),
      GTK_RESPONSE_CLOSE, NULL);
     gtk_window_set_resizable ((GtkWindow *) loaded->settings_win, 0);
 
@@ -559,7 +585,7 @@ static void configure (void)
     }
 
     config_win = gtk_dialog_new_with_buttons (_("LADSPA Host Settings"), NULL,
-     0, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+     0, _("_Close"), GTK_RESPONSE_CLOSE, NULL);
     gtk_window_set_default_size ((GtkWindow *) config_win, 480, 360);
 
     GtkWidget * vbox = gtk_dialog_get_content_area ((GtkDialog *) config_win);
