@@ -7,17 +7,19 @@
 #include "modplugbmp.h"
 
 #include <fstream>
-#include <pthread.h>
 #include <stdint.h>
 #include <sys/types.h>
 #include <math.h>
 
+#include <glib.h>
+
 #include <libmodplug/stdafx.h>
 #include <libmodplug/sndfile.h>
 
 extern "C" {
 #include <libaudcore/audstrings.h>
 #include <audacious/i18n.h>
+#include <audacious/input.h>
 }
 
 #include "archive/open.h"
@@ -30,13 +32,11 @@ ModplugXMMS::ModplugXMMS()
 {
     memset (this, 0, sizeof (* this));
     mSoundFile = new CSoundFile;
-    pthread_mutex_init (& mutex, 0);
 }
 
 ModplugXMMS::~ModplugXMMS()
 {
     delete mSoundFile;
-    pthread_mutex_destroy (& mutex);
 }
 
 bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
@@ -78,7 +78,7 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
         if (magic[0] == '6' || magic[0] == '8')
             return true;
     }
-    if (magic[2] == 'C' && magic[3] == 'H' && isdigit(magic[0]) && isdigit(magic[1])) {
+    if (magic[2] == 'C' && magic[3] == 'H' && g_ascii_isdigit(magic[0]) && g_ascii_isdigit(magic[1])) {
         int nch = (magic[0] - '0') * 10 + (magic[1] - '0');
         if ((nch % 2 == 0) && nch >= 10)
             return true;
@@ -146,35 +146,16 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
     return false;
 }
 
-void ModplugXMMS::PlayLoop(InputPlayback *playback)
+void ModplugXMMS::PlayLoop()
 {
     uint32_t lLength;
 
-    pthread_mutex_lock (& mutex);
-    seek_time = -1;
-    stop_flag = FALSE;
-    playback->set_pb_ready (playback);
-    pthread_mutex_unlock (& mutex);
-
-    while (1)
+    while (! aud_input_check_stop ())
     {
-        pthread_mutex_lock (& mutex);
-
-        if (stop_flag)
-        {
-            pthread_mutex_unlock (& mutex);
-            break;
-        }
-
+        int seek_time = aud_input_check_seek ();
         if (seek_time != -1)
-        {
             mSoundFile->SetCurrentPos (seek_time * (int64_t)
              mSoundFile->GetMaxPosition () / (mSoundFile->GetSongTime () * 1000));
-            playback->output->flush (seek_time);
-            seek_time = -1;
-        }
-
-        pthread_mutex_unlock (& mutex);
 
         lLength = mSoundFile->Read (mBuffer, mBufSize);
 
@@ -210,13 +191,9 @@ void ModplugXMMS::PlayLoop(InputPlayback *playback)
             }
         }
 
-        playback->output->write_audio (mBuffer, mBufSize);
+        aud_input_write_audio (mBuffer, mBufSize);
     }
 
-    pthread_mutex_lock (& mutex);
-    stop_flag = TRUE;
-    pthread_mutex_unlock (& mutex);
-
     //Unload the file
     mSoundFile->Destroy();
     delete mArchive;
@@ -228,7 +205,7 @@ void ModplugXMMS::PlayLoop(InputPlayback *playback)
     }
 }
 
-bool ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
+bool ModplugXMMS::PlayFile(const string& aFilename)
 {
     //open and mmap the file
     mArchive = OpenArchive(aFilename);
@@ -310,56 +287,20 @@ bool ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
 
     Tuple* ti = GetSongTuple( aFilename );
     if ( ti ) {
-        ipb->set_tuple(ipb,ti);
+        aud_input_set_tuple(ti);
     }
 
-    ipb->set_params(ipb, mSoundFile->GetNumChannels() * 1000, mModProps.mFrequency, mModProps.mChannels);
+    aud_input_set_bitrate(mSoundFile->GetNumChannels() * 1000);
 
     int fmt = (mModProps.mBits == 16) ? FMT_S16_NE : FMT_U8;
-    if (! ipb->output->open_audio (fmt, mModProps.mFrequency, mModProps.mChannels))
+    if (! aud_input_open_audio(fmt, mModProps.mFrequency, mModProps.mChannels))
         return false;
 
-    this->PlayLoop(ipb);
+    PlayLoop();
 
     return true;
 }
 
-void ModplugXMMS::Stop (InputPlayback * playback)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (!stop_flag)
-    {
-        stop_flag = TRUE;
-        playback->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
-void ModplugXMMS::pause (InputPlayback * playback, bool_t pause)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (!stop_flag)
-        playback->output->pause(pause);
-
-    pthread_mutex_unlock (& mutex);
-}
-
-void ModplugXMMS::mseek (InputPlayback * playback, int time)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (!stop_flag)
-    {
-        seek_time = time;
-        playback->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
 Tuple* ModplugXMMS::GetSongTuple(const string& aFilename)
 {
     CSoundFile* lSoundFile;
@@ -403,16 +344,14 @@ Tuple* ModplugXMMS::GetSongTuple(const string& aFilename)
     case MOD_TYPE_PSM:  tmps = "Protracker Studio Module"; break;
     default:        tmps = "ModPlug unknown"; break;
     }
-    tuple_set_str(ti, FIELD_CODEC, NULL, tmps);
-    tuple_set_str(ti, FIELD_QUALITY, NULL, _("sequenced"));
-    tuple_set_int(ti, FIELD_LENGTH, NULL, lSoundFile->GetSongTime() * 1000);
+    tuple_set_str(ti, FIELD_CODEC, tmps);
+    tuple_set_str(ti, FIELD_QUALITY, _("sequenced"));
+    tuple_set_int(ti, FIELD_LENGTH, lSoundFile->GetSongTime() * 1000);
 
-    char *tmps2 = str_to_utf8(lSoundFile->GetTitle());
+    const char *tmps2 = lSoundFile->GetTitle();
     // Chop any leading spaces off. They are annoying in the playlist.
-    char *tmps3 = tmps2; // Make another pointer so tmps2 can still be free()d
-    while ( *tmps3 == ' ' ) tmps3++ ;
-    tuple_set_str(ti, FIELD_TITLE, NULL, tmps3);
-    free(tmps2);
+    while ( *tmps2 == ' ' ) tmps2++ ;
+    tuple_set_str(ti, FIELD_TITLE, tmps2);
 
     //unload the file
     lSoundFile->Destroy();
