@@ -7,7 +7,6 @@
 #ifndef __MODPLUGXMMS_CMODPLUGXMMS_H_INCLUDED__
 #define __MODPLUGXMMS_CMODPLUGXMMS_H_INCLUDED__
 
-#include <pthread.h>
 #include <string>
 
 extern "C" {
@@ -51,10 +50,7 @@ public:
 
     bool CanPlayFileFromVFS(const std::string& aFilename, VFSFile *file);
 
-    bool PlayFile(const std::string& aFilename, InputPlayback *data);
-    void Stop(InputPlayback *data);
-    void mseek (InputPlayback * playback, int time);
-    void pause (InputPlayback * playback, bool_t paused);
+    bool PlayFile(const std::string& aFilename);
 
     Tuple* GetSongTuple(const std::string& aFilename);
 
@@ -64,10 +60,6 @@ private:
     unsigned char*  mBuffer;
     uint32_t  mBufSize;
 
-    pthread_mutex_t mutex;
-    int seek_time;
-    bool stop_flag;
-
     ModplugSettings mModProps;
 
     uint32_t  mBufTime;     //milliseconds
@@ -77,7 +69,7 @@ private:
 
     float mPreampFactor;
 
-    void PlayLoop(InputPlayback *);
+    void PlayLoop();
 };
 
 #endif //included
