@@ -23,24 +23,9 @@ int CanPlayFileFromVFS(const char* aFilename, VFSFile *VFSFile)
     return 0;
 }
 
-bool_t PlayFile(InputPlayback * data, const char * filename, VFSFile * file, int start_time, int stop_time, bool_t pause)
+bool_t PlayFile(const char * filename, VFSFile * file)
 {
-    return gModplugXMMS.PlayFile(filename, data);
-}
-
-void Stop(InputPlayback *data)
-{
-    gModplugXMMS.Stop(data);
-}
-
-void Pause (InputPlayback * playback, bool_t pause)
-{
-    gModplugXMMS.pause (playback, pause);
-}
-
-void mseek (InputPlayback * playback, int time)
-{
-    gModplugXMMS.mseek (playback, time);
+    return gModplugXMMS.PlayFile(filename);
 }
 
 Tuple* GetSongTuple(const char* aFilename, VFSFile *fd)
