@@ -27,7 +27,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <strings.h>
 
 #include <glib.h>
 
@@ -287,7 +286,7 @@ int32_t psf_start(uint8_t *buffer, uint32_t length)
 		int i;
 		for (i = 0; i < MAX_UNKNOWN_TAGS; i++)
 		{
-			if (!strcasecmp(c->tag_name[i], "psfby"))
+			if (!g_ascii_strcasecmp(c->tag_name[i], "psfby"))
 				strcpy(psfby, c->tag_data[i]);
 		}
 	}
@@ -374,14 +373,14 @@ int32_t psf_start(uint8_t *buffer, uint32_t length)
 	return AO_SUCCESS;
 }
 
-int32_t psf_execute(InputPlayback *playback)
+int32_t psf_execute(void)
 {
 	int i;
 
 	while (!stop_flag) {
 		for (i = 0; i < 44100 / 60; i++) {
 			psx_hw_slice();
-			SPUasync(384, (void *) playback);
+			SPUasync(384);
 		}
 
 		psx_hw_frame();
