@@ -45,7 +45,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <strings.h>
 
 #include <glib.h>
 #include <zlib.h>
@@ -305,7 +304,7 @@ static uint32_t load_file_ex(uint8_t *top, uint8_t *start, uint32_t len, char *f
 		printf("[%s vs %s]: ofs %08x uncomp %08x bsize %08x\n", cptr, matchname, offs, uncomp, bsize);
 		#endif
 
-		if (!strcasecmp((char *)cptr, matchname))
+		if (!g_ascii_strcasecmp((char *)cptr, matchname))
 		{
 			if ((uncomp == 0) && (bsize == 0))
 			{
@@ -579,7 +578,7 @@ int32_t psf2_start(uint8_t *buffer, uint32_t length)
 	return AO_SUCCESS;
 }
 
-int32_t psf2_execute(InputPlayback *playback)
+int32_t psf2_execute(void)
 {
 	int i;
 
@@ -587,7 +586,7 @@ int32_t psf2_execute(InputPlayback *playback)
 	{
 		for (i = 0; i < 44100 / 60; i++)
 		{
-			SPU2async(1, (void *) playback);
+			SPU2async(1);
 			ps2_hw_slice();
 		}
 
