@@ -24,15 +24,15 @@
 	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <libgen.h>
-#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <audacious/i18n.h>
+#include <audacious/input.h>
 #include <audacious/misc.h>
 #include <audacious/plugin.h>
+#include <libaudcore/audstrings.h>
 
 #include "ao.h"
 #include "corlett.h"
@@ -50,7 +50,7 @@ typedef struct {
     int32_t (*start)(uint8_t *buffer, uint32_t length);
     int32_t (*stop)(void);
     int32_t (*seek)(uint32_t);
-    int32_t (*execute)(InputPlayback *data);
+    int32_t (*execute)(void);
 } PSFEngineFunctors;
 
 static PSFEngineFunctors psf_functor_map[ENG_COUNT] = {
@@ -60,6 +60,11 @@ static PSFEngineFunctors psf_functor_map[ENG_COUNT] = {
     {spx_start, spx_stop, psf_seek, spx_execute},
 };
 
+static PSFEngineFunctors *f;
+static const char *dirpath;
+
+bool_t stop_flag = FALSE;
+
 static PSFEngine psf_probe(uint8_t *buffer)
 {
 	if (!memcmp(buffer, "PSF\x01", 4))
@@ -78,8 +83,6 @@ static PSFEngine psf_probe(uint8_t *buffer)
 }
 
 /* ao_get_lib: called to load secondary files */
-static const char *dirpath;
-
 int ao_get_lib(char *filename, uint8_t **buffer, uint64_t *length)
 {
 	void *filebuf;
@@ -94,10 +97,6 @@ int ao_get_lib(char *filename, uint8_t **buffer, uint64_t *length)
 	return AO_SUCCESS;
 }
 
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static int seek = 0;
-bool_t stop_flag = FALSE;
-
 Tuple *psf2_tuple(const char *filename, VFSFile *file)
 {
 	Tuple *t;
@@ -115,15 +114,13 @@ Tuple *psf2_tuple(const char *filename, VFSFile *file)
 
 	t = tuple_new_from_filename(filename);
 
-	tuple_set_int(t, FIELD_LENGTH, NULL, c->inf_length ? psfTimeToMS(c->inf_length) + psfTimeToMS(c->inf_fade) : -1);
-	tuple_set_str(t, FIELD_ARTIST, NULL, c->inf_artist);
-	tuple_set_str(t, FIELD_ALBUM, NULL, c->inf_game);
-	tuple_set_str(t, -1, "game", c->inf_game);
-	tuple_set_str(t, FIELD_TITLE, NULL, c->inf_title);
-	tuple_set_str(t, FIELD_COPYRIGHT, NULL, c->inf_copy);
-	tuple_set_str(t, FIELD_QUALITY, NULL, _("sequenced"));
-	tuple_set_str(t, FIELD_CODEC, NULL, "PlayStation 1/2 Audio");
-	tuple_set_str(t, -1, "console", "PlayStation 1/2");
+	tuple_set_int(t, FIELD_LENGTH, c->inf_length ? psfTimeToMS(c->inf_length) + psfTimeToMS(c->inf_fade) : -1);
+	tuple_set_str(t, FIELD_ARTIST, c->inf_artist);
+	tuple_set_str(t, FIELD_ALBUM, c->inf_game);
+	tuple_set_str(t, FIELD_TITLE, c->inf_title);
+	tuple_set_str(t, FIELD_COPYRIGHT, c->inf_copy);
+	tuple_set_str(t, FIELD_QUALITY, _("sequenced"));
+	tuple_set_str(t, FIELD_CODEC, "PlayStation 1/2 Audio");
 
 	free(c);
 	free(buf);
@@ -131,17 +128,19 @@ Tuple *psf2_tuple(const char *filename, VFSFile *file)
 	return t;
 }
 
-static bool_t psf2_play(InputPlayback * data, const char * filename, VFSFile * file, int start_time, int stop_time, bool_t pause)
+static bool_t psf2_play(const char * filename, VFSFile * file)
 {
 	void *buffer;
 	int64_t size;
 	PSFEngine eng;
-	PSFEngineFunctors *f;
 	bool_t error = FALSE;
 
-	char dirbuf[strlen (filename) + 1];
-	strcpy (dirbuf, filename);
-	dirpath = dirname (dirbuf);
+	const char * slash = strrchr (filename, '/');
+	if (! slash)
+		return FALSE;
+
+	SNCOPY (dirbuf, filename, slash + 1 - filename);
+	dirpath = dirbuf;
 
 	vfs_file_get_contents (filename, & buffer, & size);
 
@@ -159,88 +158,39 @@ static bool_t psf2_play(InputPlayback * data, const char * filename, VFSFile * f
 		return FALSE;
 	}
 
-	data->output->open_audio(FMT_S16_NE, 44100, 2);
+	aud_input_open_audio(FMT_S16_NE, 44100, 2);
 
-	data->set_params(data, 44100*2*2*8, 44100, 2);
+	aud_input_set_bitrate(44100*2*2*8);
 
 	stop_flag = FALSE;
-	data->set_pb_ready(data);
 
-	for (;;)
-	{
-		f->execute(data);
-
-		if (seek)
-		{
-			data->output->flush(seek);
-
-			f->stop();
-
-			if (f->start(buffer, size) == AO_SUCCESS)
-			{
-				f->seek(seek);
-				seek = 0;
-				continue;
-			}
-			else
-				break;
-		}
-
-		f->stop();
-		break;
-	}
-
-	pthread_mutex_lock (& mutex);
-	stop_flag = TRUE;
-	pthread_mutex_unlock (& mutex);
+	f->execute();
+	f->stop();
 
+	f = NULL;
 	dirpath = NULL;
 	free(buffer);
 
 	return ! error;
 }
 
-void psf2_update(unsigned char *buffer, long count, InputPlayback *playback)
+void psf2_update(unsigned char *buffer, long count)
 {
-	if (buffer == NULL)
+	if (! buffer || aud_input_check_stop ())
 	{
 		stop_flag = TRUE;
-
 		return;
 	}
 
-	playback->output->write_audio (buffer, count);
+	int seek = aud_input_check_seek ();
 
-	if (seek)
+	if (seek >= 0)
 	{
-		if (psf2_seek(seek))
-		{
-			playback->output->flush(seek);
-			seek = 0;
-		}
-		else
-		{
-			stop_flag = TRUE;
-			return;
-		}
+		f->seek (seek);
+		return;
 	}
-}
 
-void psf2_Stop(InputPlayback *playback)
-{
-	pthread_mutex_lock (& mutex);
-	if (! stop_flag)
-	{
-		stop_flag = TRUE;
-		playback->output->abort_write ();
-	}
-	pthread_mutex_unlock (& mutex);
-}
-
-void psf2_pause(InputPlayback *playback, bool_t pause)
-{
-	if (!stop_flag)
-		playback->output->pause(pause);
+	aud_input_write_audio (buffer, count);
 }
 
 int psf2_is_our_fd(const char *filename, VFSFile *file)
@@ -252,11 +202,6 @@ int psf2_is_our_fd(const char *filename, VFSFile *file)
 	return (psf_probe(magic) != ENG_NONE);
 }
 
-static void psf2_Seek(InputPlayback *playback, int time)
-{
-	seek = time;
-}
-
 static const char *psf2_fmts[] = { "psf", "minipsf", "psf2", "minipsf2", "spu", "spx", NULL };
 
 AUD_INPUT_PLUGIN
@@ -264,9 +209,6 @@ AUD_INPUT_PLUGIN
 	.name = N_("OpenPSF PSF1/PSF2 Decoder"),
 	.domain = PACKAGE,
 	.play = psf2_play,
-	.stop = psf2_Stop,
-	.pause = psf2_pause,
-	.mseek = psf2_Seek,
 	.probe_for_tuple = psf2_tuple,
 	.is_our_file_from_vfs = psf2_is_our_fd,
 	.extensions = psf2_fmts,
