@@ -20,12 +20,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <glib.h>
+
 #include <samplerate.h>
 
 #include <audacious/i18n.h>
 #include <audacious/misc.h>
 #include <audacious/plugin.h>
 #include <audacious/preferences.h>
+#include <libaudcore/audstrings.h>
 
 #define MIN_RATE 8000
 #define MAX_RATE 192000
@@ -66,7 +69,7 @@ void resample_cleanup (void)
         state = NULL;
     }
 
-    free (buffer);
+    g_free (buffer);
     buffer = NULL;
     buffer_samples = 0;
 }
@@ -83,7 +86,8 @@ void resample_start (int * channels, int * rate)
 
     if (aud_get_bool ("resample", "use-mappings"))
     {
-        SPRINTF (rate_s, "%d", * rate);
+        char rate_s[16];
+        str_itoa (* rate, rate_s, sizeof rate_s);
         new_rate = aud_get_int ("resample", rate_s);
     }
 
@@ -117,7 +121,7 @@ void do_resample (float * * data, int * samples, bool_t finish)
     if (buffer_samples < (int) (* samples * ratio) + 256)
     {
         buffer_samples = (int) (* samples * ratio) + 256;
-        buffer = realloc (buffer, sizeof (float) * buffer_samples);
+        buffer = g_renew (float, buffer, buffer_samples);
     }
 
     SRC_DATA d = {
@@ -172,7 +176,7 @@ static const PreferencesWidget resample_widgets[] = {
  {WIDGET_LABEL, N_("<b>Conversion</b>")},
  {WIDGET_COMBO_BOX, N_("Method:"),
   .cfg_type = VALUE_STRING, .csect = "resample", .cname = "method",
-  .data = {.combo = {method_list, sizeof method_list / sizeof method_list[0]}}},
+  .data = {.combo = {method_list, ARRAY_LEN (method_list)}}},
  {WIDGET_SPIN_BTN, N_("Rate:"),
   .cfg_type = VALUE_INT, .csect = "resample", .cname = "default-rate",
   .data = {.spin_btn = {MIN_RATE, MAX_RATE, RATE_STEP, N_("Hz")}}},
@@ -203,7 +207,7 @@ static const PreferencesWidget resample_widgets[] = {
 
 static const PluginPreferences resample_prefs = {
  .widgets = resample_widgets,
- .n_widgets = sizeof resample_widgets / sizeof resample_widgets[0]};
+ .n_widgets = ARRAY_LEN (resample_widgets)};
 
 AUD_EFFECT_PLUGIN
 (
