@@ -80,9 +80,9 @@ extern gchar *received_data;
 extern size_t received_data_size;
 
 //Data filled by the XML parser
-extern gchar *request_token;
-extern gchar *session_key;
-extern gchar *username;
+extern gchar *request_token; /* pooled */
+extern gchar *session_key; /* pooled */
+extern gchar *username; /* pooled */
 
 //scrobbler_xml_parsing.c
 extern bool_t read_authentication_test_result(char **error_code, char **error_detail);
@@ -90,6 +90,6 @@ extern bool_t read_token(char **error_code, char **error_detail);
 extern bool_t read_session_key(char **error_code, char **error_detail);
 extern bool_t read_scrobble_result(char **error_code, char **error_detail, bool_t *ignored, char **ignored_code);
 
-//scrobbler.c //TODO: refactor this
-extern gchar *remove_tabs(const char *string);
+//scrobbler.c
+extern char *clean_string(char *string);
 #endif /* SCROBBLER_H_ */
