@@ -5,6 +5,8 @@
 #include <sys/time.h>
 #include <curl/curl.h>
 
+#include <libaudcore/audstrings.h>
+
 //plugin includes
 #include "scrobbler.h"
 
@@ -58,7 +60,7 @@ static char *scrobbler_get_signature(int nparams, API_Parameter *parameters) {
     for (gint i = 0; i < nparams; i++) {
         api_sig_length += strlen(parameters[i].paramName) + strlen(parameters[i].argument);
     }
-    all_params = calloc(api_sig_length, sizeof(gchar));
+    all_params = g_new0(gchar, api_sig_length);
 
     for (int i = 0; i < nparams; i++) {
         strcat(all_params, parameters[i].paramName);
@@ -66,7 +68,7 @@ static char *scrobbler_get_signature(int nparams, API_Parameter *parameters) {
     }
 
     api_sig = g_strconcat(all_params, SCROBBLER_SHARED_SECRET, NULL);
-    free(all_params);
+    g_free(all_params);
 
 
     result = g_compute_checksum_for_string(G_CHECKSUM_MD5, api_sig, -1);
@@ -171,18 +173,22 @@ static bool_t scrobbler_request_token() {
         return FALSE;
     }
 
+    bool_t success = TRUE;
     gchar *error_code = NULL;
     gchar *error_detail = NULL;
+
     if (read_token(&error_code, &error_detail) == FALSE) {
+        success = FALSE;
         if (error_code != NULL && g_strcmp0(error_code, "8")) {
             //error code 8: There was an error granting the request token. Please try again later
+            str_unref(request_token);
             request_token = NULL;
-            return FALSE;
         }
-        return FALSE;
     }
 
-    return TRUE;
+    str_unref(error_code);
+    str_unref(error_detail);
+    return success;
 }
 
 
@@ -198,7 +204,7 @@ static bool_t update_session_key() {
                 g_strcmp0(error_code, "15") == 0    //token expired
             )) {
             AUDDBG("error code CAUGHT: %s\n", error_code);
-            g_free(session_key);
+            str_unref(session_key);
             session_key = NULL;
             result = TRUE;
         } else {
@@ -206,11 +212,10 @@ static bool_t update_session_key() {
         }
     }
 
-    if (session_key == NULL) {
-        aud_set_string("scrobbler", "session_key", "");
-    } else {
-        aud_set_string("scrobbler", "session_key", session_key);
-    }
+    aud_set_str("scrobbler", "session_key", session_key ? session_key : "");
+
+    str_unref(error_code);
+    str_unref(error_detail);
     return result;
 }
 
@@ -231,9 +236,7 @@ static bool_t scrobbler_request_session() {
 
     g_free(sessionmsg);
     //the token can only be sent once
-    if (request_token != NULL) {
-        g_free(request_token);
-    }
+    str_unref(request_token);
     request_token = NULL;
 
     return update_session_key();
@@ -248,7 +251,7 @@ static bool_t scrobbler_request_session() {
 //sets session_key to NULL if it is invalid
 static bool_t scrobbler_test_connection() {
 
-    if (session_key == NULL || strlen(session_key) == 0) {
+    if (!session_key || !session_key[0]) {
         scrobbling_enabled = FALSE;
         return TRUE;
     }
@@ -273,31 +276,32 @@ static bool_t scrobbler_test_connection() {
 
     gchar *error_code = NULL;
     gchar *error_detail = NULL;
+
     if (read_authentication_test_result(&error_code, &error_detail) == FALSE) {
         AUDDBG("Error code: %s. Detail: %s.\n", error_code, error_detail);
         if (error_code != NULL && (
                 g_strcmp0(error_code, "4") == 0 || //error code 4: Authentication Failed - You do not have permissions to access the service
                 g_strcmp0(error_code, "9") == 0    //error code 9: Invalid session key - Please re-authenticate
             )) {
-            g_free(error_code);
-            g_free(error_detail);
-            g_free(session_key);
+            str_unref(session_key);
             session_key = NULL;
-            aud_set_string("scrobbler", "session_key", "");
+            aud_set_str("scrobbler", "session_key", "");
             scrobbling_enabled = FALSE;
-            return TRUE;
         } else {
             //network problem.
             scrobbling_enabled = FALSE;
             AUDDBG("Connection NOT OK. Scrobbling disabled\n");
-            return FALSE;
+            success = FALSE;
         }
     } else {
         //THIS IS THE ONLY PLACE WHERE SCROBBLING IS SET TO ENABLED IN RUN-TIME
         scrobbling_enabled = TRUE;
         AUDDBG("Connection OK. Scrobbling enabled.\n");
-        return TRUE;
     }
+
+    str_unref(error_code);
+    str_unref(error_detail);
+    return success;
 }
 
 //called from scrobbler_init() @ scrobbler.c
@@ -426,10 +430,6 @@ static void scrobble_cached_queue() {
     gchar *scrobblemsg;
     GSList *lines_to_remove = NULL; //lines to remove because they were scrobbled (or ignored, and will not be retried)
     GSList *lines_to_retry = NULL; //lines to retry later because they were too old TODO: or "daily scrobble limit reached"
-    gchar *error_code = NULL;
-    gchar *error_detail = NULL;
-    bool_t ignored = FALSE;
-    gchar *ignored_code = NULL;
 
     pthread_mutex_lock(&log_access_mutex);
     success = g_file_get_contents(queuepath, &contents, NULL, NULL);
@@ -458,8 +458,11 @@ static void scrobble_cached_queue() {
                                                        "api_key", SCROBBLER_API_KEY,
                                                        "sk", session_key);
                 if (send_message_to_lastfm(scrobblemsg) == TRUE) {
-                    error_code = NULL;
-                    error_detail = NULL;
+                    char *error_code = NULL;
+                    char *error_detail = NULL;
+                    bool_t ignored = FALSE;
+                    char *ignored_code = NULL;
+
                     if (read_scrobble_result(&error_code, &error_detail, &ignored, &ignored_code) == TRUE) {
                         AUDDBG("SCROBBLE OK. Error code: %s. Error detail: %s\n", error_code, error_detail);
                         AUDDBG("SCROBBLE OK. ignored: %i.\n", ignored);
@@ -492,18 +495,18 @@ static void scrobble_cached_queue() {
                         else if (g_strcmp0(error_code,  "9") == 0) {
                             //Bad Session. Reauth.
                             scrobbling_enabled = FALSE;
-                            g_free(session_key);
+                            str_unref(session_key);
                             session_key = NULL;
-                            aud_set_string("scrobbler", "session_key", "");
+                            aud_set_str("scrobbler", "session_key", "");
                         }
                         else {
                             save_line_to_remove(&lines_to_remove, i);
                         }
                     }
 
-                    g_free(error_code);
-                    g_free(error_detail);
-
+                    str_unref(error_code);
+                    str_unref(error_detail);
+                    str_unref(ignored_code);
                 } else {
                     AUDDBG("Could not scrobble a track on the queue. Network problem?\n");
                     //scrobble to be retried
@@ -552,58 +555,39 @@ static void send_now_playing() {
    */
   Tuple *curr_track = now_playing_track;
 
-  gchar *tab_remover;
+  char *artist = clean_string(tuple_get_str(curr_track, FIELD_ARTIST));
+  char *title  = clean_string(tuple_get_str(curr_track, FIELD_TITLE));
+  char *album  = clean_string(tuple_get_str(curr_track, FIELD_ALBUM));
 
-  gchar *artist = tuple_get_str(curr_track, FIELD_ARTIST, NULL);
-  gchar *album = tuple_get_str(curr_track, FIELD_ALBUM, NULL);
-  gchar *title = tuple_get_str(curr_track, FIELD_TITLE, NULL);
+  int track  = tuple_get_int(curr_track, FIELD_TRACK_NUMBER);
+  int length = tuple_get_int(curr_track, FIELD_LENGTH);
 
-  tab_remover = remove_tabs(artist);
-  str_unref(artist);
-  artist = tab_remover;
-
-  tab_remover = remove_tabs(album);
-  str_unref(album);
-  album = tab_remover;
-
-  tab_remover = remove_tabs(title);
-  str_unref(title);
-  title = tab_remover;
-
-  tab_remover = NULL;
-
-  gchar *number = g_strdup_printf("%i", tuple_get_int(curr_track, FIELD_TRACK_NUMBER, NULL));
-  gchar *length = g_strdup_printf("%i", tuple_get_int(curr_track, FIELD_LENGTH, NULL) / 1000);
   tuple_unref(curr_track);
 
-
-  if (artist != NULL && strlen(artist) > 0 &&
-       title != NULL && strlen(title)  > 0) {
+  if (artist[0] && title[0] && length > 0) {
+    char *track_str = (track > 0) ? int_to_str(track) : str_get("");
+    char *length_str = int_to_str(length / 1000);
 
     gchar *playingmsg = create_message_to_lastfm("track.updateNowPlaying",
                                             7,
                                            "artist", artist,
-                                           "album", (album == NULL ? "" : album),
+                                           "album", album,
                                            "track", title,
-                                           "trackNumber", number,
-                                           "duration", length,
+                                           "trackNumber", track_str,
+                                           "duration", length_str,
                                            "api_key", SCROBBLER_API_KEY,
                                            "sk", session_key);
-    g_free(artist);
-    g_free(album);
-    g_free(title);
-    g_free(number);
-    g_free(length);
 
     bool_t success = send_message_to_lastfm(playingmsg);
     g_free(playingmsg);
+
+    str_unref(track_str);
+    str_unref(length_str);
+
     if (success == FALSE) {
       AUDDBG("Network problems. Could not send \"now playing\" to last.fm\n");
       scrobbling_enabled = FALSE;
-      return;
-    }
-
-    if (read_scrobble_result(&error_code, &error_detail, &ignored, &ignored_code) == TRUE) {
+    } else if (read_scrobble_result(&error_code, &error_detail, &ignored, &ignored_code) == TRUE) {
       //see scrobble_cached_queue()
       AUDDBG("NOW PLAYING OK.\n");
     } else {
@@ -614,47 +598,52 @@ static void send_now_playing() {
         //Bad Session. Reauth.
         //We don't really care about any other errors.
         scrobbling_enabled = FALSE;
-        g_free(session_key);
+        str_unref(session_key);
         session_key = NULL;
-        aud_set_string("scrobbler", "session_key", "");
+        aud_set_str("scrobbler", "session_key", "");
       }
 
     }
-    g_free(error_code);
-    g_free(error_detail);
-    g_free(ignored_code);
     //We don't care if the now playing was not accepted, no need to read the result from the server.
 
   }
+
+  str_unref(artist);
+  str_unref(title);
+  str_unref(album);
+
+  str_unref(error_code);
+  str_unref(error_detail);
+  str_unref(ignored_code);
 }
 
 static void treat_permission_check_request() {
-    if (session_key == NULL || strlen(session_key) == 0) {
+    if (!session_key || !session_key[0]) {
         perm_result = PERMISSION_DENIED;
 
-        if (request_token == NULL || strlen(request_token) == 0) {
-            if (scrobbler_request_token() == FALSE || request_token == NULL || strlen(request_token) == 0) {
+        if (!request_token || !request_token[0]) {
+            if (scrobbler_request_token() == FALSE || !request_token || !request_token[0]) {
                 perm_result = PERMISSION_NONET;
             } //else PERMISSION_DENIED
 
         } else if (scrobbler_request_session() == FALSE) {
             perm_result = PERMISSION_NONET;
 
-        } else if (session_key == NULL || strlen(session_key) == 0) {
+        } else if (!session_key || !session_key[0]) {
             //This means we had a token, a session was requested now,
             //but the token was not accepted or expired.
             //Ask for a new token now.
-            if (scrobbler_request_token() == FALSE || request_token == NULL || strlen(request_token) == 0) {
+            if (scrobbler_request_token() == FALSE || !request_token || !request_token[0]) {
                 perm_result = PERMISSION_NONET;
             } //else PERMISSION_DENIED
         }
     }
-    if (session_key != NULL && strlen(session_key) != 0 ){
+    if (session_key && session_key[0]) {
         if (scrobbler_test_connection() == FALSE) {
             perm_result = PERMISSION_NONET;
 
-            if (session_key == NULL || strlen(session_key) == 0) {
-                if (scrobbler_request_token() != FALSE && request_token != NULL && strlen(request_token) != 0) {
+            if (!session_key || !session_key[0]) {
+                if (scrobbler_request_token() != FALSE && request_token && request_token[0]) {
                     perm_result = PERMISSION_DENIED;
                 } //else PERMISSION_NONET
             }
@@ -671,7 +660,7 @@ static void treat_permission_check_request() {
               * c) the user fiddled with the audacious config file and
               * the key is now invalid
               */
-                if (scrobbler_request_token() != FALSE && request_token != NULL && strlen(request_token) != 0) {
+                if (scrobbler_request_token() != FALSE && request_token && request_token[0]) {
                     perm_result = PERMISSION_DENIED;
                 } else {
                     perm_result = PERMISSION_NONET;
@@ -688,16 +677,16 @@ static void treat_permission_check_request() {
 // TRUE if a new session_key was obtained
 static bool_t treat_migrate_config() {
 
-    char *password = aud_get_string("audioscrobbler","password");
-    if (password == NULL || strlen(password) == 0) {
-        g_free(password);
+    char *password = aud_get_str("audioscrobbler","password");
+    if (!password[0]) {
+        str_unref(password);
         return FALSE;
     }
 
-    char *username = aud_get_string("audioscrobbler","username");
-    if (username == NULL || strlen(username) == 0) {
-        g_free(password);
-        g_free(username);
+    char *username = aud_get_str("audioscrobbler","username");
+    if (!username[0]) {
+        str_unref(password);
+        str_unref(username);
         return FALSE;
     }
 
@@ -709,8 +698,8 @@ static bool_t treat_migrate_config() {
                                                  "authToken", authToken,
                                                  "username", username,
                                                  "api_key", SCROBBLER_API_KEY);
-    g_free(username);
-    g_free(password);
+    str_unref(username);
+    str_unref(password);
     g_free(checksumThis);
     g_free(authToken);
 
@@ -721,15 +710,10 @@ static bool_t treat_migrate_config() {
 
     g_free(sessionmsg);
 
-    if (update_session_key() == TRUE) {
-        if(session_key != NULL && strlen(session_key) != 0) {
-            return TRUE;
-        } else {
-            return FALSE;
-        }
-    } else {
+    if (!update_session_key())
         return FALSE;
-    }
+
+    return (session_key && session_key[0]);
 }
 
 
@@ -742,7 +726,7 @@ gpointer scrobbling_thread (gpointer input_data) {
           if (treat_migrate_config() == FALSE) {
             aud_interface_show_error(_("Audacious is now using an improved version of the Last.fm Scrobbler.\nPlease check the Preferences for the Scrobbler plugin."));
           }
-          aud_set_string("scrobbler", "migrated", "true");
+          aud_set_str("scrobbler", "migrated", "true");
           migrate_config_requested = FALSE;
 
         } else if (permission_check_requested) {
@@ -750,8 +734,9 @@ gpointer scrobbling_thread (gpointer input_data) {
             permission_check_requested = FALSE;
 
         } else if (invalidate_session_requested) {
+            str_unref(session_key);
             session_key = NULL;
-            aud_set_string("scrobbler", "session_key", "");
+            aud_set_str("scrobbler", "session_key", "");
             invalidate_session_requested = FALSE;
 
         } else if (now_playing_requested) {
@@ -791,7 +776,7 @@ gpointer scrobbling_thread (gpointer input_data) {
     }//while(scrobbler_running)
 
     //reset all vars to their initial values
-    free(received_data);
+    g_free(received_data);
     received_data = NULL;
     received_data_size = 0;
 
