@@ -38,17 +38,13 @@ static void clean_data() {
 //returns:
 // NULL if an error occurs or the attribute was not found
 // the attribute's value if it was found
-static xmlChar *get_attribute_value (xmlChar *node_expression, xmlChar *attribute) {
+static char *get_attribute_value (const char *node_expression, const char *attribute) {
     if (doc == NULL || context == NULL) {
         AUDDBG("Response from last.fm not parsed successfully. Did you call prepare_data?\n");
         return NULL;
     }
 
-    xmlXPathObjectPtr statusObj;
-    xmlChar *result;
-
-
-    statusObj = xmlXPathEvalExpression((xmlChar *) node_expression, context);
+    xmlXPathObjectPtr statusObj = xmlXPathEvalExpression((xmlChar *) node_expression, context);
     if (statusObj == NULL) {
         AUDDBG ("Error in xmlXPathEvalExpression.\n");
         return NULL;
@@ -59,26 +55,28 @@ static xmlChar *get_attribute_value (xmlChar *node_expression, xmlChar *attribut
         return NULL;
     }
 
-    result = xmlGetProp(statusObj->nodesetval->nodeTab[0], (xmlChar *) attribute);
+    xmlChar *prop = xmlGetProp(statusObj->nodesetval->nodeTab[0], (xmlChar *) attribute);
+
+    char *result = NULL;
+    if (prop && prop[0])
+        result = str_get((char *) prop);
 
     xmlXPathFreeObject(statusObj);
+    xmlFree(prop);
+
     AUDDBG("RESULT FOR THIS FUNCTION: %s.\n", result);
     return result;
 }
 
 //returns:
 // NULL if an error occurs or the node was not found
-static xmlChar *get_node_string (const char * node_expression) {
+static char *get_node_string (const char *node_expression) {
     if (doc == NULL || context == NULL) {
         AUDDBG("Response from last.fm not parsed successfully. Did you call prepare_data?\n");
         return NULL;
     }
 
-    xmlXPathObjectPtr statusObj;
-    xmlChar *result;
-
-
-    statusObj = xmlXPathEvalExpression((xmlChar *) node_expression, context);
+    xmlXPathObjectPtr statusObj = xmlXPathEvalExpression((xmlChar *) node_expression, context);
     if (statusObj == NULL) {
         AUDDBG ("Error in xmlXPathEvalExpression.\n");
         return NULL;
@@ -89,8 +87,15 @@ static xmlChar *get_node_string (const char * node_expression) {
         return NULL;
     }
 
-    result = xmlNodeListGetString(doc, statusObj->nodesetval->nodeTab[0]->children, 1);
+    xmlChar *string = xmlNodeListGetString(doc, statusObj->nodesetval->nodeTab[0]->children, 1);
+
+    char *result = NULL;
+    if (string && string[0])
+        result = str_get((char *) string);
+
     xmlXPathFreeObject(statusObj);
+    xmlFree(string);
+
     AUDDBG("RESULT FOR THIS FUNCTION: %s.\n", result);
     return result;
 }
@@ -100,23 +105,23 @@ static xmlChar *get_node_string (const char * node_expression) {
 // NULL if an error occurs
 // "true" if the the command succeeded
 // "false" if an error occurred. error_code and error_detail should be checked in this case
-static xmlChar *check_status (xmlChar **error_code, xmlChar **error_detail) {
+static char *check_status (char **error_code, char **error_detail) {
     (*error_code) = NULL;
     (*error_detail) = NULL;
 
-    xmlChar *status = get_attribute_value((xmlChar *) "/lfm[@status]", (xmlChar *) "status");
-    if (status == NULL || xmlStrlen(status) == 0) {
+    char *status = get_attribute_value("/lfm[@status]", "status");
+    if (!status) {
         AUDDBG("last.fm not answering according to the API.\n");
         return NULL;
     }
 
     AUDDBG ("status is %s.\n", status);
-    if (!xmlStrEqual(status, (xmlChar *) "ok")) {
+    if (strcmp(status, "ok")) {
 
-        (*error_code) = get_attribute_value((xmlChar *) "/lfm/error[@code]", (xmlChar *) "code");
-        if ((*error_code) == NULL || xmlStrlen(*error_code) == 0) {
+        (*error_code) = get_attribute_value("/lfm/error[@code]", "code");
+        if (!(*error_code)) {
             AUDDBG("Weird API answer. Last.fm says status is %s but there is no error code?\n", status);
-            xmlFree(status);
+            str_unref(status);
             status = NULL;
         } else {
             (*error_detail) = get_node_string("/lfm/error");
@@ -137,68 +142,50 @@ static xmlChar *check_status (xmlChar **error_code, xmlChar **error_detail) {
  *    * error_code_out and error_detail_out must be checked:
  *      * They are NULL if an API communication error occur
  */
-bool_t read_scrobble_result(char **error_code_out, char **error_detail_out, bool_t *ignored_out, char **ignored_code_out) {
-    xmlChar *status;
-    xmlChar *error_code = NULL;
-    xmlChar *error_detail = NULL;
+bool_t read_scrobble_result(char **error_code, char **error_detail, bool_t *ignored, char **ignored_code) {
 
-    bool_t ignored = FALSE;
-    xmlChar *ignored_code = NULL;
+    *error_code = NULL;
+    *error_detail = NULL;
+    *ignored = FALSE;
+    *ignored_code = NULL;
 
     bool_t result = TRUE;
 
-
     if (!prepare_data()) {
         AUDDBG("Could not read received data from last.fm. What's up?\n");
         return FALSE;
     }
 
-    status = check_status(&error_code, &error_detail);
-    (*error_code_out) = g_strdup((gchar *) error_code);
-    (*error_detail_out) = g_strdup((gchar *) error_detail);
+    char *status = check_status(error_code, error_detail);
 
-
-    if (status == NULL || xmlStrlen(status) == 0) {
+    if (!status) {
         AUDDBG("Status was NULL. Invalid API answer.\n");
         clean_data();
         return FALSE;
     }
 
-    if (xmlStrEqual(status, (xmlChar *) "failed")) {
-        AUDDBG("Error code: %s. Detail: %s.\n", error_code, error_detail);
+    if (!strcmp(status, "failed")) {
+        AUDDBG("Error code: %s. Detail: %s.\n", *error_code, *error_detail);
         result = FALSE;
 
     } else {
         //TODO: We are assuming that only one track is scrobbled per request! This will have to be
         //re-done to support multiple tracks being scrobbled in batch
-        xmlChar *ignored_scrobble = get_attribute_value((xmlChar *) "/lfm/scrobbles[@ignored]", (xmlChar *) "ignored");
+        char *ignored_scrobble = get_attribute_value("/lfm/scrobbles[@ignored]", "ignored");
 
-        if (ignored_scrobble != NULL && ! xmlStrEqual(ignored_scrobble, (xmlChar *) "0")) {
+        if (ignored_scrobble && strcmp(ignored_scrobble, "0")) {
           //The track was ignored
           //TODO: this assumes ignored_scrobble == 1!!!
-          ignored = TRUE;
-          ignored_code = get_attribute_value((xmlChar *) "/lfm/scrobbles/scrobble/ignoredMessage[@code]", (xmlChar *) "code");
+          *ignored = TRUE;
+          *ignored_code = get_attribute_value("/lfm/scrobbles/scrobble/ignoredMessage[@code]", "code");
         }
-        if (ignored_scrobble != NULL) {
-          xmlFree(ignored_scrobble);
-        }
-        AUDDBG("ignored? %i, ignored_code: %s\n", ignored, ignored_code);
+
+        str_unref(ignored_scrobble);
+
+        AUDDBG("ignored? %i, ignored_code: %s\n", *ignored, *ignored_code);
     }
 
-    (*ignored_out) = ignored;
-    (*ignored_code_out) = g_strdup((gchar *) ignored_code);
-
-
-    xmlFree(status);
-    if (error_code != NULL) {
-        xmlFree(error_code);
-    }
-    if (error_detail != NULL) {
-        xmlFree(error_detail);
-    }
-    if (ignored_code != NULL) {
-        xmlFree(ignored_code);
-    }
+    str_unref(status);
 
     clean_data();
     return result;
@@ -206,47 +193,39 @@ bool_t read_scrobble_result(char **error_code_out, char **error_detail_out, bool
 
 //returns
 //FALSE if there was an error with the connection
-bool_t read_authentication_test_result (char **error_code_out, char **error_detail_out) {
-    xmlChar *status;
-    xmlChar *error_code = NULL;
-    xmlChar *error_detail = NULL;
-    bool_t result = TRUE;
+bool_t read_authentication_test_result (char **error_code, char **error_detail) {
 
+    *error_code = NULL;
+    *error_detail = NULL;
+
+    bool_t result = TRUE;
 
     if (!prepare_data()) {
         AUDDBG("Could not read received data from last.fm. What's up?\n");
         return FALSE;
     }
 
-    status = check_status(&error_code, &error_detail);
-    (*error_code_out) = g_strdup((gchar *) error_code);
-    (*error_detail_out) = g_strdup((gchar *) error_detail);
+    char *status = check_status(error_code, error_detail);
 
-    if (status == NULL || xmlStrlen(status) == 0) {
+    if (!status) {
         AUDDBG("Status was NULL. Invalid API answer.\n");
         clean_data();
         return FALSE;
     }
 
-    if (xmlStrEqual(status, (xmlChar *) "failed")) {
+    if (!strcmp(status, "failed")) {
         result = FALSE;
 
     } else {
-        username = (gchar *) get_attribute_value((xmlChar *) "/lfm/recommendations[@user]", (xmlChar *) "user");
-        if (username == NULL || strlen(username) == 0) {
+        str_unref(username);
+        username = get_attribute_value("/lfm/recommendations[@user]", "user");
+        if (!username) {
           AUDDBG("last.fm not answering according to the API.\n");
           result = FALSE;
         }
     }
 
-
-    xmlFree(status);
-    if (error_code != NULL) {
-        xmlFree(error_code);
-    }
-    if (error_detail != NULL) {
-        xmlFree(error_detail);
-    }
+    str_unref(status);
 
     clean_data();
     return result;
@@ -254,36 +233,35 @@ bool_t read_authentication_test_result (char **error_code_out, char **error_deta
 
 
 
-bool_t read_token (char **error_code_out, char **error_detail_out) {
-    xmlChar *status;
-    xmlChar *error_code = NULL;
-    xmlChar *error_detail = NULL;
-    bool_t result = TRUE;
+bool_t read_token (char **error_code, char **error_detail) {
 
+    *error_code = NULL;
+    *error_detail = NULL;
+
+    bool_t result = TRUE;
 
     if (!prepare_data()) {
         AUDDBG("Could not read received data from last.fm. What's up?\n");
         return FALSE;
     }
 
-    status = check_status(&error_code, &error_detail);
-    (*error_code_out) = g_strdup((gchar *) error_code);
-    (*error_detail_out) = g_strdup((gchar *) error_detail);
+    char *status = check_status(error_code, error_detail);
 
-    if (status == NULL || xmlStrlen(status) == 0) {
+    if (!status) {
         AUDDBG("Status was NULL. Invalid API answer.\n");
         clean_data();
         return FALSE;
     }
 
-    if (xmlStrEqual(status, (xmlChar *) "failed")) {
-        AUDDBG("Error code: %s. Detail: %s.\n", error_code, error_detail);
+    if (!strcmp(status, "failed")) {
+        AUDDBG("Error code: %s. Detail: %s.\n", *error_code, *error_detail);
         result = FALSE;
     }
     else {
-        request_token = (gchar *) get_node_string("/lfm/token");
+        str_unref(request_token);
+        request_token = get_node_string("/lfm/token");
 
-        if (request_token == NULL || strlen(request_token) == 0) {
+        if (!request_token || !request_token[0]) {
             AUDDBG("Could not read the received token. Something's wrong with the API?\n");
             result = FALSE;
         }
@@ -292,13 +270,7 @@ bool_t read_token (char **error_code_out, char **error_detail_out) {
         }
     }
 
-    xmlFree(status);
-    if (error_code != NULL) {
-        xmlFree(error_code);
-    }
-    if (error_detail != NULL) {
-        xmlFree(error_detail);
-    }
+    str_unref(status);
 
     clean_data();
     return result;
@@ -306,37 +278,35 @@ bool_t read_token (char **error_code_out, char **error_detail_out) {
 
 
 
-bool_t read_session_key(char **error_code_out, char **error_detail_out) {
-    xmlChar *status;
-    xmlChar *error_code = NULL;
-    xmlChar *error_detail = NULL;
-    bool_t result = TRUE;
+bool_t read_session_key(char **error_code, char **error_detail) {
 
+    *error_code = NULL;
+    *error_detail = NULL;
+
+    bool_t result = TRUE;
 
     if (!prepare_data()) {
         AUDDBG("Could not read received data from last.fm. What's up?\n");
         return FALSE;
     }
 
-    status = check_status(&error_code, &error_detail);
-    (*error_code_out) = g_strdup((gchar *) error_code);
-    (*error_detail_out) = g_strdup((gchar *) error_detail);
+    char *status = check_status(error_code, error_detail);
 
-    if (status == NULL || xmlStrlen(status) == 0) {
+    if (!status) {
         AUDDBG("Status was NULL or empty. Invalid API answer.\n");
         clean_data();
         return FALSE;
     }
 
-    if (xmlStrEqual(status, (xmlChar *) "failed")) {
-        AUDDBG("Error code: %s. Detail: %s.\n", error_code, error_detail);
+    if (!strcmp(status, "failed")) {
+        AUDDBG("Error code: %s. Detail: %s.\n", *error_code, *error_detail);
         result = FALSE;
 
     } else {
-        g_free(session_key);
-        session_key = (gchar *) get_node_string("/lfm/session/key");
+        str_unref(session_key);
+        session_key = get_node_string("/lfm/session/key");
 
-        if (session_key == NULL || strlen(session_key) == 0) {
+        if (!session_key || !session_key[0]) {
             AUDDBG("Could not read the received session key. Something's wrong with the API?\n");
             result = FALSE;
         } else {
@@ -344,13 +314,7 @@ bool_t read_session_key(char **error_code_out, char **error_detail_out) {
         }
     }
 
-    xmlFree(status);
-    if (error_code != NULL) {
-        xmlFree(error_code);
-    }
-    if (error_detail != NULL) {
-        xmlFree(error_detail);
-    }
+    str_unref(status);
 
     clean_data();
     return result;
