@@ -29,6 +29,7 @@
 #include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 #include <libaudgui/list.h>
+#include <libaudgui/menu.h>
 
 #define MAX_RESULTS 100
 #define SEARCH_DELAY 300
@@ -49,7 +50,7 @@ typedef struct {
 } SearchState;
 
 static int playlist_id;
-static char * * search_terms;
+static Index * search_terms;
 
 static GHashTable * added_table;
 static GHashTable * database;
@@ -68,17 +69,15 @@ static Item * item_new (int field, char * name, Item * parent)
     Item * item = g_slice_new (Item);
     item->field = field;
     item->name = name;
-    item->folded = g_utf8_casefold (name, -1);
+    item->folded = str_tolower_utf8 (name);
     item->parent = parent;
     item->matches = g_array_new (FALSE, FALSE, sizeof (int));
 
-    /* speed things up by using g_direct_equal() instead of g_str_equal()
-       because identical pooled strings have the same pointer */
     if (field == TITLE)
         item->children = NULL;
     else
-        item->children = g_hash_table_new_full (g_str_hash, g_direct_equal,
-         NULL, (GDestroyNotify) item_free);
+        item->children = g_hash_table_new_full ((GHashFunc) str_hash,
+         (GEqualFunc) str_equal, NULL, (GDestroyNotify) item_free);
 
     return item;
 }
@@ -89,7 +88,7 @@ static void item_free (Item * item)
         g_hash_table_destroy (item->children);
 
     str_unref (item->name);
-    g_free (item->folded);
+    str_unref (item->folded);
     g_array_free (item->matches, TRUE);
     g_slice_free (Item, item);
 }
@@ -141,26 +140,27 @@ static int get_playlist (bool_t require_added, bool_t require_scanned)
 
 static void set_search_phrase (const char * phrase)
 {
-    g_strfreev (search_terms);
-
-    char * folded = g_utf8_casefold (phrase, -1);
-    search_terms = g_strsplit (folded, " ", -1);
-    g_free (folded);
+    char * folded = str_tolower_utf8 (phrase);
+    index_free_full (search_terms, (IndexFreeFunc) str_unref);
+    search_terms = str_list_to_index (folded, " ");
+    str_unref (folded);
 }
 
 static char * get_path (void)
 {
-    char * path = aud_get_string ("search-tool", "path");
+    char * path = aud_get_str ("search-tool", "path");
     if (g_file_test (path, G_FILE_TEST_EXISTS))
         return path;
 
-    g_free (path);
-    path = g_build_filename (g_get_home_dir (), "Music", NULL);
+    str_unref (path);
+
+    path = filename_build (g_get_home_dir (), "Music");
     if (g_file_test (path, G_FILE_TEST_EXISTS))
         return path;
 
-    g_free (path);
-    return g_strdup (g_get_home_dir ());
+    str_unref (path);
+
+    return str_get (g_get_home_dir ());
 }
 
 static void destroy_added_table (void)
@@ -188,10 +188,8 @@ static void create_database (int list)
 {
     destroy_database ();
 
-    /* speed things up by using g_direct_equal() instead of g_str_equal()
-       because identical pooled strings have the same pointer */
-    database = g_hash_table_new_full (g_str_hash, g_direct_equal, NULL,
-     (GDestroyNotify) item_free);
+    database = g_hash_table_new_full ((GHashFunc) str_hash, (GEqualFunc)
+     str_equal, NULL, (GDestroyNotify) item_free);
 
     int entries = aud_playlist_entry_count (list);
 
@@ -264,20 +262,21 @@ static void search_cb (void * key, void * _item, void * _state)
         return;
 
     int oldmask = state->mask;
+    int count = index_count (search_terms);
 
-    for (int t = 0, bit = 1; search_terms[t]; t ++, bit <<= 1)
+    for (int t = 0, bit = 1; t < count; t ++, bit <<= 1)
     {
         if (! (state->mask & bit))
             continue; /* skip term if it is already found */
 
-        if (strstr (item->folded, search_terms[t]))
+        if (strstr (item->folded, index_get (search_terms, t)))
             state->mask &= ~bit; /* we found it */
         else if (! item->children)
             break; /* quit early if there are no children to search */
     }
 
     if (! state->mask)
-        index_append (state->items[item->field], item);
+        index_insert (state->items[item->field], -1, item);
 
     if (item->children)
         g_hash_table_foreach (item->children, search_cb, state);
@@ -288,7 +287,7 @@ static void search_cb (void * key, void * _item, void * _state)
 static int item_compare (const void * _a, const void * _b)
 {
     const Item * a = _a, * b = _b;
-    return string_compare (a->name, b->name);
+    return str_compare (a->name, b->name);
 }
 
 static void do_search (void)
@@ -304,9 +303,7 @@ static void do_search (void)
         state.items[f] = index_new ();
 
     /* effectively limits number of search terms to 32 */
-    state.mask = 0;
-    for (int t = 0, bit = 1; search_terms[t]; t ++, bit <<= 1)
-        state.mask |= bit;
+    state.mask = (1 << index_count (search_terms)) - 1;
 
     g_hash_table_foreach (database, search_cb, & state);
 
@@ -321,7 +318,7 @@ static void do_search (void)
         if (count)
         {
             index_sort (state.items[f], item_compare);
-            index_copy_append (state.items[f], 0, items, count);
+            index_copy_insert (state.items[f], 0, items, -1, count);
             total += count;
         }
 
@@ -336,7 +333,7 @@ static void do_search (void)
 
 static bool_t filter_cb (const char * filename, void * unused)
 {
-    return added_table && ! g_hash_table_lookup_extended (added_table, filename, NULL, NULL);
+    return added_table && ! g_hash_table_contains (added_table, filename);
 }
 
 static void begin_add (const char * path)
@@ -346,18 +343,22 @@ static void begin_add (const char * path)
     if (list < 0)
         list = create_playlist ();
 
-    aud_set_string ("search-tool", "path", path);
+    aud_set_str ("search-tool", "path", path);
 
     char * uri = filename_to_uri (path);
     g_return_if_fail (uri);
-    char * prefix = g_str_has_suffix (uri, "/") ? g_strdup (uri) : g_strconcat (uri, "/", NULL);
+
+    if (! g_str_has_suffix (uri, "/"))
+    {
+        SCONCAT2 (temp, uri, "/");
+        str_unref (uri);
+        uri = str_get (temp);
+    }
 
     destroy_added_table ();
 
-    /* speed things up by using g_direct_equal() instead of g_str_equal()
-       because identical pooled strings have the same pointer */
-    added_table = g_hash_table_new_full (g_str_hash, g_direct_equal,
-     (GDestroyNotify) str_unref, NULL);
+    added_table = g_hash_table_new_full ((GHashFunc) str_hash, (GEqualFunc)
+     str_equal, (GDestroyNotify) str_unref, NULL);
 
     int entries = aud_playlist_entry_count (list);
 
@@ -365,8 +366,7 @@ static void begin_add (const char * path)
     {
         char * filename = aud_playlist_entry_get_filename (list, entry);
 
-        if (g_str_has_prefix (filename, prefix) && ! g_hash_table_lookup_extended
-         (added_table, filename, NULL, NULL))
+        if (g_str_has_prefix (filename, uri) && ! g_hash_table_contains (added_table, filename))
         {
             aud_playlist_entry_set_selected (list, entry, FALSE);
             g_hash_table_insert (added_table, filename, NULL);
@@ -382,11 +382,9 @@ static void begin_add (const char * path)
     aud_playlist_remove_failed (list);
 
     Index * add = index_new ();
-    index_append (add, str_get (uri));
+    index_insert (add, -1, uri);
     aud_playlist_entry_insert_filtered (list, -1, add, NULL, filter_cb, NULL, FALSE);
 
-    g_free (uri);
-    g_free (prefix);
     adding = TRUE;
 }
 
@@ -506,7 +504,7 @@ static bool_t search_init (void)
 {
     find_playlist ();
 
-    set_search_phrase ("");
+    search_terms = index_new ();
     items = index_new ();
     selection = g_array_new (FALSE, FALSE, 1);
 
@@ -531,7 +529,7 @@ static void search_cleanup (void)
         search_source = 0;
     }
 
-    g_strfreev (search_terms);
+    index_free_full (search_terms, (IndexFreeFunc) str_unref);
     search_terms = NULL;
 
     index_free (items);
@@ -562,8 +560,8 @@ static void do_add (bool_t play, char * * title)
         for (int m = 0; m < item->matches->len; m ++)
         {
             int entry = g_array_index (item->matches, int, m);
-            index_append (filenames, aud_playlist_entry_get_filename (list, entry));
-            index_append (tuples, aud_playlist_entry_get_tuple (list, entry, TRUE));
+            index_insert (filenames, -1, aud_playlist_entry_get_filename (list, entry));
+            index_insert (tuples, -1, aud_playlist_entry_get_tuple (list, entry, TRUE));
         }
 
         n_selected ++;
@@ -672,27 +670,14 @@ static void list_activate_row (void * user, int row)
 
 static void list_right_click (void * user, GdkEventButton * event)
 {
+    static const AudguiMenuItem items[] = {
+        {N_("_Play"), "media-playback-start", .func = action_play},
+        {N_("_Create Playlist"), "document-new", .func = action_create_playlist},
+        {N_("_Add to Playlist"), "list-add", .func = action_add_to_playlist}
+    };
+
     GtkWidget * menu = gtk_menu_new ();
-
-    GtkWidget * item = gtk_image_menu_item_new_from_stock (GTK_STOCK_MEDIA_PLAY, NULL);
-    g_signal_connect (item, "activate", (GCallback) action_play, NULL);
-    gtk_widget_show (item);
-    gtk_menu_shell_append ((GtkMenuShell *) menu, item);
-
-    item = gtk_image_menu_item_new_with_mnemonic (_("_Create Playlist"));
-    gtk_image_menu_item_set_image ((GtkImageMenuItem *) item,
-     gtk_image_new_from_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_MENU));
-    g_signal_connect (item, "activate", (GCallback) action_create_playlist, NULL);
-    gtk_widget_show (item);
-    gtk_menu_shell_append ((GtkMenuShell *) menu, item);
-
-    item = gtk_image_menu_item_new_with_mnemonic (_("_Add to Playlist"));
-    gtk_image_menu_item_set_image ((GtkImageMenuItem *) item,
-     gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
-    g_signal_connect (item, "activate", (GCallback) action_add_to_playlist, NULL);
-    gtk_widget_show (item);
-    gtk_menu_shell_append ((GtkMenuShell *) menu, item);
-
+    audgui_menu_init (menu, items, ARRAY_LEN (items), NULL);
     gtk_menu_popup ((GtkMenu *) menu, NULL, NULL, NULL, NULL, event->button, event->time);
 }
 
@@ -724,10 +709,8 @@ static void * search_get_widget (void)
     GtkWidget * vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 
     entry = gtk_entry_new ();
-    gtk_entry_set_icon_from_stock ((GtkEntry *) entry, GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_FIND);
-#if GTK_CHECK_VERSION (3, 2, 0)
+    gtk_entry_set_icon_from_icon_name ((GtkEntry *) entry, GTK_ENTRY_ICON_PRIMARY, "edit-find");
     gtk_entry_set_placeholder_text ((GtkEntry *) entry, _("Search library"));
-#endif
     g_signal_connect (entry, "destroy", (GCallback) gtk_widget_destroyed, & entry);
     gtk_box_pack_start ((GtkBox *) vbox, entry, FALSE, FALSE, 0);
 
@@ -767,11 +750,11 @@ static void * search_get_widget (void)
 
     char * path = get_path ();
     gtk_file_chooser_set_filename ((GtkFileChooser *) chooser, path);
-    g_free (path);
+    str_unref (path);
 
     GtkWidget * button = gtk_button_new ();
-    gtk_container_add ((GtkContainer *) button, gtk_image_new_from_stock
-     (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON));
+    gtk_container_add ((GtkContainer *) button, gtk_image_new_from_icon_name
+     ("view-refresh", GTK_ICON_SIZE_BUTTON));
     gtk_button_set_relief ((GtkButton *) button, GTK_RELIEF_NONE);
     gtk_box_pack_start ((GtkBox *) hbox, button, FALSE, FALSE, 0);
 
