@@ -27,6 +27,7 @@
 #include <string.h>
 
 #include <libaudcore/audstrings.h>
+#include <audacious/input.h>
 #include <audacious/plugin.h>
 
 #include "xs_sidplay2.h"
@@ -110,16 +111,13 @@ void xs_close(void)
 /*
  * Start playing the given file
  */
-bool_t xs_play_file(InputPlayback *pb, const char *filename, VFSFile *file,
- int start_time, int stop_time, bool_t pause)
+bool_t xs_play_file(const char *filename, VFSFile *file)
 {
     xs_tuneinfo_t *tmpTune;
     int audioBufSize, bufRemaining, tmpLength, subTune = -1;
     char *audioBuffer = NULL, *oversampleBuffer = NULL;
     Tuple *tmpTuple;
 
-    assert(pb);
-
     uri_parse (filename, NULL, NULL, NULL, & subTune);
 
     /* Get tune information */
@@ -179,8 +177,7 @@ bool_t xs_play_file(InputPlayback *pb, const char *filename, VFSFile *file,
     }
 
     /* Open the audio output */
-    if (!pb->output->open_audio(FMT_S16_NE, xs_status.audioFrequency,
-     channels))
+    if (!aud_input_open_audio(FMT_S16_NE, xs_status.audioFrequency, channels))
     {
         xs_error("Couldn't open audio output (fmt=%x, freq=%i, nchan=%i)!\n",
             FMT_S16_NE,
@@ -196,43 +193,30 @@ bool_t xs_play_file(InputPlayback *pb, const char *filename, VFSFile *file,
     tmpTuple = tuple_new_from_filename(tmpTune->sidFilename);
     xs_get_song_tuple_info(tmpTuple, tmpTune, xs_status.currSong);
 
-    xs_status.stop_flag = FALSE;
     pthread_mutex_unlock(&xs_status_mutex);
 
-    pb->set_tuple(pb, tmpTuple);
-    pb->set_params (pb, -1, xs_status.audioFrequency, channels);
-    pb->set_pb_ready(pb);
+    aud_input_set_tuple(tmpTuple);
 
-    while (1)
+    while (! aud_input_check_stop ())
     {
-        pthread_mutex_lock(&xs_status_mutex);
-
-        if (xs_status.stop_flag)
-        {
-            pthread_mutex_unlock(&xs_status_mutex);
-            break;
-        }
-
-        pthread_mutex_unlock(&xs_status_mutex);
-
         bufRemaining = xs_sidplayfp_fillbuffer(&xs_status, audioBuffer, audioBufSize);
 
-        pb->output->write_audio (audioBuffer, bufRemaining);
+        aud_input_write_audio (audioBuffer, bufRemaining);
 
         /* Check if we have played enough */
         if (xs_cfg.playMaxTimeEnable) {
             if (xs_cfg.playMaxTimeUnknown) {
                 if (tmpLength < 0 &&
-                    pb->output->written_time() >= xs_cfg.playMaxTime * 1000)
+                    aud_input_written_time() >= xs_cfg.playMaxTime * 1000)
                     break;
             } else {
-                if (pb->output->written_time() >= xs_cfg.playMaxTime * 1000)
+                if (aud_input_written_time() >= xs_cfg.playMaxTime * 1000)
                     break;
             }
         }
 
         if (tmpLength >= 0) {
-            if (pb->output->written_time() >= tmpLength * 1000)
+            if (aud_input_written_time() >= tmpLength * 1000)
                 break;
         }
     }
@@ -247,7 +231,6 @@ DONE:
      * next entry in the playlist .. or whatever it wishes.
      */
     pthread_mutex_lock(&xs_status_mutex);
-    xs_status.stop_flag = TRUE;
 
     /* Free tune information */
     xs_sidplayfp_delete(&xs_status);
@@ -264,59 +247,15 @@ xs_err_exit:
 }
 
 
-/*
- * Stop playing
- * Here we set the playing status to stop and wait for playing
- * thread to shut down. In any "correctly" done plugin, this is
- * also the function where you close the output-plugin, but since
- * XMMS-SID has special behaviour (audio opened/closed in the
- * playing thread), we don't do that here.
- *
- * Finally tune and other memory allocations are free'd.
- */
-void xs_stop(InputPlayback *pb)
-{
-    /* Lock xs_status and stop playing thread */
-    pthread_mutex_lock(&xs_status_mutex);
-
-    if (! xs_status.stop_flag)
-    {
-        xs_status.stop_flag = TRUE;
-        pb->output->abort_write ();
-    }
-
-    pthread_mutex_unlock(&xs_status_mutex);
-}
-
-
-/*
- * Pause/unpause the playing
- */
-void xs_pause (InputPlayback * pb, bool_t pauseState)
-{
-    pthread_mutex_lock(&xs_status_mutex);
-    pb->output->pause(pauseState);
-    pthread_mutex_unlock(&xs_status_mutex);
-}
-
-
 /*
  * Return song information Tuple
  */
 static void xs_get_song_tuple_info(Tuple *tuple, xs_tuneinfo_t *info, int subTune)
 {
-    char *tmpStr;
-
-    tmpStr = str_to_utf8(info->sidName);
-    tuple_set_str(tuple, FIELD_TITLE, NULL, tmpStr);
-    free(tmpStr);
-    tmpStr = str_to_utf8(info->sidComposer);
-    tuple_set_str(tuple, FIELD_ARTIST, NULL, tmpStr);
-    free(tmpStr);
-    tmpStr = str_to_utf8(info->sidCopyright);
-    tuple_set_str(tuple, FIELD_COPYRIGHT, NULL, tmpStr);
-    free(tmpStr);
-    tuple_set_str(tuple, FIELD_CODEC, NULL, info->sidFormat);
+    tuple_set_str(tuple, FIELD_TITLE, info->sidName);
+    tuple_set_str(tuple, FIELD_ARTIST, info->sidComposer);
+    tuple_set_str(tuple, FIELD_COPYRIGHT, info->sidCopyright);
+    tuple_set_str(tuple, FIELD_CODEC, info->sidFormat);
 
 #if 0
     switch (info->sidModel) {
@@ -334,7 +273,7 @@ static void xs_get_song_tuple_info(Tuple *tuple, xs_tuneinfo_t *info, int subTun
 
     if (subTune > 0 && subTune <= info->nsubTunes) {
         int tmpInt = info->subTunes[subTune - 1].tuneLength;
-        tuple_set_int(tuple, FIELD_LENGTH, NULL, (tmpInt < 0) ? -1 : tmpInt * 1000);
+        tuple_set_int(tuple, FIELD_LENGTH, (tmpInt < 0) ? -1 : tmpInt * 1000);
 
 #if 0
         tmpInt = info->subTunes[subTune - 1].tuneSpeed;
@@ -358,9 +297,9 @@ static void xs_get_song_tuple_info(Tuple *tuple, xs_tuneinfo_t *info, int subTun
     } else
         subTune = 1;
 
-    tuple_set_int(tuple, FIELD_SUBSONG_NUM, NULL, info->nsubTunes);
-    tuple_set_int(tuple, FIELD_SUBSONG_ID, NULL, subTune);
-    tuple_set_int(tuple, FIELD_TRACK_NUMBER, NULL, subTune);
+    tuple_set_int(tuple, FIELD_SUBSONG_NUM, info->nsubTunes);
+    tuple_set_int(tuple, FIELD_SUBSONG_ID, subTune);
+    tuple_set_int(tuple, FIELD_TRACK_NUMBER, subTune);
 }
 
 
@@ -394,7 +333,7 @@ Tuple * xs_probe_for_tuple(const char *filename, VFSFile *fd)
 
     /* Get information from URL */
     tuple = tuple_new_from_filename (filename);
-    tune = tuple_get_int (tuple, FIELD_SUBSONG_NUM, NULL);
+    tune = tuple_get_int (tuple, FIELD_SUBSONG_NUM);
 
     /* Get tune information from emulation engine */
     pthread_mutex_lock(&xs_status_mutex);
@@ -426,8 +365,6 @@ AUD_INPUT_PLUGIN
     .init = xs_init,                    /* Initialization */
     .cleanup = xs_close,                /* Cleanup */
     .play = xs_play_file,               /* Play given file */
-    .stop = xs_stop,                    /* Stop playing */
-    .pause = xs_pause,                  /* Pause playing */
     .probe_for_tuple = xs_probe_for_tuple,
 
     .extensions = xs_sid_fmts,          /* File ext assist */
