@@ -40,13 +40,12 @@
 #include "actions-mainwin.h"
 #include "actions-playlist.h"
 #include "dnd.h"
+#include "menus.h"
 #include "plugin.h"
 #include "skins_cfg.h"
 #include "ui_equalizer.h"
-#include "ui_hints.h"
 #include "ui_main.h"
 #include "ui_main_evlisteners.h"
-#include "ui_manager.h"
 #include "ui_playlist.h"
 #include "ui_skinned_button.h"
 #include "ui_skinned_horizontal_slider.h"
@@ -59,6 +58,7 @@
 #include "ui_skinned_window.h"
 #include "ui_vis.h"
 #include "util.h"
+#include "view.h"
 
 #define SEEK_THRESHOLD 200 /* milliseconds */
 #define SEEK_TIMEOUT 100 /* milliseconds */
@@ -76,7 +76,7 @@ static GtkWidget *mainwin_rew, *mainwin_fwd;
 static GtkWidget *mainwin_eject;
 static GtkWidget *mainwin_play, *mainwin_pause, *mainwin_stop;
 
-static GtkWidget *mainwin_shuffle, *mainwin_repeat;
+GtkWidget *mainwin_shuffle, *mainwin_repeat;
 GtkWidget *mainwin_eq, *mainwin_pl;
 
 GtkWidget *mainwin_info;
@@ -94,7 +94,7 @@ GtkWidget *mainwin_svis;
 
 GtkWidget *mainwin_sposition = NULL;
 
-static GtkWidget *mainwin_menurow;
+GtkWidget *mainwin_menurow;
 static GtkWidget *mainwin_volume, *mainwin_balance;
 GtkWidget *mainwin_position;
 
@@ -105,82 +105,51 @@ static GtkWidget *mainwin_sstop, *mainwin_sfwd, *mainwin_seject, *mainwin_about;
 static gboolean mainwin_info_text_locked = FALSE;
 static guint mainwin_volume_release_timeout = 0;
 
-static void change_timer_mode(void);
 static void mainwin_position_motion_cb (void);
 static void mainwin_position_release_cb (void);
 static void mainwin_set_volume_diff (gint diff);
 
 static void format_time (gchar buf[7], gint time, gint length)
 {
-    if (config.timer_mode == TIMER_REMAINING && length > 0)
+    bool_t zero = aud_get_bool (NULL, "leading_zero");
+    bool_t remaining = aud_get_bool ("skins", "show_remaining_time");
+
+    if (remaining && length > 0)
     {
-        if (length - time < 60000)         /* " -0:SS" */
-            snprintf (buf, 7, " -0:%02d", (length - time) / 1000);
-        else if (length - time < 6000000)  /* "-MM:SS" */
-            snprintf (buf, 7, "%3d:%02d", (time - length) / 60000, (length - time) / 1000 % 60);
-        else                               /* "-HH:MM" */
-            snprintf (buf, 7, "%3d:%02d", (time - length) / 3600000, (length - time) / 60000 % 60);
+        time = (length - time) / 1000;
+
+        if (time < 60)
+            snprintf (buf, 7, zero ? "-00:%02d" : " -0:%02d", time);
+        else if (time < 6000)
+            snprintf (buf, 7, zero ? "%03d:%02d" : "%3d:%02d", -time / 60, time % 60);
+        else
+            snprintf (buf, 7, "%3d:%02d", -time / 3600, time / 60 % 60);
     }
     else
     {
-        if (time < 60000000)  /* MMM:SS */
-            snprintf (buf, 7, "%3d:%02d", time / 60000, time / 1000 % 60);
-        else                  /* HHH:MM */
-            snprintf (buf, 7, "%3d:%02d", time / 3600000, time / 60000 % 60);
+        time /= 1000;
+
+        if (time < 6000)
+            snprintf (buf, 7, zero ? " %02d:%02d" : " %2d:%02d", time / 60, time % 60);
+        else if (time < 60000)
+            snprintf (buf, 7, "%3d:%02d", time / 60, time % 60);
+        else
+            snprintf (buf, 7, "%3d:%02d", time / 3600, time / 60 % 60);
     }
-
-    buf[3] = 0;
-}
-
-static void
-mainwin_set_shade(gboolean shaded)
-{
-    GtkAction *action = gtk_action_group_get_action(toggleaction_group_others,
-                                                    "roll up player");
-    gtk_toggle_action_set_active( GTK_TOGGLE_ACTION(action) , shaded );
 }
 
 void mainwin_set_shape (void)
 {
-    gint id = config.player_shaded ? SKIN_MASK_MAIN_SHADE : SKIN_MASK_MAIN;
+    int id = aud_get_bool ("skins", "player_shaded") ? SKIN_MASK_MAIN_SHADE : SKIN_MASK_MAIN;
     gtk_widget_shape_combine_region (mainwin, active_skin->masks[id]);
 }
 
-static void mainwin_vis_set_type (VisType mode)
-{
-    GtkAction *action;
-
-    switch ( mode )
-    {
-        case VIS_ANALYZER:
-            action = gtk_action_group_get_action(radioaction_group_vismode,
-                                                 "vismode analyzer");
-            break;
-        case VIS_SCOPE:
-            action = gtk_action_group_get_action(radioaction_group_vismode,
-                                                 "vismode scope");
-            break;
-        case VIS_VOICEPRINT:
-            action = gtk_action_group_get_action(radioaction_group_vismode,
-                                                 "vismode voiceprint");
-            break;
-        case VIS_OFF:
-        default:
-            action = gtk_action_group_get_action(radioaction_group_vismode,
-                                                 "vismode off");
-            break;
-    }
-
-    gtk_toggle_action_set_active( GTK_TOGGLE_ACTION(action) , TRUE );
-}
-
 static void
 mainwin_menubtn_cb(void)
 {
     gint x, y;
     gtk_window_get_position(GTK_WINDOW(mainwin), &x, &y);
-    ui_popup_menu_show (UI_MENU_MAIN, x + 6, y + MAINWIN_SHADED_HEIGHT, FALSE,
-     FALSE, 1, GDK_CURRENT_TIME);
+    menu_popup (UI_MENU_MAIN, x + 6, y + MAINWIN_SHADED_HEIGHT, FALSE, FALSE, 1, GDK_CURRENT_TIME);
 }
 
 static void mainwin_minimize_cb (void)
@@ -194,27 +163,7 @@ static void mainwin_minimize_cb (void)
 static void
 mainwin_shade_toggle(void)
 {
-    mainwin_set_shade(!config.player_shaded);
-}
-
-static gboolean mainwin_vis_cb (GtkWidget * widget, GdkEventButton * event)
-{
-    if (event->button == 1) {
-        config.vis_type++;
-
-        if (config.vis_type > VIS_OFF)
-            config.vis_type = VIS_ANALYZER;
-
-        ui_vis_clear_data (mainwin_vis);
-        ui_svis_clear_data (mainwin_svis);
-
-        mainwin_vis_set_type(config.vis_type);
-    }
-    else if (event->button == 3)
-        ui_popup_menu_show(UI_MENU_VISUALIZATION, event->x_root, event->y_root,
-         FALSE, FALSE, 3, event->time);
-
-    return TRUE;
+    view_set_player_shaded (! aud_get_bool ("skins", "player_shaded"));
 }
 
 static gchar *mainwin_tb_old_text = NULL;
@@ -259,13 +208,6 @@ static void mainwin_set_info_text (const gchar * text)
         textbox_set_text (mainwin_info, text);
 }
 
-static gboolean status_message_enabled;
-
-void mainwin_enable_status_message (gboolean enable)
-{
-    status_message_enabled = enable;
-}
-
 static gint status_message_source = 0;
 
 static gboolean clear_status_message (void * unused)
@@ -277,9 +219,6 @@ static gboolean clear_status_message (void * unused)
 
 void mainwin_show_status_message (const gchar * message)
 {
-    if (! status_message_enabled)
-        return;
-
     if (status_message_source)
         g_source_remove (status_message_source);
 
@@ -308,15 +247,18 @@ mainwin_set_song_title(const gchar * title)
 
 static void setup_widget (GtkWidget * widget, gint x, gint y, gboolean show)
 {
-    GtkRequisition size;
-    gtk_widget_get_preferred_size (widget, & size, NULL);
-
     /* leave no-show-all widgets alone (they are shown/hidden elsewhere) */
     if (! gtk_widget_get_no_show_all (widget))
     {
+        int width, height;
+
+        /* use get_size_request(), not get_preferred_size() */
+        /* get_preferred_size() will return 0x0 for hidden widgets */
+        gtk_widget_get_size_request (widget, & width, & height);
+
         /* hide widgets that are outside the window boundary */
-        if (x < 0 || x + size.width > active_skin->properties.mainwin_width ||
-         y < 0 || y + size.height > active_skin->properties.mainwin_height)
+        if (x < 0 || x + width > active_skin->properties.mainwin_width ||
+         y < 0 || y + height > active_skin->properties.mainwin_height)
             show = FALSE;
 
         gtk_widget_set_visible (widget, show);
@@ -365,12 +307,14 @@ void mainwin_refresh_hints (void)
     setup_widget (mainwin_shade, p->mainwin_shade_x, p->mainwin_shade_y, TRUE);
     setup_widget (mainwin_close, p->mainwin_close_x, p->mainwin_close_y, TRUE);
 
-    if (config.player_shaded)
+    if (aud_get_bool ("skins", "player_shaded"))
         window_set_size (mainwin, MAINWIN_SHADED_WIDTH, MAINWIN_SHADED_HEIGHT);
     else
         window_set_size (mainwin, p->mainwin_width, p->mainwin_height);
 }
 
+/* note that the song info is not translated since it is displayed using
+ * the skinned bitmap font, which supports only the English alphabet */
 void mainwin_set_song_info (gint bitrate, gint samplerate, gint channels)
 {
     gchar scratch[32];
@@ -399,23 +343,22 @@ void mainwin_set_song_info (gint bitrate, gint samplerate, gint channels)
     ui_skinned_monostereo_set_num_channels (mainwin_monostereo, channels);
 
     if (bitrate > 0)
-        snprintf (scratch, sizeof scratch, "%d %s", bitrate / 1000, _("kbps"));
+        snprintf (scratch, sizeof scratch, "%d kbps", bitrate / 1000);
     else
         scratch[0] = 0;
 
     if (samplerate > 0)
     {
         length = strlen (scratch);
-        snprintf (scratch + length, sizeof scratch - length, "%s%d %s", length ?
-         ", " : "", samplerate / 1000, _("kHz"));
+        snprintf (scratch + length, sizeof scratch - length, "%s%d kHz", length ?
+         ", " : "", samplerate / 1000);
     }
 
     if (channels > 0)
     {
         length = strlen (scratch);
         snprintf (scratch + length, sizeof scratch - length, "%s%s", length ?
-         ", " : "", channels > 2 ? _("surround") : channels > 1 ? _("stereo") :
-         _("mono"));
+         ", " : "", channels > 2 ? "surround" : channels > 1 ? "stereo" : "mono");
     }
 
     textbox_set_text (mainwin_othertext, scratch);
@@ -490,16 +433,17 @@ mainwin_mouse_button_press(GtkWidget * widget,
                            GdkEventButton * event,
                            gpointer callback_data)
 {
-    if (event->button == 1 && event->type == GDK_2BUTTON_PRESS && event->y < 14)
+    if (event->button == 1 && event->type == GDK_2BUTTON_PRESS &&
+     event->window == gtk_widget_get_window (widget) && event->y < 14)
     {
-        mainwin_set_shade(!config.player_shaded);
+        mainwin_shade_toggle ();
         return TRUE;
     }
 
     if (event->button == 3)
     {
-        ui_popup_menu_show (UI_MENU_MAIN, event->x_root, event->y_root, FALSE,
-         FALSE, event->button, event->time);
+        menu_popup (UI_MENU_MAIN, event->x_root, event->y_root, FALSE, FALSE,
+         event->button, event->time);
         return TRUE;
     }
 
@@ -508,8 +452,8 @@ mainwin_mouse_button_press(GtkWidget * widget,
 
 static void mainwin_playback_rpress (GtkWidget * button, GdkEventButton * event)
 {
-    ui_popup_menu_show (UI_MENU_PLAYBACK, event->x_root, event->y_root,
-     FALSE, FALSE, event->button, event->time);
+    menu_popup (UI_MENU_PLAYBACK, event->x_root, event->y_root, FALSE, FALSE,
+     event->button, event->time);
 }
 
 gboolean mainwin_keypress (GtkWidget * widget, GdkEventKey * event,
@@ -590,22 +534,6 @@ mainwin_drag_data_received(GtkWidget * widget,
      (selection_data);
     g_return_if_fail (data);
 
-    if (str_has_prefix_nocase (data, "fonts:///"))
-    {
-        const gchar * path = data;
-        gchar *decoded = g_filename_from_uri(path, NULL, NULL);
-
-        if (decoded == NULL)
-            return;
-
-        config.playlist_font = g_strconcat(decoded, strrchr(config.playlist_font, ' '), NULL);
-        ui_skinned_playlist_set_font (playlistwin_list, config.playlist_font);
-
-        g_free(decoded);
-
-        return;
-    }
-
     if (str_has_prefix_nocase (data, "file:///"))
     {
         if (str_has_suffix_nocase (data, ".wsz\r\n") || str_has_suffix_nocase
@@ -703,13 +631,13 @@ static void mainwin_fwd_release (GtkWidget * button, GdkEventButton * event)
  {seek_release (button, event, FALSE); }
 
 static void mainwin_shuffle_cb (GtkWidget * button, GdkEventButton * event)
- {check_set (toggleaction_group_others, "playback shuffle", button_get_active (button)); }
+ {aud_set_bool (NULL, "shuffle", button_get_active (button)); }
 static void mainwin_repeat_cb (GtkWidget * button, GdkEventButton * event)
- {check_set (toggleaction_group_others, "playback repeat", button_get_active (button)); }
+ {aud_set_bool (NULL, "repeat", button_get_active (button)); }
 static void mainwin_eq_cb (GtkWidget * button, GdkEventButton * event)
- {equalizerwin_show (button_get_active (button)); }
+ {view_set_show_equalizer (button_get_active (button)); }
 static void mainwin_pl_cb (GtkWidget * button, GdkEventButton * event)
- {playlistwin_show (button_get_active (button)); }
+ {view_set_show_playlist (button_get_active (button)); }
 
 static void mainwin_spos_set_knob (void)
 {
@@ -898,31 +826,6 @@ static void mainwin_set_volume_diff (gint diff)
         g_timeout_add(700, (GSourceFunc)(mainwin_volume_release_cb), NULL);
 }
 
-static void mainwin_real_show (gboolean show)
-{
-    if (show)
-        gtk_window_present ((GtkWindow *) mainwin);
-    else
-        gtk_widget_hide (mainwin);
-}
-
-void mainwin_show (gboolean show)
-{
-    GtkAction * a;
-
-    a = gtk_action_group_get_action (toggleaction_group_others, "show player");
-
-    if (a && gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (a)) != show)
-        gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (a), show);
-    else
-    {
-        mainwin_real_show (show);
-        equalizerwin_show (config.equalizer_visible);
-        playlistwin_show (config.playlist_visible);
-        start_stop_visual (FALSE);
-    }
-}
-
 void mainwin_mr_change (MenuRowItem i)
 {
     switch (i) {
@@ -930,7 +833,7 @@ void mainwin_mr_change (MenuRowItem i)
             mainwin_lock_info_text(_("Options Menu"));
             break;
         case MENUROW_ALWAYS:
-            if (config.always_on_top)
+            if (aud_get_bool ("skins", "always_on_top"))
                 mainwin_lock_info_text(_("Disable 'Always On Top'"));
             else
                 mainwin_lock_info_text(_("Enable 'Always On Top'"));
@@ -938,12 +841,7 @@ void mainwin_mr_change (MenuRowItem i)
         case MENUROW_FILEINFOBOX:
             mainwin_lock_info_text(_("File Info Box"));
             break;
-        case MENUROW_SCALE:
-            break;
-        case MENUROW_VISUALIZATION:
-            mainwin_lock_info_text(_("Visualization Menu"));
-            break;
-        case MENUROW_NONE:
+        default:
             break;
     }
 }
@@ -952,222 +850,39 @@ void mainwin_mr_release (MenuRowItem i, GdkEventButton * event)
 {
     switch (i) {
         case MENUROW_OPTIONS:
-            ui_popup_menu_show(UI_MENU_VIEW, event->x_root, event->y_root,
-             FALSE, FALSE, 1, event->time);
+            menu_popup (UI_MENU_VIEW, event->x_root, event->y_root, FALSE, FALSE, 1, event->time);
             break;
         case MENUROW_ALWAYS:
-            gtk_toggle_action_set_active ((GtkToggleAction *)
-             gtk_action_group_get_action (toggleaction_group_others,
-             "view always on top"), config.always_on_top);
+            view_set_on_top (! aud_get_bool ("skins", "always_on_top"));
             break;
         case MENUROW_FILEINFOBOX:
             audgui_infowin_show_current ();
             break;
-        case MENUROW_SCALE:
-            break;
-        case MENUROW_VISUALIZATION:
-            ui_popup_menu_show(UI_MENU_VISUALIZATION, event->x_root,
-             event->y_root, FALSE, FALSE, 1, event->time);
-            break;
-        case MENUROW_NONE:
+        default:
             break;
     }
 
     mainwin_release_info_text();
 }
 
-static void
-set_timer_mode(TimerMode mode)
-{
-    if (mode == TIMER_ELAPSED)
-        check_set(radioaction_group_viewtime, "view time elapsed", TRUE);
-    else
-        check_set(radioaction_group_viewtime, "view time remaining", TRUE);
-}
-
 gboolean
 change_timer_mode_cb(GtkWidget *widget, GdkEventButton *event)
 {
-    if (event->button == 1) {
-        change_timer_mode();
-    } else if (event->button == 3)
+    if (event->button == 1)
+        view_set_show_remaining (! aud_get_bool ("skins", "show_remaining_time"));
+    else if (event->button == 3)
         return FALSE;
 
     return TRUE;
 }
 
-static void change_timer_mode(void) {
-    if (config.timer_mode == TIMER_ELAPSED)
-        set_timer_mode(TIMER_REMAINING);
-    else
-        set_timer_mode(TIMER_ELAPSED);
-    if (aud_drct_get_playing())
-        mainwin_update_song_info();
-}
-
-void
-mainwin_setup_menus(void)
-{
-    set_timer_mode(config.timer_mode);
-
-    /* View menu */
-
-    check_set(toggleaction_group_others, "view always on top", config.always_on_top);
-    check_set(toggleaction_group_others, "view put on all workspaces", config.sticky);
-    check_set(toggleaction_group_others, "roll up player", config.player_shaded);
-    check_set(toggleaction_group_others, "roll up playlist editor", config.playlist_shaded);
-    check_set(toggleaction_group_others, "roll up equalizer", config.equalizer_shaded);
-
-    mainwin_enable_status_message (FALSE);
-
-    /* Songname menu */
-
-    check_set(toggleaction_group_others, "autoscroll songname", config.autoscroll);
-    check_set (toggleaction_group_others, "stop after current song",
-     aud_get_bool (NULL, "stop_after_current_song"));
-
-    /* Playback menu */
-
-    check_set (toggleaction_group_others, "playback repeat", aud_get_bool (NULL, "repeat"));
-    check_set (toggleaction_group_others, "playback shuffle", aud_get_bool (NULL, "shuffle"));
-    check_set (toggleaction_group_others, "playback no playlist advance",
-     aud_get_bool (NULL, "no_playlist_advance"));
-
-    mainwin_enable_status_message (TRUE);
-
-    /* Visualization menu */
-
-    switch ( config.vis_type )
-    {
-        case VIS_ANALYZER:
-            check_set(radioaction_group_vismode, "vismode analyzer", TRUE);
-            break;
-        case VIS_SCOPE:
-            check_set(radioaction_group_vismode, "vismode scope", TRUE);
-            break;
-        case VIS_VOICEPRINT:
-            check_set(radioaction_group_vismode, "vismode voiceprint", TRUE);
-            break;
-        case VIS_OFF:
-        default:
-            check_set(radioaction_group_vismode, "vismode off", TRUE);
-            break;
-    }
-
-    switch ( config.analyzer_mode )
-    {
-        case ANALYZER_FIRE:
-            check_set(radioaction_group_anamode, "anamode fire", TRUE);
-            break;
-        case ANALYZER_VLINES:
-            check_set(radioaction_group_anamode, "anamode vertical lines", TRUE);
-            break;
-        case ANALYZER_NORMAL:
-        default:
-            check_set(radioaction_group_anamode, "anamode normal", TRUE);
-            break;
-    }
-
-    switch ( config.analyzer_type )
-    {
-        case ANALYZER_BARS:
-            check_set(radioaction_group_anatype, "anatype bars", TRUE);
-            break;
-        case ANALYZER_LINES:
-        default:
-            check_set(radioaction_group_anatype, "anatype lines", TRUE);
-            break;
-    }
-
-    check_set(toggleaction_group_others, "anamode peaks", config.analyzer_peaks );
-
-    switch ( config.scope_mode )
-    {
-        case SCOPE_LINE:
-            check_set(radioaction_group_scomode, "scomode line", TRUE);
-            break;
-        case SCOPE_SOLID:
-            check_set(radioaction_group_scomode, "scomode solid", TRUE);
-            break;
-        case SCOPE_DOT:
-        default:
-            check_set(radioaction_group_scomode, "scomode dot", TRUE);
-            break;
-    }
-
-    switch ( config.voiceprint_mode )
-    {
-        case VOICEPRINT_FIRE:
-            check_set(radioaction_group_vprmode, "vprmode fire", TRUE);
-            break;
-        case VOICEPRINT_ICE:
-            check_set(radioaction_group_vprmode, "vprmode ice", TRUE);
-            break;
-        case VOICEPRINT_NORMAL:
-        default:
-            check_set(radioaction_group_vprmode, "vprmode normal", TRUE);
-            break;
-    }
-
-    switch ( config.vu_mode )
-    {
-        case VU_SMOOTH:
-            check_set(radioaction_group_wshmode, "wshmode smooth", TRUE);
-            break;
-        case VU_NORMAL:
-        default:
-            check_set(radioaction_group_wshmode, "wshmode normal", TRUE);
-            break;
-    }
-
-    switch ( config.analyzer_falloff )
-    {
-        case FALLOFF_SLOW:
-            check_set(radioaction_group_anafoff, "anafoff slow", TRUE);
-            break;
-        case FALLOFF_MEDIUM:
-            check_set(radioaction_group_anafoff, "anafoff medium", TRUE);
-            break;
-        case FALLOFF_FAST:
-            check_set(radioaction_group_anafoff, "anafoff fast", TRUE);
-            break;
-        case FALLOFF_FASTEST:
-            check_set(radioaction_group_anafoff, "anafoff fastest", TRUE);
-            break;
-        case FALLOFF_SLOWEST:
-        default:
-            check_set(radioaction_group_anafoff, "anafoff slowest", TRUE);
-            break;
-    }
-
-    switch ( config.peaks_falloff )
-    {
-        case FALLOFF_SLOW:
-            check_set(radioaction_group_peafoff, "peafoff slow", TRUE);
-            break;
-        case FALLOFF_MEDIUM:
-            check_set(radioaction_group_peafoff, "peafoff medium", TRUE);
-            break;
-        case FALLOFF_FAST:
-            check_set(radioaction_group_peafoff, "peafoff fast", TRUE);
-            break;
-        case FALLOFF_FASTEST:
-            check_set(radioaction_group_peafoff, "peafoff fastest", TRUE);
-            break;
-        case FALLOFF_SLOWEST:
-        default:
-            check_set(radioaction_group_peafoff, "peafoff slowest", TRUE);
-            break;
-    }
-}
-
 static gboolean mainwin_info_button_press (GtkWidget * widget, GdkEventButton *
  event)
 {
     if (event->type == GDK_BUTTON_PRESS && event->button == 3)
     {
-        ui_popup_menu_show (UI_MENU_SONGNAME, event->x_root, event->y_root,
-         FALSE, FALSE, event->button, event->time);
+        menu_popup (UI_MENU_PLAYBACK, event->x_root, event->y_root, FALSE,
+         FALSE, event->button, event->time);
         return TRUE;
     }
 
@@ -1232,10 +947,12 @@ mainwin_create_widgets(void)
 
     mainwin_shuffle = button_new_toggle (46, 15, 28, 0, 28, 15, 28, 30, 28, 45, SKIN_SHUFREP, SKIN_SHUFREP);
     window_put_widget (mainwin, FALSE, mainwin_shuffle, 164, 89);
+    button_set_active (mainwin_shuffle, aud_get_bool (NULL, "shuffle"));
     button_on_release (mainwin_shuffle, mainwin_shuffle_cb);
 
     mainwin_repeat = button_new_toggle (28, 15, 0, 0, 0, 15, 0, 30, 0, 45, SKIN_SHUFREP, SKIN_SHUFREP);
     window_put_widget (mainwin, FALSE, mainwin_repeat, 210, 89);
+    button_set_active (mainwin_repeat, aud_get_bool (NULL, "repeat"));
     button_on_release (mainwin_repeat, mainwin_repeat_cb);
 
     mainwin_eq = button_new_toggle (23, 12, 0, 61, 46, 61, 0, 73, 46, 73, SKIN_SHUFREP, SKIN_SHUFREP);
@@ -1246,8 +963,11 @@ mainwin_create_widgets(void)
     window_put_widget (mainwin, FALSE, mainwin_pl, 242, 58);
     button_on_release (mainwin_pl, mainwin_pl_cb);
 
+    char * font = aud_get_str ("skins", "mainwin_font");
     mainwin_info = textbox_new (153, "", config.mainwin_use_bitmapfont ? NULL :
-     config.mainwin_font, config.autoscroll);
+     font, config.autoscroll);
+    str_unref (font);
+
     window_put_widget (mainwin, FALSE, mainwin_info, 112, 27);
     g_signal_connect (mainwin_info, "button-press-event", (GCallback)
      mainwin_info_button_press, NULL);
@@ -1306,7 +1026,6 @@ mainwin_create_widgets(void)
 
     mainwin_vis = ui_vis_new ();
     window_put_widget (mainwin, FALSE, mainwin_vis, 24, 43);
-    g_signal_connect(mainwin_vis, "button-press-event", G_CALLBACK(mainwin_vis_cb), NULL);
 
     mainwin_position = hslider_new (0, 219, SKIN_POSBAR, 248, 10, 0, 0, 29, 10, 248, 0, 278, 0);
     window_put_widget (mainwin, FALSE, mainwin_position, 16, 72);
@@ -1357,7 +1076,6 @@ mainwin_create_widgets(void)
 
     mainwin_svis = ui_svis_new ();
     window_put_widget (mainwin, TRUE, mainwin_svis, 79, 5);
-    g_signal_connect(mainwin_svis, "button-press-event", G_CALLBACK(mainwin_vis_cb), NULL);
 
     mainwin_sposition = hslider_new (1, 13, SKIN_TITLEBAR, 17, 7, 0, 36, 3, 7, 17, 36, 17, 36);
     window_put_widget (mainwin, TRUE, mainwin_sposition, 226, 4);
@@ -1386,7 +1104,7 @@ static void show_widgets (void)
     gtk_widget_set_no_show_all (mainwin_position, TRUE);
     gtk_widget_set_no_show_all (mainwin_sposition, TRUE);
 
-    window_set_shaded (mainwin, config.player_shaded);
+    window_set_shaded (mainwin, aud_get_bool ("skins", "player_shaded"));
     window_show_all (mainwin);
 }
 
@@ -1394,47 +1112,33 @@ static gboolean state_cb (GtkWidget * widget, GdkEventWindowState * event,
  void * unused)
 {
     if (event->changed_mask & GDK_WINDOW_STATE_STICKY)
-    {
-        config.sticky = (event->new_window_state & GDK_WINDOW_STATE_STICKY) ?
-         TRUE : FALSE;
-
-        GtkToggleAction * action = (GtkToggleAction *)
-         gtk_action_group_get_action (toggleaction_group_others,
-         "view put on all workspaces");
-        gtk_toggle_action_set_active (action, config.sticky);
-    }
+        view_set_sticky (!! (event->new_window_state & GDK_WINDOW_STATE_STICKY));
 
     if (event->changed_mask & GDK_WINDOW_STATE_ABOVE)
-    {
-        config.always_on_top = (event->new_window_state &
-         GDK_WINDOW_STATE_ABOVE) ? TRUE : FALSE;
-
-        GtkToggleAction * action = (GtkToggleAction *)
-         gtk_action_group_get_action (toggleaction_group_others,
-         "view always on top");
-        gtk_toggle_action_set_active (action, config.always_on_top);
-    }
+        view_set_on_top (!! (event->new_window_state & GDK_WINDOW_STATE_ABOVE));
 
     return TRUE;
 }
 
 static void mainwin_draw (GtkWidget * window, cairo_t * cr)
 {
-    gint width = config.player_shaded ? MAINWIN_SHADED_WIDTH : active_skin->properties.mainwin_width;
-    gint height = config.player_shaded ? MAINWIN_SHADED_HEIGHT : active_skin->properties.mainwin_height;
+    bool_t shaded = aud_get_bool ("skins", "player_shaded");
+    int width = shaded ? MAINWIN_SHADED_WIDTH : active_skin->properties.mainwin_width;
+    int height = shaded ? MAINWIN_SHADED_HEIGHT : active_skin->properties.mainwin_height;
 
     skin_draw_pixbuf (cr, SKIN_MAIN, 0, 0, 0, 0, width, height);
-    skin_draw_mainwin_titlebar (cr, config.player_shaded, TRUE);
+    skin_draw_mainwin_titlebar (cr, shaded, TRUE);
 }
 
 static void
 mainwin_create_window(void)
 {
-    gint width = config.player_shaded ? MAINWIN_SHADED_WIDTH : active_skin->properties.mainwin_width;
-    gint height = config.player_shaded ? MAINWIN_SHADED_HEIGHT : active_skin->properties.mainwin_height;
+    bool_t shaded = aud_get_bool ("skins", "player_shaded");
+    int width = shaded ? MAINWIN_SHADED_WIDTH : active_skin->properties.mainwin_width;
+    int height = shaded ? MAINWIN_SHADED_HEIGHT : active_skin->properties.mainwin_height;
 
     mainwin = window_new (& config.player_x, & config.player_y, width, height,
-     TRUE, config.player_shaded, mainwin_draw);
+     TRUE, shaded, mainwin_draw);
 
     gtk_window_set_title(GTK_WINDOW(mainwin), _("Audacious"));
 
@@ -1473,12 +1177,10 @@ mainwin_create(void)
 {
     mainwin_create_window();
 
-    gtk_window_add_accel_group( GTK_WINDOW(mainwin) , ui_manager_get_accel_group() );
+    gtk_window_add_accel_group ((GtkWindow *) mainwin, menu_get_accel_group ());
 
     mainwin_create_widgets();
     show_widgets ();
-
-    status_message_enabled = TRUE;
 }
 
 static void mainwin_update_volume (void)
@@ -1553,165 +1255,6 @@ void mainwin_update_song_info (void)
     mainwin_update_time_slider (time, length);
 }
 
-/* toggleactionentries actions */
-
-void action_anamode_peaks (GtkToggleAction * action)
-{
-    config.analyzer_peaks = gtk_toggle_action_get_active (action);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_autoscroll_songname (GtkToggleAction * action)
-{
-    config.autoscroll = gtk_toggle_action_get_active (action);
-    textbox_set_scroll (mainwin_info, config.autoscroll);
-    textbox_set_scroll (playlistwin_sinfo, config.autoscroll);
-}
-
-void action_playback_noplaylistadvance (GtkToggleAction * action)
-{
-    aud_set_bool (NULL, "no_playlist_advance", gtk_toggle_action_get_active (action));
-
-    if (gtk_toggle_action_get_active (action))
-        mainwin_show_status_message (_("Single mode."));
-    else
-        mainwin_show_status_message (_("Playlist mode."));
-}
-
-void action_playback_repeat (GtkToggleAction * action)
-{
-    aud_set_bool (NULL, "repeat", gtk_toggle_action_get_active (action));
-    button_set_active (mainwin_repeat, gtk_toggle_action_get_active (action));
-}
-
-void action_playback_shuffle (GtkToggleAction * action)
-{
-    aud_set_bool (NULL, "shuffle", gtk_toggle_action_get_active (action));
-    button_set_active (mainwin_shuffle, gtk_toggle_action_get_active (action));
-}
-
-void action_stop_after_current_song (GtkToggleAction * action)
-{
-    gboolean active = gtk_toggle_action_get_active (action);
-
-    if (active != aud_get_bool (NULL, "stop_after_current_song"))
-    {
-        if (active)
-            mainwin_show_status_message (_("Stopping after song."));
-        else
-            mainwin_show_status_message (_("Not stopping after song."));
-
-        aud_set_bool (NULL, "stop_after_current_song", active);
-    }
-}
-
-void action_view_always_on_top (GtkToggleAction * action)
-{
-    gboolean on_top = gtk_toggle_action_get_active (action);
-
-    if (config.always_on_top != on_top)
-    {
-        config.always_on_top = on_top;
-        ui_skinned_menurow_update (mainwin_menurow);
-        hint_set_always (config.always_on_top);
-    }
-}
-
-void action_view_on_all_workspaces (GtkToggleAction * action)
-{
-    gboolean sticky = gtk_toggle_action_get_active (action);
-
-    if (config.sticky != sticky)
-    {
-        config.sticky = sticky;
-        hint_set_sticky (sticky);
-    }
-}
-
-void action_roll_up_player (GtkToggleAction * action)
-{
-    config.player_shaded = gtk_toggle_action_get_active (action);
-    window_set_shaded (mainwin, config.player_shaded);
-
-    gint width = config.player_shaded ? MAINWIN_SHADED_WIDTH : active_skin->properties.mainwin_width;
-    gint height = config.player_shaded ? MAINWIN_SHADED_HEIGHT : active_skin->properties.mainwin_height;
-    window_set_size (mainwin, width, height);
-    mainwin_set_shape ();
-}
-
-void action_show_player (GtkToggleAction * action)
-{
-    mainwin_show(gtk_toggle_action_get_active(action));
-}
-
-
-/* radioactionentries actions (one callback for each radio group) */
-
-void action_anafoff (GtkAction * action, GtkRadioAction * current)
-{
-    config.analyzer_falloff = gtk_radio_action_get_current_value (current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_anamode (GtkAction * action, GtkRadioAction * current)
-{
-    config.analyzer_mode = gtk_radio_action_get_current_value (current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_anatype (GtkAction * action, GtkRadioAction * current)
-{
-    config.analyzer_type = gtk_radio_action_get_current_value (current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_peafoff (GtkAction * action, GtkRadioAction * current)
-{
-    config.peaks_falloff = gtk_radio_action_get_current_value (current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_scomode (GtkAction * action, GtkRadioAction * current)
-{
-    config.scope_mode = gtk_radio_action_get_current_value (current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_vismode (GtkAction * action, GtkRadioAction * current)
-{
-    config.vis_type = gtk_radio_action_get_current_value (current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-
-    start_stop_visual (FALSE);
-}
-
-void action_vprmode (GtkAction * action, GtkRadioAction * current)
-{
-    config.voiceprint_mode = gtk_radio_action_get_current_value(current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_wshmode (GtkAction * action, GtkRadioAction * current)
-{
-    config.vu_mode = gtk_radio_action_get_current_value(current);
-    ui_vis_clear_data (mainwin_vis);
-    ui_svis_clear_data (mainwin_svis);
-}
-
-void action_viewtime (GtkAction * action, GtkRadioAction * current)
-{
-    config.timer_mode = gtk_radio_action_get_current_value (current);
-}
-
-
 /* actionentries actions */
 
 void action_play_file (void)
