@@ -34,22 +34,24 @@
 #include <audacious/i18n.h>
 #include <audacious/misc.h>
 #include <audacious/playlist.h>
+#include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 #include <libaudgui/libaudgui.h>
 
 #include "actions-mainwin.h"
 #include "dnd.h"
 #include "drag-handle.h"
+#include "menus.h"
 #include "plugin.h"
 #include "skins_cfg.h"
 #include "ui_main.h"
-#include "ui_manager.h"
 #include "ui_playlist.h"
 #include "ui_skinned_button.h"
 #include "ui_skinned_playlist.h"
 #include "ui_skinned_playlist_slider.h"
 #include "ui_skinned_textbox.h"
 #include "ui_skinned_window.h"
+#include "view.h"
 
 #define PLAYLISTWIN_MIN_WIDTH           MAINWIN_WIDTH
 #define PLAYLISTWIN_MIN_HEIGHT          MAINWIN_HEIGHT
@@ -57,6 +59,8 @@
 #define PLAYLISTWIN_HEIGHT_SNAP         29
 #define PLAYLISTWIN_SHADED_HEIGHT       MAINWIN_SHADED_HEIGHT
 
+#define APPEND(b, ...) snprintf (b + strlen (b), sizeof b - strlen (b), __VA_ARGS__)
+
 gint active_playlist = -1, active_length = 0;
 gchar * active_title = NULL;
 
@@ -88,31 +92,12 @@ static gboolean song_changed;
 
 static void playlistwin_update_info (void)
 {
-    gchar *text, *sel_text, *tot_text;
-    gint64 selection, total;
+    char s1[16], s2[16];
+    audgui_format_time (s1, sizeof s1, aud_playlist_get_selected_length (active_playlist));
+    audgui_format_time (s2, sizeof s2, aud_playlist_get_total_length (active_playlist));
 
-    total = aud_playlist_get_total_length (active_playlist) / 1000;
-    selection = aud_playlist_get_selected_length (active_playlist) / 1000;
-
-    if (selection >= 3600)
-        sel_text = g_strdup_printf ("%" PRId64 ":%02" PRId64 ":%02" PRId64,
-        selection / 3600, selection / 60 % 60, selection % 60);
-    else
-        sel_text = g_strdup_printf ("%" PRId64 ":%02" PRId64,
-        selection / 60, selection % 60);
-
-    if (total >= 3600)
-        tot_text = g_strdup_printf ("%" PRId64 ":%02" PRId64 ":%02" PRId64,
-        total / 3600, total / 60 % 60, total % 60);
-    else
-        tot_text = g_strdup_printf ("%" PRId64 ":%02" PRId64,
-        total / 60, total % 60);
-
-    text = g_strconcat(sel_text, "/", tot_text, NULL);
-    textbox_set_text (playlistwin_info, text);
-    g_free(text);
-    g_free(tot_text);
-    g_free(sel_text);
+    SCONCAT3 (buf, s1, "/", s2);
+    textbox_set_text (playlistwin_info, buf);
 }
 
 static void update_rollup_text (void)
@@ -128,16 +113,18 @@ static void update_rollup_text (void)
         gint length = aud_playlist_entry_get_length (playlist, entry, TRUE);
 
         if (aud_get_bool (NULL, "show_numbers_in_pl"))
-            snprintf (scratch, sizeof scratch, "%d. ", 1 + entry);
+            APPEND (scratch, "%d. ", 1 + entry);
 
         gchar * title = aud_playlist_entry_get_title (playlist, entry, TRUE);
-        snprintf (scratch + strlen (scratch), sizeof scratch - strlen (scratch),
-         "%s", title);
+        APPEND (scratch, "%s", title);
         str_unref (title);
 
         if (length > 0)
-            snprintf (scratch + strlen (scratch), sizeof scratch - strlen
-             (scratch), " (%d:%02d)", length / 60000, length / 1000 % 60);
+        {
+            char buf[16];
+            audgui_format_time (buf, sizeof buf, length);
+            APPEND (scratch, " (%s)", buf);
+        }
     }
 
     textbox_set_text (playlistwin_sinfo, scratch);
@@ -156,19 +143,10 @@ void playlistwin_update (void)
         real_update ();
 }
 
-static void
-playlistwin_set_shade_menu(gboolean shaded)
-{
-    GtkAction *action = gtk_action_group_get_action(
-      toggleaction_group_others , "roll up playlist editor" );
-    gtk_toggle_action_set_active( GTK_TOGGLE_ACTION(action) , shaded );
-    playlistwin_update ();
-}
-
 static void
 playlistwin_shade_toggle(void)
 {
-    playlistwin_set_shade_menu(!config.playlist_shaded);
+    view_set_playlist_shaded (! aud_get_bool ("skins", "playlist_shaded"));
 }
 
 static void playlistwin_scroll (gboolean up)
@@ -216,8 +194,8 @@ static void copy_selected_to_new (gint playlist)
     {
         if (aud_playlist_entry_get_selected (playlist, entry))
         {
-            index_append (filenames, aud_playlist_entry_get_filename (playlist, entry));
-            index_append (tuples, aud_playlist_entry_get_tuple (playlist, entry, TRUE));
+            index_insert (filenames, -1, aud_playlist_entry_get_filename (playlist, entry));
+            index_insert (tuples, -1, aud_playlist_entry_get_tuple (playlist, entry, TRUE));
         }
     }
 
@@ -243,11 +221,11 @@ playlistwin_select_search(void)
     searchdlg_win = gtk_dialog_new_with_buttons(
       _("Search entries in active playlist") , GTK_WINDOW(mainwin) ,
       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT ,
-      GTK_STOCK_CANCEL , GTK_RESPONSE_REJECT , GTK_STOCK_OK , GTK_RESPONSE_ACCEPT , NULL );
+      _("Cancel") , GTK_RESPONSE_REJECT , _("Search") , GTK_RESPONSE_ACCEPT , NULL );
 
     /* help text and logo */
     searchdlg_hbox = gtk_box_new( GTK_ORIENTATION_HORIZONTAL , 4 );
-    searchdlg_logo = gtk_image_new_from_stock( GTK_STOCK_FIND , GTK_ICON_SIZE_DIALOG );
+    searchdlg_logo = gtk_image_new_from_icon_name( "edit-find" , GTK_ICON_SIZE_DIALOG );
     searchdlg_helptext = gtk_label_new( _("Select entries in playlist by filling one or more "
       "fields. Fields use regular expressions syntax, case-insensitive. If you don't know how "
       "regular expressions work, simply insert a literal portion of what you're searching for.") );
@@ -336,19 +314,19 @@ playlistwin_select_search(void)
 
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_title) );
          AUDDBG("title=\"%s\"\n", searchdata);
-         tuple_set_str(tuple, FIELD_TITLE, NULL, searchdata);
+         tuple_set_str(tuple, FIELD_TITLE, searchdata);
 
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_album) );
          AUDDBG("album=\"%s\"\n", searchdata);
-         tuple_set_str(tuple, FIELD_ALBUM, NULL, searchdata);
+         tuple_set_str(tuple, FIELD_ALBUM, searchdata);
 
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_performer) );
          AUDDBG("performer=\"%s\"\n", searchdata);
-         tuple_set_str(tuple, FIELD_ARTIST, NULL, searchdata);
+         tuple_set_str(tuple, FIELD_ARTIST, searchdata);
 
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_file_name) );
          AUDDBG("filename=\"%s\"\n", searchdata);
-         tuple_set_str(tuple, FIELD_FILE_NAME, NULL, searchdata);
+         tuple_set_str(tuple, FIELD_FILE_NAME, searchdata);
 
          /* check if previous selection should be cleared before searching */
          if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(searchdlg_checkbt_clearprevsel)) == TRUE )
@@ -413,7 +391,7 @@ static void playlistwin_resize (gint w, gint h)
     if (tx < PLAYLISTWIN_MIN_WIDTH)
         tx = PLAYLISTWIN_MIN_WIDTH;
 
-    if (!config.playlist_shaded)
+    if (! aud_get_bool ("skins", "playlist_shaded"))
     {
         ty = (h - PLAYLISTWIN_MIN_HEIGHT) / PLAYLISTWIN_HEIGHT_SNAP;
         ty = (ty * PLAYLISTWIN_HEIGHT_SNAP) + PLAYLISTWIN_MIN_HEIGHT;
@@ -492,11 +470,11 @@ playlistwin_press(GtkWidget * widget,
                   GdkEventButton * event,
                   gpointer callback_data)
 {
-    if (event->button == 1 && event->type == GDK_2BUTTON_PRESS && event->y < 14)
+    if (event->button == 1 && event->type == GDK_2BUTTON_PRESS &&
+     event->window == gtk_widget_get_window (widget) && event->y < 14)
         playlistwin_shade_toggle();
     else if (event->button == 3)
-        ui_popup_menu_show(UI_MENU_PLAYLIST, event->x_root, event->y_root,
-         FALSE, FALSE, 3, event->time);
+        menu_popup (UI_MENU_PLAYLIST, event->x_root, event->y_root, FALSE, FALSE, 3, event->time);
 
     return TRUE;
 }
@@ -517,21 +495,21 @@ void playlistwin_set_time (const gchar * minutes, const gchar * seconds)
 static void drag_motion (GtkWidget * widget, GdkDragContext * context, gint x,
  gint y, guint time, void * unused)
 {
-    if (! config.playlist_shaded)
+    if (! aud_get_bool ("skins", "playlist_shaded"))
         ui_skinned_playlist_hover (playlistwin_list, x - 12, y - 20);
 }
 
 static void drag_leave (GtkWidget * widget, GdkDragContext * context, guint time,
  void * unused)
 {
-    if (! config.playlist_shaded)
+    if (! aud_get_bool ("skins", "playlist_shaded"))
         ui_skinned_playlist_hover_end (playlistwin_list);
 }
 
 static void drag_drop (GtkWidget * widget, GdkDragContext * context, gint x,
  gint y, guint time, void * unused)
 {
-    if (config.playlist_shaded)
+    if (aud_get_bool ("skins", "playlist_shaded"))
         drop_position = -1;
     else
     {
@@ -550,7 +528,7 @@ static void drag_data_received (GtkWidget * widget, GdkDragContext * context,
 
 static void playlistwin_hide (void)
 {
-    playlistwin_show (0);
+    view_set_show_playlist (FALSE);
 }
 
 static void resize_press (void)
@@ -561,27 +539,29 @@ static void resize_press (void)
 
 static void resize_drag (gint x_offset, gint y_offset)
 {
+    bool_t shaded = aud_get_bool ("skins", "playlist_shaded");
+
     /* compromise between rounding and truncating; this has no real
      * justification at all other than it "looks about right". */
     playlistwin_resize (resize_base_width + x_offset + PLAYLISTWIN_WIDTH_SNAP /
      3, resize_base_height + y_offset + PLAYLISTWIN_HEIGHT_SNAP / 3);
-    window_set_size (playlistwin, config.playlist_width, config.playlist_shaded
-     ? PLAYLISTWIN_SHADED_HEIGHT : config.playlist_height);
+    window_set_size (playlistwin, config.playlist_width, shaded ?
+     PLAYLISTWIN_SHADED_HEIGHT : config.playlist_height);
 }
 
 static void button_add_cb (GtkWidget * button, GdkEventButton * event)
 {
     gint xpos, ypos;
     gtk_window_get_position ((GtkWindow *) playlistwin, & xpos, & ypos);
-    ui_popup_menu_show (UI_MENU_PLAYLIST_ADD, xpos + 12, ypos +
-     config.playlist_height - 8, FALSE, TRUE, event->button, event->time);
+    menu_popup (UI_MENU_PLAYLIST_ADD, xpos + 12, ypos + config.playlist_height -
+     8, FALSE, TRUE, event->button, event->time);
 }
 
 static void button_sub_cb (GtkWidget * button, GdkEventButton * event)
 {
     gint xpos, ypos;
     gtk_window_get_position ((GtkWindow *) playlistwin, & xpos, & ypos);
-    ui_popup_menu_show (UI_MENU_PLAYLIST_REMOVE, xpos + 40, ypos +
+    menu_popup (UI_MENU_PLAYLIST_REMOVE, xpos + 40, ypos +
      config.playlist_height - 8, FALSE, TRUE, event->button, event->time);
 }
 
@@ -589,7 +569,7 @@ static void button_sel_cb (GtkWidget * button, GdkEventButton * event)
 {
     gint xpos, ypos;
     gtk_window_get_position ((GtkWindow *) playlistwin, & xpos, & ypos);
-    ui_popup_menu_show (UI_MENU_PLAYLIST_SELECT, xpos + 68, ypos +
+    menu_popup (UI_MENU_PLAYLIST_SELECT, xpos + 68, ypos +
      config.playlist_height - 8, FALSE, TRUE, event->button, event->time);
 }
 
@@ -597,17 +577,16 @@ static void button_misc_cb (GtkWidget * button, GdkEventButton * event)
 {
     gint xpos, ypos;
     gtk_window_get_position ((GtkWindow *) playlistwin, & xpos, & ypos);
-    ui_popup_menu_show (UI_MENU_PLAYLIST_SORT, xpos + 100, ypos +
-     config.playlist_height - 8, FALSE, TRUE, event->button, event->time);
+    menu_popup (UI_MENU_PLAYLIST_SORT, xpos + 100, ypos + config.playlist_height
+     - 8, FALSE, TRUE, event->button, event->time);
 }
 
 static void button_list_cb (GtkWidget * button, GdkEventButton * event)
 {
     gint xpos, ypos;
     gtk_window_get_position ((GtkWindow *) playlistwin, & xpos, & ypos);
-    ui_popup_menu_show (UI_MENU_PLAYLIST_GENERAL, xpos + config.playlist_width -
-     12, ypos + config.playlist_height - 8, TRUE, TRUE, event->button,
-     event->time);
+    menu_popup (UI_MENU_PLAYLIST, xpos + config.playlist_width - 12, ypos +
+     config.playlist_height - 8, TRUE, TRUE, event->button, event->time);
 }
 
 static void
@@ -634,8 +613,10 @@ playlistwin_create_widgets(void)
     window_put_widget (playlistwin, FALSE, playlistwin_close, w - 11, 3);
     button_on_release (playlistwin_close, (ButtonCB) playlistwin_hide);
 
-    playlistwin_list = ui_skinned_playlist_new (w - 31, h - 58, config.playlist_font);
+    char * font = aud_get_str ("skins", "playlist_font");
+    playlistwin_list = ui_skinned_playlist_new (w - 31, h - 58, font);
     window_put_widget (playlistwin, FALSE, playlistwin_list, 12, 20);
+    str_unref (font);
 
     /* playlist list box slider */
     playlistwin_slider = ui_skinned_playlist_slider_new (playlistwin_list, h - 58);
@@ -721,7 +702,7 @@ playlistwin_create_widgets(void)
 
 static void pl_win_draw (GtkWidget * window, cairo_t * cr)
 {
-    if (config.playlist_shaded)
+    if (aud_get_bool ("skins", "playlist_shaded"))
         skin_draw_playlistwin_shaded (cr, config.playlist_width, TRUE);
     else
         skin_draw_playlistwin_frame (cr, config.playlist_width,
@@ -731,9 +712,11 @@ static void pl_win_draw (GtkWidget * window, cairo_t * cr)
 static void
 playlistwin_create_window(void)
 {
+    bool_t shaded = aud_get_bool ("skins", "playlist_shaded");
+
     playlistwin = window_new (& config.playlist_x, & config.playlist_y,
-     config.playlist_width, config.playlist_shaded ? PLAYLISTWIN_SHADED_HEIGHT :
-     config.playlist_height, FALSE, config.playlist_shaded, pl_win_draw);
+     config.playlist_width, shaded ? PLAYLISTWIN_SHADED_HEIGHT :
+     config.playlist_height, FALSE, shaded, pl_win_draw);
 
     gtk_window_set_title(GTK_WINDOW(playlistwin), _("Audacious Playlist Editor"));
 
@@ -827,7 +810,7 @@ playlistwin_create(void)
     playlistwin_create_widgets();
     window_show_all (playlistwin);
 
-    gtk_window_add_accel_group(GTK_WINDOW(playlistwin), ui_manager_get_accel_group());
+    gtk_window_add_accel_group ((GtkWindow *) playlistwin, menu_get_accel_group ());
 
     aud_playlist_select_all (active_playlist, FALSE);
 
@@ -853,30 +836,6 @@ void playlistwin_unhook (void)
     active_title = NULL;
 }
 
-static void playlistwin_real_show (gboolean show)
-{
-    if (show)
-        gtk_window_present ((GtkWindow *) playlistwin);
-    else
-        gtk_widget_hide (playlistwin);
-}
-
-void playlistwin_show (char show)
-{
-    GtkAction * a;
-
-    a = gtk_action_group_get_action (toggleaction_group_others, "show playlist editor");
-
-    if (a && gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (a)) != show)
-        gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (a), show);
-    else
-    {
-        config.playlist_visible = show;
-        button_set_active (mainwin_pl, show);
-        playlistwin_real_show (show && gtk_widget_get_visible (mainwin));
-    }
-}
-
 void action_playlist_track_info(void)
 {
     playlistwin_fileinfo();
@@ -1101,6 +1060,11 @@ void action_playlist_next (void)
         aud_playlist_set_active (0);
 }
 
+void action_playlist_rename (void)
+{
+    audgui_show_playlist_rename (active_playlist);
+}
+
 void action_playlist_delete (void)
 {
     audgui_confirm_playlist_delete (active_playlist);
@@ -1158,16 +1122,3 @@ playlistwin_select_search_kp_cb(GtkWidget *entry, GdkEventKey *event,
             return FALSE;
     }
 }
-
-void action_show_playlist_editor (GtkToggleAction * action)
-{
-    playlistwin_show (gtk_toggle_action_get_active (action));
-}
-
-void action_roll_up_playlist_editor (GtkToggleAction * action)
-{
-    config.playlist_shaded = gtk_toggle_action_get_active (action);
-    window_set_shaded (playlistwin, config.playlist_shaded);
-    window_set_size (playlistwin, config.playlist_width, config.playlist_shaded
-     ? PLAYLISTWIN_SHADED_HEIGHT : config.playlist_height);
-}
