@@ -30,9 +30,9 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "draw-compat.h"
+#include "menus.h"
 #include "skins_cfg.h"
 #include "ui_playlist.h"
-#include "ui_manager.h"
 #include "ui_skin.h"
 #include "ui_skinned_playlist.h"
 #include "ui_skinned_playlist_slider.h"
@@ -217,13 +217,12 @@ DRAW_FUNC_BEGIN (playlist_draw)
     for (gint i = data->first; i < data->first + data->rows && i <
      active_length; i ++)
     {
-        gint len = aud_playlist_entry_get_length (active_playlist, i, TRUE);
-        gchar buf[16];
+        int len = aud_playlist_entry_get_length (active_playlist, i, TRUE);
+        if (len <= 0)
+            continue;
 
-        if (len > 0)
-            snprintf (buf, sizeof buf, "%d:%02d", len / 60000, len / 1000 % 60);
-        else
-            buf[0] = 0;
+        char buf[16];
+        audgui_format_time (buf, sizeof buf, len);
 
         layout = gtk_widget_create_pango_layout (wid, buf);
         pango_layout_set_font_description (layout, data->font);
@@ -782,7 +781,7 @@ static gboolean playlist_button_press (GtkWidget * list, GdkEventButton * event)
                     select_single (data, FALSE, position);
             }
 
-            ui_popup_menu_show ((position == -1) ? UI_MENU_PLAYLIST :
+            menu_popup ((position == -1) ? UI_MENU_PLAYLIST :
              UI_MENU_PLAYLIST_CONTEXT, event->x_root, event->y_root, FALSE,
              FALSE, 3, event->time);
             break;
