@@ -39,17 +39,16 @@
  */
 
 #include <math.h>
-#include <pthread.h>
 #include <stdlib.h>
 
+#include <glib.h>
+
 #include <sndfile.h>
 
+#include <audacious/input.h>
 #include <audacious/plugin.h>
 #include <audacious/i18n.h>
-
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static int seek_value;
-static bool_t stop_flag;
+#include <libaudcore/audstrings.h>
 
 /* Virtual file access wrappers for libsndfile
  */
@@ -96,14 +95,14 @@ static void copy_string (SNDFILE * sf, int sf_id, Tuple * tup, int tup_id)
 {
     const char * str = sf_get_string (sf, sf_id);
     if (str)
-        tuple_set_str (tup, tup_id, NULL, str);
+        tuple_set_str (tup, tup_id, str);
 }
 
 static void copy_int (SNDFILE * sf, int sf_id, Tuple * tup, int tup_id)
 {
     const char * str = sf_get_string (sf, sf_id);
     if (str && atoi (str))
-        tuple_set_int (tup, tup_id, NULL, atoi (str));
+        tuple_set_int (tup, tup_id, atoi (str));
 }
 
 static Tuple * get_song_tuple (const char * filename, VFSFile * file)
@@ -134,7 +133,7 @@ static Tuple * get_song_tuple (const char * filename, VFSFile * file)
 
     if (sfinfo.samplerate > 0)
     {
-        tuple_set_int(ti, FIELD_LENGTH, NULL,
+        tuple_set_int(ti, FIELD_LENGTH,
         (int) ceil (1000.0 * sfinfo.frames / sfinfo.samplerate));
     }
 
@@ -288,8 +287,7 @@ static Tuple * get_song_tuple (const char * filename, VFSFile * file)
     return ti;
 }
 
-static bool_t play_start (InputPlayback * playback, const char * filename,
- VFSFile * file, int start_time, int stop_time, bool_t pause)
+static bool_t play_start (const char * filename, VFSFile * file)
 {
     if (file == NULL)
         return FALSE;
@@ -300,99 +298,35 @@ static bool_t play_start (InputPlayback * playback, const char * filename,
     if (sndfile == NULL)
         return FALSE;
 
-    if (! playback->output->open_audio (FMT_FLOAT, sfinfo.samplerate,
+    if (! aud_input_open_audio (FMT_FLOAT, sfinfo.samplerate,
      sfinfo.channels))
     {
         sf_close (sndfile);
         return FALSE;
     }
 
-    /* Fix me!  Find out bitrate from libsndfile.  The old calculation was based
-     * on the decoded data and therefore wrong for anything but floating-point
-     * files. */
-    playback->set_params (playback, 0, sfinfo.samplerate, sfinfo.channels);
-
-    seek_value = (start_time > 0) ? start_time : -1;
-    stop_flag = FALSE;
-    playback->set_pb_ready(playback);
-
     int size = sfinfo.channels * (sfinfo.samplerate / 50);
-    float * buffer = malloc (sizeof (float) * size);
+    float * buffer = g_new (float, size);
 
-    while (stop_time < 0 || playback->output->written_time () < stop_time)
+    while (! aud_input_check_stop ())
     {
-        pthread_mutex_lock (& mutex);
-
-        if (stop_flag)
-        {
-            pthread_mutex_unlock (& mutex);
-            break;
-        }
-
+        int seek_value = aud_input_check_seek ();
         if (seek_value != -1)
-        {
-            sf_seek (sndfile, (int64_t) seek_value * sfinfo.samplerate / 1000,
-             SEEK_SET);
-            playback->output->flush (seek_value);
-            seek_value = -1;
-        }
-
-        pthread_mutex_unlock (& mutex);
+            sf_seek (sndfile, (int64_t) seek_value * sfinfo.samplerate / 1000, SEEK_SET);
 
         int samples = sf_read_float (sndfile, buffer, size);
-
         if (! samples)
             break;
 
-        playback->output->write_audio (buffer, sizeof (float) * samples);
+        aud_input_write_audio (buffer, sizeof (float) * samples);
     }
 
     sf_close (sndfile);
-    free (buffer);
-
-    pthread_mutex_lock (& mutex);
-    stop_flag = TRUE;
-    pthread_mutex_unlock (& mutex);
+    g_free (buffer);
 
     return TRUE;
 }
 
-static void play_pause (InputPlayback * p, bool_t pause)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (! stop_flag)
-        p->output->pause (pause);
-
-    pthread_mutex_unlock (& mutex);
-}
-
-static void play_stop (InputPlayback * p)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (! stop_flag)
-    {
-        stop_flag = TRUE;
-        p->output->abort_write ();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
-static void file_mseek (InputPlayback * p, int time)
-{
-    pthread_mutex_lock (& mutex);
-
-    if (! stop_flag)
-    {
-        seek_value = time;
-        p->output->abort_write();
-    }
-
-    pthread_mutex_unlock (& mutex);
-}
-
 static int
 is_our_file_from_vfs(const char *filename, VFSFile *fin)
 {
@@ -436,12 +370,9 @@ AUD_INPUT_PLUGIN
     .domain = PACKAGE,
     .about_text = plugin_about,
     .play = play_start,
-    .stop = play_stop,
-    .pause = play_pause,
     .probe_for_tuple = get_song_tuple,
     .is_our_file_from_vfs = is_our_file_from_vfs,
     .extensions = sndfile_fmts,
-    .mseek = file_mseek,
 
     /* low priority fallback (but before ffaudio) */
     .priority = 9,
