@@ -172,32 +172,32 @@ static void do_command (char * cmd)
         int length = aud_playlist_entry_get_length (playlist, pos, FALSE);
         if (length > 0)
         {
-            g_snprintf(numbuf, sizeof(numbuf), "%d", length);
+            str_itoa (length, numbuf, sizeof numbuf);
             formatter_associate(formatter, 'l', numbuf);
         }
         else
             formatter_associate(formatter, 'l', "0");
 
         playing = aud_drct_get_playing();
-        g_snprintf(numbuf, sizeof(numbuf), "%d", playing);
+        str_itoa (playing, numbuf, sizeof numbuf);
         formatter_associate(formatter, 'p', numbuf);
 
         if (playing)
         {
             int brate, srate, chans;
             aud_drct_get_info (& brate, & srate, & chans);
-            snprintf (numbuf, sizeof numbuf, "%d", brate);
+            str_itoa (brate, numbuf, sizeof numbuf);
             formatter_associate (formatter, 'r', numbuf);
-            snprintf (numbuf, sizeof numbuf, "%d", srate);
+            str_itoa (srate, numbuf, sizeof numbuf);
             formatter_associate (formatter, 'F', numbuf);
-            snprintf (numbuf, sizeof numbuf, "%d", chans);
+            str_itoa (chans, numbuf, sizeof numbuf);
             formatter_associate (formatter, 'c', numbuf);
         }
 
         Tuple * tuple = aud_playlist_entry_get_tuple
             (aud_playlist_get_active (), pos, 0);
 
-        char * artist = tuple ? tuple_get_str (tuple, FIELD_ARTIST, NULL) : NULL;
+        char * artist = tuple ? tuple_get_str (tuple, FIELD_ARTIST) : NULL;
         if (artist)
         {
             formatter_associate(formatter, 'a', artist);
@@ -206,7 +206,7 @@ static void do_command (char * cmd)
         else
             formatter_associate(formatter, 'a', "");
 
-        char * album = tuple ? tuple_get_str (tuple, FIELD_ALBUM, NULL) : NULL;
+        char * album = tuple ? tuple_get_str (tuple, FIELD_ALBUM) : NULL;
         if (album)
         {
             formatter_associate(formatter, 'b', album);
@@ -215,7 +215,7 @@ static void do_command (char * cmd)
         else
             formatter_associate(formatter, 'b', "");
 
-        char * title = tuple ? tuple_get_str (tuple, FIELD_TITLE, NULL) : NULL;
+        char * title = tuple ? tuple_get_str (tuple, FIELD_TITLE) : NULL;
         if (title)
         {
             formatter_associate(formatter, 'T', title);
@@ -241,10 +241,10 @@ static void do_command (char * cmd)
 
 static void read_config(void)
 {
-    cmd_line = aud_get_string("song_change", "cmd_line");
-    cmd_line_after = aud_get_string("song_change", "cmd_line_after");
-    cmd_line_end = aud_get_string("song_change", "cmd_line_end");
-    cmd_line_ttc = aud_get_string("song_change", "cmd_line_ttc");
+    cmd_line = aud_get_str("song_change", "cmd_line");
+    cmd_line_after = aud_get_str("song_change", "cmd_line_after");
+    cmd_line_end = aud_get_str("song_change", "cmd_line_end");
+    cmd_line_ttc = aud_get_str("song_change", "cmd_line_ttc");
 }
 
 static void cleanup(void)
@@ -262,43 +262,35 @@ static void cleanup(void)
         ttc_prevs = NULL;
     }
 
-    g_free(cmd_line);
-    g_free(cmd_line_after);
-    g_free(cmd_line_end);
-    g_free(cmd_line_ttc);
+    str_unref(cmd_line);
+    str_unref(cmd_line_after);
+    str_unref(cmd_line_end);
+    str_unref(cmd_line_ttc);
+
     cmd_line = NULL;
     cmd_line_after = NULL;
     cmd_line_end = NULL;
     cmd_line_ttc = NULL;
+
     signal(SIGCHLD, SIG_DFL);
 }
 
 static void save_and_close(gchar * cmd, gchar * cmd_after, gchar * cmd_end, gchar * cmd_ttc)
 {
-    aud_set_string("song_change", "cmd_line", cmd);
-    aud_set_string("song_change", "cmd_line_after", cmd_after);
-    aud_set_string("song_change", "cmd_line_end", cmd_end);
-    aud_set_string("song_change", "cmd_line_ttc", cmd_ttc);
+    aud_set_str("song_change", "cmd_line", cmd);
+    aud_set_str("song_change", "cmd_line_after", cmd_after);
+    aud_set_str("song_change", "cmd_line_end", cmd_end);
+    aud_set_str("song_change", "cmd_line_ttc", cmd_ttc);
 
-    if (cmd_line != NULL)
-        g_free(cmd_line);
+    str_unref(cmd_line);
+    str_unref(cmd_line_after);
+    str_unref(cmd_line_end);
+    str_unref(cmd_line_ttc);
 
-    cmd_line = g_strdup(cmd);
-
-    if (cmd_line_after != NULL)
-        g_free(cmd_line_after);
-
-    cmd_line_after = g_strdup(cmd_after);
-
-    if (cmd_line_end != NULL)
-        g_free(cmd_line_end);
-
-    cmd_line_end = g_strdup(cmd_end);
-
-    if (cmd_line_ttc != NULL)
-        g_free(cmd_line_ttc);
-
-    cmd_line_ttc = g_strdup(cmd_ttc);
+    cmd_line = str_get(cmd);
+    cmd_line_after = str_get(cmd_after);
+    cmd_line_end = str_get(cmd_end);
+    cmd_line_ttc = str_get(cmd_ttc);
 }
 
 static int check_command(char *command)
@@ -481,7 +473,7 @@ static void * custom_warning (void)
 
     bbox_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
 
-    cmd_warn_img = gtk_image_new_from_stock("gtk-dialog-warning", GTK_ICON_SIZE_MENU);
+    cmd_warn_img = gtk_image_new_from_icon_name("dialog-warning", GTK_ICON_SIZE_MENU);
     gtk_box_pack_start(GTK_BOX(bbox_hbox), cmd_warn_img, FALSE, FALSE, 0);
 
     temp = g_strdup_printf(_("<span size='small'>Parameters passed to the shell should be encapsulated in quotes. Doing otherwise is a security risk.</span>"));
@@ -495,7 +487,7 @@ static void * custom_warning (void)
 }
 
 static const PreferencesWidget settings[] = {
-    {WIDGET_BOX, N_("Commands"), .data = {.box = {elements, G_N_ELEMENTS (elements), .frame = TRUE}}},
+    {WIDGET_BOX, N_("Commands"), .data = {.box = {elements, ARRAY_LEN (elements), .frame = TRUE}}},
     {WIDGET_CUSTOM, .data = {.populate = custom_warning}}};
 
 static void configure_init(void)
@@ -525,7 +517,7 @@ static void configure_cleanup(void)
 
 static const PluginPreferences preferences = {
     .widgets = settings,
-    .n_widgets = G_N_ELEMENTS (settings),
+    .n_widgets = ARRAY_LEN (settings),
     .init = configure_init,
     .cleanup = configure_cleanup,
 };
