@@ -18,7 +18,6 @@
  *
  */
 
-#include <glib.h>
 #include <ogg/ogg.h>
 #include <vorbis/codec.h>
 #include <vorbis/vorbisfile.h>
@@ -26,9 +25,11 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
-
 #include <unistd.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <audacious/debug.h>
 #include <audacious/plugin.h>
 #include <audacious/i18n.h>
@@ -79,7 +80,7 @@ static void dictionary_to_vorbis_comment (vorbis_comment * vc, GHashTable * dict
 static void insert_str_tuple_field_to_dictionary (const Tuple * tuple, int
  fieldn, GHashTable * dict, const char * key)
 {
-    char * val = tuple_get_str (tuple, fieldn, NULL);
+    char * val = tuple_get_str (tuple, fieldn);
 
     if (val && val[0])
         g_hash_table_insert (dict, str_get (key), str_ref (val));
@@ -92,18 +93,15 @@ static void insert_str_tuple_field_to_dictionary (const Tuple * tuple, int
 static void insert_int_tuple_field_to_dictionary (const Tuple * tuple, int
  fieldn, GHashTable * dict, const char * key)
 {
-    int val = tuple_get_int (tuple, fieldn, NULL);
+    int val = tuple_get_int (tuple, fieldn);
 
     if (val > 0)
-    {
-        SPRINTF (buf, "%d", val);
-        g_hash_table_insert (dict, str_get (key), str_get (buf));
-    }
+        g_hash_table_insert (dict, str_get (key), int_to_str (val));
     else
         g_hash_table_remove (dict, key);
 }
 
-gboolean vorbis_update_song_tuple (const Tuple * tuple, VFSFile * fd)
+gboolean vorbis_update_song_tuple (const char * filename, VFSFile * fd, const Tuple * tuple)
 {
 
     vcedit_state *state;
@@ -141,18 +139,6 @@ gboolean vorbis_update_song_tuple (const Tuple * tuple, VFSFile * fd)
     return ret;
 }
 
-gchar * filename_to_uri (const gchar * filename)
-{
-    gchar * utf8 = g_locale_to_utf8 (filename, -1, NULL, NULL, NULL);
-    if (! utf8)
-        utf8 = g_strdup (filename);
-
-    gchar * uri = g_filename_to_uri (utf8, NULL, NULL);
-
-    g_free (utf8);
-    return uri;
-}
-
 #define COPY_BUF 65536
 
 gboolean copy_vfs (VFSFile * in, VFSFile * out)
@@ -204,7 +190,7 @@ gboolean write_and_pivot_files (vcedit_state * state)
     VFSFile * temp_vfs = vfs_fopen (temp_uri, "r+");
     g_return_val_if_fail (temp_vfs, FALSE);
 
-    g_free (temp_uri);
+    str_unref (temp_uri);
 
     if (vcedit_write (state, temp_vfs) < 0)
     {
@@ -225,7 +211,7 @@ gboolean write_and_pivot_files (vcedit_state * state)
 
     vfs_fclose (temp_vfs);
 
-    if (unlink (temp) < 0)
+    if (g_unlink (temp) < 0)
         fprintf (stderr, "Failed to delete temp file: %s.\n", temp);
 
     g_free (temp);
