@@ -29,7 +29,6 @@
 #define DEBUG*/
 
 #include <glib.h>
-#include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
@@ -39,6 +38,7 @@
 #include <vorbis/vorbisfile.h>
 
 #include <audacious/i18n.h>
+#include <audacious/input.h>
 #include <audacious/misc.h>
 #include <audacious/plugin.h>
 #include <libaudcore/audstrings.h>
@@ -79,80 +79,56 @@ ov_callbacks vorbis_callbacks_stream = {
     NULL
 };
 
-static gint seek_value;
-static gboolean stop_flag = FALSE;
-static pthread_mutex_t seek_mutex = PTHREAD_MUTEX_INITIALIZER;
-
-static gint
-vorbis_check_fd(const gchar *filename, VFSFile *stream)
+static bool_t vorbis_check_fd (const char * filename, VFSFile * file)
 {
-    OggVorbis_File vfile;
-    gint result;
+    ogg_sync_state oy = {0};
+    ogg_stream_state os = {0};
+    ogg_page og = {0};
+    ogg_packet op = {0};
 
-    /*
-     * The open function performs full stream detection and machine
-     * initialization.  If it returns zero, the stream *is* Vorbis and
-     * we're fully ready to decode.
-     */
+    bool_t result = FALSE;
 
-    memset(&vfile, 0, sizeof(vfile));
+    ogg_sync_init (& oy);
 
-    result = ov_test_callbacks (stream, & vfile, NULL, 0, vfs_is_streaming
-     (stream) ? vorbis_callbacks_stream : vorbis_callbacks);
+    while (1)
+    {
+        int64_t bytes = ogg_sync_pageseek (& oy, & og);
 
-    switch (result) {
-    case OV_EREAD:
-#ifdef DEBUG
-        g_message("** vorbis.c: Media read error: %s", filename);
-#endif
-        return FALSE;
-        break;
-    case OV_ENOTVORBIS:
-#ifdef DEBUG
-        g_message("** vorbis.c: Not Vorbis data: %s", filename);
-#endif
-        return FALSE;
-        break;
-    case OV_EVERSION:
-#ifdef DEBUG
-        g_message("** vorbis.c: Version mismatch: %s", filename);
-#endif
-        return FALSE;
-        break;
-    case OV_EBADHEADER:
-#ifdef DEBUG
-        g_message("** vorbis.c: Invalid Vorbis bistream header: %s",
-                  filename);
-#endif
-        return FALSE;
-        break;
-    case OV_EFAULT:
-#ifdef DEBUG
-        g_message("** vorbis.c: Internal logic fault while reading %s",
-                  filename);
-#endif
-        return FALSE;
-        break;
-    case 0:
-        break;
-    default:
-        break;
+        if (bytes < 0) /* skipped some bytes */
+            continue;
+        if (bytes > 0) /* got a page */
+            break;
+
+        void * buffer = ogg_sync_buffer (& oy, 2048);
+        bytes = vfs_fread (buffer, 1, 2048, file);
+
+        if (bytes <= 0)
+            goto end;
+
+        ogg_sync_wrote (& oy, bytes);
     }
 
-    ov_clear(&vfile);
-    return TRUE;
+    if (! ogg_page_bos (& og))
+        goto end;
+
+    ogg_stream_init (& os, ogg_page_serialno (& og));
+    ogg_stream_pagein (& os, & og);
+
+    if (ogg_stream_packetout (& os, & op) > 0 && vorbis_synthesis_idheader (& op))
+        result = TRUE;
+
+end:
+    ogg_sync_clear (& oy);
+    ogg_stream_clear (& os);
+
+    return result;
 }
 
 static void
-set_tuple_str(Tuple *tuple, const gint nfield, const gchar *field,
+set_tuple_str(Tuple *tuple, const gint nfield,
     vorbis_comment *comment, gchar *key)
 {
-    gchar *str = vorbis_comment_query(comment, key, 0);
-    if (str != NULL) {
-        gchar *tmp = str_to_utf8(str);
-        tuple_set_str(tuple, nfield, field, tmp);
-        g_free(tmp);
-    }
+    tuple_set_str (tuple, nfield, vorbis_comment_query (comment, key, 0));
 }
 
 static Tuple *
@@ -168,28 +144,28 @@ get_tuple_for_vorbisfile(OggVorbis_File * vorbisfile, const gchar *filename)
      (vorbisfile, -1) * 1000;
 
     /* associate with tuple */
-    tuple_set_int(tuple, FIELD_LENGTH, NULL, length);
+    tuple_set_int(tuple, FIELD_LENGTH, length);
 
     if ((comment = ov_comment(vorbisfile, -1)) != NULL) {
         gchar *tmps;
-        set_tuple_str(tuple, FIELD_TITLE, NULL, comment, "title");
-        set_tuple_str(tuple, FIELD_ARTIST, NULL, comment, "artist");
-        set_tuple_str(tuple, FIELD_ALBUM, NULL, comment, "album");
-        set_tuple_str(tuple, FIELD_GENRE, NULL, comment, "genre");
-        set_tuple_str(tuple, FIELD_COMMENT, NULL, comment, "comment");
+        set_tuple_str(tuple, FIELD_TITLE, comment, "title");
+        set_tuple_str(tuple, FIELD_ARTIST, comment, "artist");
+        set_tuple_str(tuple, FIELD_ALBUM, comment, "album");
+        set_tuple_str(tuple, FIELD_GENRE, comment, "genre");
+        set_tuple_str(tuple, FIELD_COMMENT, comment, "comment");
 
         if ((tmps = vorbis_comment_query(comment, "tracknumber", 0)) != NULL)
-            tuple_set_int(tuple, FIELD_TRACK_NUMBER, NULL, atoi(tmps));
+            tuple_set_int(tuple, FIELD_TRACK_NUMBER, atoi(tmps));
 
         if ((tmps = vorbis_comment_query (comment, "date", 0)) != NULL)
-            tuple_set_int (tuple, FIELD_YEAR, NULL, atoi (tmps));
+            tuple_set_int (tuple, FIELD_YEAR, atoi (tmps));
     }
 
     vorbis_info * info = ov_info (vorbisfile, -1);
     tuple_set_format (tuple, "Ogg Vorbis", info->channels, info->rate,
      info->bitrate_nominal / 1000);
 
-    tuple_set_str(tuple, FIELD_MIMETYPE, NULL, "application/ogg");
+    tuple_set_str(tuple, FIELD_MIMETYPE, "application/ogg");
 
     return tuple;
 }
@@ -285,8 +261,7 @@ vorbis_interleave_buffer(float **pcm, int samples, int ch, float *pcmout)
 #define PCM_FRAMES 1024
 #define PCM_BUFSIZE (PCM_FRAMES * 2)
 
-static gboolean vorbis_play (InputPlayback * playback, const gchar * filename,
- VFSFile * file, gint start_time, gint stop_time, gboolean pause)
+static gboolean vorbis_play (const gchar * filename, VFSFile * file)
 {
     if (file == NULL)
         return FALSE;
@@ -299,9 +274,6 @@ static gboolean vorbis_play (InputPlayback * playback, const gchar * filename,
     gint bytes, channels, samplerate, br;
     gchar * title = NULL;
 
-    seek_value = (start_time > 0) ? start_time : -1;
-    stop_flag = FALSE;
-
     memset(&vf, 0, sizeof(vf));
 
     gboolean error = FALSE;
@@ -322,22 +294,15 @@ static gboolean vorbis_play (InputPlayback * playback, const gchar * filename,
     channels = vi->channels;
     samplerate = vi->rate;
 
-    playback->set_params (playback, br, samplerate, channels);
+    aud_input_set_bitrate (br);
 
-    if (!playback->output->open_audio(FMT_FLOAT, samplerate, channels)) {
+    if (!aud_input_open_audio(FMT_FLOAT, samplerate, channels)) {
         error = TRUE;
         goto play_cleanup;
     }
 
-    playback->output->flush (start_time);
-
-    if (pause)
-        playback->output->pause (TRUE);
-
     vorbis_update_replaygain(&vf, &rg_info);
-    playback->output->set_replaygain_info (& rg_info);
-
-    playback->set_pb_ready(playback);
+    aud_input_set_gain (& rg_info);
 
     /*
      * Note that chaining changes things here; A vorbis file may
@@ -346,45 +311,24 @@ static gboolean vorbis_play (InputPlayback * playback, const gchar * filename,
      * using the ov_ interface.
      */
 
-    while (1)
+    while (! aud_input_check_stop ())
     {
-        if (stop_time >= 0 && playback->output->written_time () >= stop_time)
-            goto DRAIN;
+        int seek_value = aud_input_check_seek();
 
-        pthread_mutex_lock (& seek_mutex);
-
-        if (stop_flag)
+        if (seek_value >= 0 && ov_time_seek (& vf, (double) seek_value / 1000) < 0)
         {
-            pthread_mutex_unlock (& seek_mutex);
+            fprintf (stderr, "vorbis: seek failed\n");
+            error = TRUE;
             break;
         }
 
-        if (seek_value >= 0)
-        {
-            if (ov_time_seek (& vf, (double) seek_value / 1000) < 0)
-            {
-                fprintf (stderr, "vorbis: seek failed\n");
-                error = TRUE;
-                pthread_mutex_unlock (& seek_mutex);
-                break;
-            }
-
-            playback->output->flush (seek_value);
-            seek_value = -1;
-        }
-
-        pthread_mutex_unlock (& seek_mutex);
-
         gint current_section = last_section;
         bytes = ov_read_float(&vf, &pcm, PCM_FRAMES, &current_section);
         if (bytes == OV_HOLE)
             continue;
 
         if (bytes <= 0)
-        {
-DRAIN:
             break;
-        }
 
         bytes = vorbis_interleave_buffer (pcm, bytes, channels, pcmout);
 
@@ -398,7 +342,7 @@ DRAIN:
                 g_free (title);
                 title = g_strdup (new_title);
 
-                playback->set_tuple (playback, get_tuple_for_vorbisfile (& vf,
+                aud_input_set_tuple (get_tuple_for_vorbisfile (& vf,
                  filename));
             }
         }
@@ -420,32 +364,27 @@ DRAIN:
                 samplerate = vi->rate;
                 channels = vi->channels;
 
-                if (!playback->output->open_audio(FMT_FLOAT, vi->rate, vi->channels)) {
+                if (!aud_input_open_audio(FMT_FLOAT, vi->rate, vi->channels)) {
                     error = TRUE;
                     goto stop_processing;
                 }
 
-                playback->output->flush(ov_time_tell(&vf) * 1000);
                 vorbis_update_replaygain(&vf, &rg_info);
-                playback->output->set_replaygain_info (& rg_info); /* audio reopened */
+                aud_input_set_gain (& rg_info); /* audio reopened */
             }
         }
 
-        playback->output->write_audio (pcmout, bytes);
+        aud_input_write_audio (pcmout, bytes);
 
 stop_processing:
 
         if (current_section != last_section)
         {
-            playback->set_params (playback, br, samplerate, channels);
+            aud_input_set_bitrate (br);
             last_section = current_section;
         }
     } /* main loop */
 
-    pthread_mutex_lock (& seek_mutex);
-    stop_flag = TRUE;
-    pthread_mutex_unlock (& seek_mutex);
-
 play_cleanup:
 
     ov_clear(&vf);
@@ -453,42 +392,6 @@ play_cleanup:
     return ! error;
 }
 
-static void vorbis_stop (InputPlayback * playback)
-{
-    pthread_mutex_lock (& seek_mutex);
-
-    if (! stop_flag)
-    {
-        stop_flag = TRUE;
-        playback->output->abort_write ();
-    }
-
-    pthread_mutex_unlock (& seek_mutex);
-}
-
-static void vorbis_pause (InputPlayback * playback, gboolean pause)
-{
-    pthread_mutex_lock (& seek_mutex);
-
-    if (! stop_flag)
-        playback->output->pause (pause);
-
-    pthread_mutex_unlock (& seek_mutex);
-}
-
-static void vorbis_mseek (InputPlayback * playback, gint time)
-{
-    pthread_mutex_lock (& seek_mutex);
-
-    if (! stop_flag)
-    {
-        seek_value = time;
-        playback->output->abort_write ();
-    }
-
-    pthread_mutex_unlock (& seek_mutex);
-}
-
 static Tuple * get_song_tuple (const gchar * filename, VFSFile * file)
 {
     OggVorbis_File vfile;          /* avoid thread interaction */
@@ -521,37 +424,56 @@ static gboolean get_song_image (const gchar * filename, VFSFile * file,
     if (! comment)
         goto ERR;
 
-    const gchar * s = vorbis_comment_query (comment, "METADATA_BLOCK_PICTURE", 0);
-    if (! s)
-        goto ERR;
+    const gchar * s;
 
-    gsize length2;
-    void * data2 = g_base64_decode (s, & length2);
-    if (! data2 || length2 < 8)
-        goto PARSE_ERR;
+    if ((s = vorbis_comment_query (comment, "METADATA_BLOCK_PICTURE", 0)))
+    {
+        gsize length2;
+        void * data2 = g_base64_decode (s, & length2);
+        if (! data2 || length2 < 8)
+            goto PARSE_ERR;
 
-    gint mime_length = GUINT32_FROM_BE (* (guint32 *) (data2 + 4));
-    if (length2 < 8 + mime_length + 4)
-        goto PARSE_ERR;
+        gint mime_length = GUINT32_FROM_BE (* (guint32 *) (data2 + 4));
+        if (length2 < 8 + mime_length + 4)
+            goto PARSE_ERR;
 
-    gint desc_length = GUINT32_FROM_BE (* (guint32 *) (data2 + 8 + mime_length));
-    if (length2 < 8 + mime_length + 4 + desc_length + 20)
-        goto PARSE_ERR;
+        gint desc_length = GUINT32_FROM_BE (* (guint32 *) (data2 + 8 + mime_length));
+        if (length2 < 8 + mime_length + 4 + desc_length + 20)
+            goto PARSE_ERR;
 
-    * size = GUINT32_FROM_BE (* (guint32 *) (data2 + 8 + mime_length + 4 + desc_length + 16));
-    if (length2 < 8 + mime_length + 4 + desc_length + 20 + * size)
-        goto PARSE_ERR;
+        * size = GUINT32_FROM_BE (* (guint32 *) (data2 + 8 + mime_length + 4 + desc_length + 16));
+        if (length2 < 8 + mime_length + 4 + desc_length + 20 + * size)
+            goto PARSE_ERR;
 
-    * data = g_malloc (* size);
-    memcpy (* data, (char *) data2 + 8 + mime_length + 4 + desc_length + 20, * size);
+        * data = g_memdup ((char *) data2 + 8 + mime_length + 4 + desc_length + 20, * size);
 
-    g_free (data2);
-    ov_clear (& vfile);
-    return TRUE;
+        g_free (data2);
+        ov_clear (& vfile);
+        return TRUE;
 
-PARSE_ERR:
-    fprintf (stderr, "vorbis: Error parsing METADATA_BLOCK_PICTURE in %s.\n", filename);
-    g_free (data2);
+    PARSE_ERR:
+        fprintf (stderr, "vorbis: Error parsing METADATA_BLOCK_PICTURE in %s.\n", filename);
+        g_free (data2);
+    }
+
+    if ((s = vorbis_comment_query (comment, "COVERART", 0)))
+    {
+        gsize length2;
+        void * data2 = g_base64_decode (s, & length2);
+
+        if (! data2 || ! length2)
+        {
+            fprintf (stderr, "vorbis: Error parsing COVERART in %s.\n", filename);
+            g_free (data2);
+            goto ERR;
+        }
+
+        * data = data2;
+        * size = length2;
+
+        ov_clear (& vfile);
+        return TRUE;
+    }
 
 ERR:
     ov_clear (& vfile);
@@ -583,9 +505,6 @@ AUD_INPUT_PLUGIN
     .domain = PACKAGE,
     .about_text = vorbis_about,
     .play = vorbis_play,
-    .stop = vorbis_stop,
-    .pause = vorbis_pause,
-    .mseek = vorbis_mseek,
     .probe_for_tuple = get_song_tuple,
     .get_song_image = get_song_image,
     .update_song_tuple = vorbis_update_song_tuple,
