@@ -1,15 +1,12 @@
 #ifndef VTX_H
 #define VTX_H
 
-#include <config.h>
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 
 #include <audacious/plugin.h>
 
-void vtx_about(void);
 void vtx_file_info(const char *filename);
 
 #endif
