@@ -24,24 +24,20 @@
 	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <libgen.h>
-#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <audacious/i18n.h>
+#include <audacious/input.h>
 #include <audacious/misc.h>
 #include <audacious/plugin.h>
+#include <libaudcore/audstrings.h>
 
 #include "ao.h"
 #include "corlett.h"
 #include "vio2sf.h"
 
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static int seek_value = -1;
-static bool_t stop_flag = FALSE;
-
 /* xsf_get_lib: called to load secondary files */
 static const char *dirpath;
 
@@ -76,15 +72,13 @@ Tuple *xsf_tuple(const char *filename, VFSFile *fd)
 
 	t = tuple_new_from_filename(filename);
 
-	tuple_set_int(t, FIELD_LENGTH, NULL, c->inf_length ? psfTimeToMS(c->inf_length) + psfTimeToMS(c->inf_fade) : -1);
-	tuple_set_str(t, FIELD_ARTIST, NULL, c->inf_artist);
-	tuple_set_str(t, FIELD_ALBUM, NULL, c->inf_game);
-	tuple_set_str(t, -1, "game", c->inf_game);
-	tuple_set_str(t, FIELD_TITLE, NULL, c->inf_title);
-	tuple_set_str(t, FIELD_COPYRIGHT, NULL, c->inf_copy);
-	tuple_set_str(t, FIELD_QUALITY, NULL, _("sequenced"));
-	tuple_set_str(t, FIELD_CODEC, NULL, "GBA/Nintendo DS Audio");
-	tuple_set_str(t, -1, "console", "GBA/Nintendo DS");
+	tuple_set_int(t, FIELD_LENGTH, c->inf_length ? psfTimeToMS(c->inf_length) + psfTimeToMS(c->inf_fade) : -1);
+	tuple_set_str(t, FIELD_ARTIST, c->inf_artist);
+	tuple_set_str(t, FIELD_ALBUM, c->inf_game);
+	tuple_set_str(t, FIELD_TITLE, c->inf_title);
+	tuple_set_str(t, FIELD_COPYRIGHT, c->inf_copy);
+	tuple_set_str(t, FIELD_QUALITY, _("sequenced"));
+	tuple_set_str(t, FIELD_CODEC, "GBA/Nintendo DS Audio");
 
 	free(c);
 	free(buf);
@@ -117,7 +111,7 @@ static int xsf_get_length(const char *filename)
 	return length;
 }
 
-static bool_t xsf_play(InputPlayback * playback, const char * filename, VFSFile * file, int start_time, int stop_time, bool_t pause)
+static bool_t xsf_play(const char * filename, VFSFile * file)
 {
 	void *buffer;
 	int64_t size;
@@ -127,9 +121,12 @@ static bool_t xsf_play(InputPlayback * playback, const char * filename, VFSFile
 	float pos;
 	bool_t error = FALSE;
 
-	char dirbuf[strlen (filename) + 1];
-	strcpy (dirbuf, filename);
-	dirpath = dirname (dirbuf);
+	const char * slash = strrchr (filename, '/');
+	if (! slash)
+		return FALSE;
+
+	SNCOPY (dirbuf, filename, slash + 1 - filename);
+	dirpath = dirbuf;
 
 	vfs_file_get_contents (filename, & buffer, & size);
 
@@ -139,40 +136,31 @@ static bool_t xsf_play(InputPlayback * playback, const char * filename, VFSFile
 		goto ERR_NO_CLOSE;
 	}
 
-	if (!playback->output->open_audio(FMT_S16_NE, 44100, 2))
+	if (!aud_input_open_audio(FMT_S16_NE, 44100, 2))
 	{
 		error = TRUE;
 		goto ERR_NO_CLOSE;
 	}
 
-	playback->set_params(playback, 44100*2*2*8, 44100, 2);
+	aud_input_set_bitrate(44100*2*2*8);
 
-	if (pause)
-		playback->output->pause (TRUE);
-
-	stop_flag = FALSE;
-	playback->set_pb_ready(playback);
-
-	while (!stop_flag)
+	while (! aud_input_check_stop ())
 	{
-		pthread_mutex_lock (& mutex);
+		int seek_value = aud_input_check_seek ();
 
 		if (seek_value >= 0)
 		{
-			if (seek_value > playback->output->written_time ())
+			if (seek_value > aud_input_written_time ())
 			{
-				pos = playback->output->written_time ();
+				pos = aud_input_written_time ();
 
 				while (pos < seek_value)
 				{
 					xsf_gen(samples, seglen);
 					pos += 16.666;
 				}
-
-				playback->output->flush(seek_value);
-				seek_value = -1;
 			}
-			else if (seek_value < playback->output->written_time ())
+			else if (seek_value < aud_input_written_time ())
 			{
 				xsf_term();
 
@@ -184,9 +172,6 @@ static bool_t xsf_play(InputPlayback * playback, const char * filename, VFSFile
 						xsf_gen(samples, seglen);
 						pos += 16.666; /* each segment is 16.666ms */
 					}
-
-					playback->output->flush(seek_value);
-					seek_value = -1;
 				}
 			   	else
 				{
@@ -196,22 +181,16 @@ static bool_t xsf_play(InputPlayback * playback, const char * filename, VFSFile
 			}
 		}
 
-		pthread_mutex_unlock (& mutex);
-
 		xsf_gen(samples, seglen);
-		playback->output->write_audio((uint8_t *)samples, seglen * 4);
+		aud_input_write_audio((uint8_t *)samples, seglen * 4);
 
-		if (playback->output->written_time() >= length)
+		if (aud_input_written_time() >= length)
 			goto CLEANUP;
 	}
 
 CLEANUP:
 	xsf_term();
 
-	pthread_mutex_lock (& mutex);
-	stop_flag = TRUE;
-	pthread_mutex_unlock (& mutex);
-
 ERR_NO_CLOSE:
 	dirpath = NULL;
 	free(buffer);
@@ -219,29 +198,6 @@ ERR_NO_CLOSE:
 	return !error;
 }
 
-void xsf_stop(InputPlayback *playback)
-{
-	pthread_mutex_lock (& mutex);
-
-	if (!stop_flag)
-	{
-		stop_flag = TRUE;
-		playback->output->abort_write();
-	}
-
-	pthread_mutex_unlock (& mutex);
-}
-
-void xsf_pause(InputPlayback *playback, bool_t pause)
-{
-	pthread_mutex_lock (& mutex);
-
-	if (!stop_flag)
-		playback->output->pause(pause);
-
-	pthread_mutex_unlock (& mutex);
-}
-
 int xsf_is_our_fd(const char *filename, VFSFile *file)
 {
 	char magic[4];
@@ -254,19 +210,6 @@ int xsf_is_our_fd(const char *filename, VFSFile *file)
 	return 0;
 }
 
-void xsf_seek(InputPlayback *playback, int time)
-{
-	pthread_mutex_lock (& mutex);
-
-	if (!stop_flag)
-	{
-		seek_value = time;
-		playback->output->abort_write();
-	}
-
-	pthread_mutex_unlock (& mutex);
-}
-
 static const char *xsf_fmts[] = { "2sf", "mini2sf", NULL };
 
 AUD_INPUT_PLUGIN
@@ -274,9 +217,6 @@ AUD_INPUT_PLUGIN
 	.name = N_("2SF Decoder"),
 	.domain = PACKAGE,
 	.play = xsf_play,
-	.stop = xsf_stop,
-	.pause = xsf_pause,
-	.mseek = xsf_seek,
 	.probe_for_tuple = xsf_tuple,
 	.is_our_file_from_vfs = xsf_is_our_fd,
 	.extensions = xsf_fmts,
